@@ -26,12 +26,12 @@
 #include <sqlite3.h>
 #include <cstring>
 
-#include "common/syncjournaldb.h"
-#include "version.h"
-#include "filesystembase.h"
 #include "common/asserts.h"
 #include "common/checksums.h"
 #include "common/preparedsqlquerymanager.h"
+#include "common/syncjournaldb.h"
+#include "common/version.h"
+#include "filesystembase.h"
 
 #include "common/c_jhash.h"
 
@@ -250,6 +250,10 @@ bool SyncJournalDb::sqlFail(const QString &log, const SqlQuery &query)
 
 bool SyncJournalDb::checkConnect()
 {
+    if (_closed) {
+        qCWarning(lcDb) << Q_FUNC_INFO << "after the db was closed";
+        return false;
+    }
     if (autotestFailCounter >= 0) {
         if (!autotestFailCounter--) {
             qCInfo(lcDb) << "Error Simulated";
@@ -258,13 +262,6 @@ bool SyncJournalDb::checkConnect()
     }
 
     if (_db.isOpen()) {
-        // Unfortunately the sqlite isOpen check can return true even when the underlying storage
-        // has become unavailable - and then some operations may cause crashes. See #6049
-        if (!QFile::exists(_dbFile)) {
-            qCWarning(lcDb) << "Database open, but file" << _dbFile << "does not exist";
-            close();
-            return false;
-        }
         return true;
     }
 
@@ -515,10 +512,10 @@ bool SyncJournalDb::checkConnect()
         forceRemoteDiscovery = true;
 
         createQuery.prepare("INSERT INTO version VALUES (?1, ?2, ?3, ?4);");
-        createQuery.bindValue(1, MIRALL_VERSION_MAJOR);
-        createQuery.bindValue(2, MIRALL_VERSION_MINOR);
-        createQuery.bindValue(3, MIRALL_VERSION_PATCH);
-        createQuery.bindValue(4, MIRALL_VERSION_BUILD);
+        const auto segments = OCC::Version::versionWithBuildNumber().segments();
+        for (int i = 0; i < segments.size(); ++i) {
+            createQuery.bindValue(i + 1, segments[i]);
+        }
         if (!createQuery.exec()) {
             return sqlFail(QStringLiteral("Update version"), createQuery);
         }
@@ -543,13 +540,13 @@ bool SyncJournalDb::checkConnect()
         }
 
         // Not comparing the BUILD id here, correct?
-        if (!(major == MIRALL_VERSION_MAJOR && minor == MIRALL_VERSION_MINOR && patch == MIRALL_VERSION_PATCH)) {
+        if (QVersionNumber(major, minor, patch) != OCC::Version::version()) {
             createQuery.prepare("UPDATE version SET major=?1, minor=?2, patch =?3, custom=?4 "
                                 "WHERE major=?5 AND minor=?6 AND patch=?7;");
-            createQuery.bindValue(1, MIRALL_VERSION_MAJOR);
-            createQuery.bindValue(2, MIRALL_VERSION_MINOR);
-            createQuery.bindValue(3, MIRALL_VERSION_PATCH);
-            createQuery.bindValue(4, MIRALL_VERSION_BUILD);
+            const auto segments = OCC::Version::versionWithBuildNumber().segments();
+            for (int i = 0; i < segments.size(); ++i) {
+                createQuery.bindValue(i + 1, segments[i]);
+            }
             createQuery.bindValue(5, major);
             createQuery.bindValue(6, minor);
             createQuery.bindValue(7, patch);
@@ -625,8 +622,14 @@ void SyncJournalDb::close()
     _db.close();
     clearEtagStorageFilter();
     _metadataTableIsEmpty = false;
+    _closed = true;
 }
 
+void SyncJournalDb::allowReopen()
+{
+    Q_ASSERT(_closed);
+    _closed = false;
+}
 
 bool SyncJournalDb::updateDatabaseStructure()
 {
@@ -890,9 +893,9 @@ Result<void, QString> SyncJournalDb::setFileRecord(const SyncJournalFileRecord &
         if (fileId.isEmpty())
             fileId = "";
         QByteArray remotePerm = record._remotePerm.toDbValue();
-        QByteArray checksumType, checksum;
-        parseChecksumHeader(record._checksumHeader, &checksumType, &checksum);
-        int contentChecksumTypeId = mapChecksumType(checksumType);
+
+        const auto checksumHeader = ChecksumHeader::parseChecksumHeader(record._checksumHeader);
+        int contentChecksumTypeId = mapChecksumType(checksumHeader.type());
         const auto query = _queryManager.get(PreparedSqlQueryManager::SetFileRecordQuery, QByteArrayLiteral("INSERT OR REPLACE INTO metadata "
                                                                                                             "(phash, pathlen, path, inode, uid, gid, mode, modtime, type, md5, fileid, remotePerm, filesize, ignoredChildrenRemote, contentChecksum, contentChecksumTypeId) "
                                                                                                             "VALUES (?1 , ?2, ?3 , ?4 , ?5 , ?6 , ?7,  ?8 , ?9 , ?10, ?11, ?12, ?13, ?14, ?15, ?16);"),
@@ -915,7 +918,7 @@ Result<void, QString> SyncJournalDb::setFileRecord(const SyncJournalFileRecord &
         query->bindValue(12, remotePerm);
         query->bindValue(13, record._fileSize);
         query->bindValue(14, record._serverHasIgnoredFiles ? 1 : 0);
-        query->bindValue(15, checksum);
+        query->bindValue(15, checksumHeader.checksum());
         query->bindValue(16, contentChecksumTypeId);
 
         if (!query->exec()) {
@@ -1200,7 +1203,7 @@ int SyncJournalDb::getFileRecordCount()
 
 bool SyncJournalDb::updateFileRecordChecksum(const QString &filename,
     const QByteArray &contentChecksum,
-    const QByteArray &contentChecksumType)
+    CheckSums::Algorithm contentChecksumType)
 {
     QMutexLocker locker(&_mutex);
 
@@ -1404,31 +1407,63 @@ SyncJournalDb::UploadInfo SyncJournalDb::getUploadInfo(const QString &file)
 {
     QMutexLocker locker(&_mutex);
 
+
+    if (!checkConnect()) {
+        return {};
+    }
+
+    const auto query = _queryManager.get(PreparedSqlQueryManager::GetUploadInfoQuery,
+        QByteArrayLiteral("SELECT chunk, transferid, errorcount, size, modtime, contentChecksum FROM uploadinfo WHERE path=?1"),
+        _db);
+    if (!query) {
+        return {};
+    }
+    query->bindValue(1, file);
+
+    if (!query->exec()) {
+        return {};
+    }
+
     UploadInfo res;
+    if (query->next().hasData) {
+        res._chunk = query->intValue(0);
+        res._transferid = query->intValue(1);
+        res._errorCount = query->intValue(2);
+        res._size = query->int64Value(3);
+        res._modtime = query->int64Value(4);
+        res._contentChecksum = query->baValue(5);
+        res._valid = true;
+    }
+    return res;
+}
 
-    if (checkConnect()) {
-        const auto query = _queryManager.get(PreparedSqlQueryManager::GetUploadInfoQuery, QByteArrayLiteral("SELECT chunk, transferid, errorcount, size, modtime, contentChecksum FROM "
-                                                                                                            "uploadinfo WHERE path=?1"),
-            _db);
-        if (!query) {
-            return res;
-        }
-        query->bindValue(1, file);
+std::vector<SyncJournalDb::UploadInfo> SyncJournalDb::getUploadInfos()
+{
+    QMutexLocker locker(&_mutex);
 
-        if (!query->exec()) {
-            return res;
-        }
+    const auto query = _queryManager.get(PreparedSqlQueryManager::GetAllUploadInfoQuery,
+        QByteArrayLiteral("SELECT chunk, transferid, errorcount, size, modtime, contentChecksum, path FROM uploadinfo"),
+        _db);
+    if (!query) {
+        return {};
+    }
 
-        if (query->next().hasData) {
-            bool ok = true;
-            res._chunk = query->intValue(0);
-            res._transferid = query->intValue(1);
-            res._errorCount = query->intValue(2);
-            res._size = query->int64Value(3);
-            res._modtime = query->int64Value(4);
-            res._contentChecksum = query->baValue(5);
-            res._valid = ok;
-        }
+    if (!query->exec()) {
+        return {};
+    }
+
+    std::vector<UploadInfo> res;
+    while (query->next().hasData) {
+        UploadInfo info;
+        info._chunk = query->intValue(0);
+        info._transferid = query->intValue(1);
+        info._errorCount = query->intValue(2);
+        info._size = query->int64Value(3);
+        info._modtime = query->int64Value(4);
+        info._contentChecksum = query->baValue(5);
+        info._path = query->baValue(6);
+        info._valid = true;
+        res.push_back(std::move(info));
     }
     return res;
 }
@@ -1474,20 +1509,20 @@ void SyncJournalDb::setUploadInfo(const QString &file, const SyncJournalDb::Uplo
 QVector<uint> SyncJournalDb::deleteStaleUploadInfos(const QSet<QString> &keep)
 {
     QMutexLocker locker(&_mutex);
-    QVector<uint> ids;
 
     if (!checkConnect()) {
-        return ids;
+        return {};
     }
 
     SqlQuery query(_db);
     query.prepare("SELECT path,transferid FROM uploadinfo");
 
     if (!query.exec()) {
-        return ids;
+        return {};
     }
 
     QStringList superfluousPaths;
+    QVector<uint> ids;
 
     while (query.next().hasData) {
         const QString file = query.stringValue(0);
@@ -1496,7 +1531,7 @@ QVector<uint> SyncJournalDb::deleteStaleUploadInfos(const QSet<QString> &keep)
             ids.append(query.intValue(1));
         }
     }
-
+    qCDebug(lcDb) << Q_FUNC_INFO << "Keep:" << keep << "Removing Stale:" << superfluousPaths;
     const auto deleteUploadInfoQuery = _queryManager.get(PreparedSqlQueryManager::DeleteUploadInfoQuery);
     deleteBatch(*deleteUploadInfoQuery, superfluousPaths, QStringLiteral("uploadinfo"));
     return ids;
@@ -1858,23 +1893,26 @@ QByteArray SyncJournalDb::getChecksumType(int checksumTypeId)
     return query->baValue(0);
 }
 
-int SyncJournalDb::mapChecksumType(const QByteArray &checksumType)
+int SyncJournalDb::mapChecksumType(CheckSums::Algorithm checksumType)
 {
-    if (checksumType.isEmpty()) {
+    if (checksumType == CheckSums::Algorithm::PARSE_ERROR || checksumType == CheckSums::Algorithm::NONE) {
         return 0;
     }
 
-    auto it =  _checksymTypeCache.find(checksumType);
-    if (it != _checksymTypeCache.end())
+    auto it = _checksymTypeCache.constFind(checksumType);
+    if (it != _checksymTypeCache.cend()) {
         return *it;
+    }
 
+    // the type is mapped as utf8
+    const auto typeName = CheckSums::toQString(checksumType).toUtf8();
     // Ensure the checksum type is in the db
     {
         const auto query = _queryManager.get(PreparedSqlQueryManager::InsertChecksumTypeQuery, QByteArrayLiteral("INSERT OR IGNORE INTO checksumtype (name) VALUES (?1)"), _db);
         if (!query) {
             return 0;
         }
-        query->bindValue(1, checksumType);
+        query->bindValue(1, typeName);
         if (!query->exec()) {
             return 0;
         }
@@ -1882,11 +1920,11 @@ int SyncJournalDb::mapChecksumType(const QByteArray &checksumType)
 
     // Retrieve the id
     {
-        const auto query = _queryManager.get(PreparedSqlQueryManager::GetChecksumTypeIdQuery, QByteArrayLiteral("SELECT id FROM checksumtype WHERE name=?1"), _db);
+        const auto query = _queryManager.get(PreparedSqlQueryManager::GetChecksumTypeIdQuery, QByteArrayLiteral("SELECT id FROM checksumtype WHERE UPPER(name)=?1"), _db);
         if (!query) {
             return 0;
         }
-        query->bindValue(1, checksumType);
+        query->bindValue(1, typeName);
         if (!query->exec()) {
             return 0;
         }
@@ -2100,9 +2138,10 @@ Optional<PinState> SyncJournalDb::PinStateInterface::effectiveForPath(const QByt
     auto next = query->next();
     if (!next.ok)
         return {};
-    // If the root path has no setting, assume AlwaysLocal
-    if (!next.hasData)
-        return PinState::AlwaysLocal;
+    // If the root path has no setting, assume Unspecified
+    if (!next.hasData) {
+        return PinState::Unspecified;
+    }
 
     return static_cast<PinState>(query->intValue(0));
 }
