@@ -18,26 +18,27 @@
  */
 #include "config.h"
 
-#include "common/utility.h"
+#include "common/asserts.h"
 #include "common/filesystembase.h"
-#include "version.h"
+#include "common/utility.h"
+#include "common/version.h"
 
 // Note:  This file must compile without QtGui
+#include <QCollator>
 #include <QCoreApplication>
-#include <QSettings>
-#include <QTextStream>
+#include <QDateTime>
 #include <QDir>
 #include <QFile>
-#include <QUrl>
-#include <QProcess>
 #include <QObject>
-#include <QThread>
-#include <QDateTime>
-#include <QSysInfo>
+#include <QProcess>
+#include <QRandomGenerator>
+#include <QRegularExpression>
+#include <QSettings>
 #include <QStandardPaths>
-#include <QCollator>
 #include <QSysInfo>
-
+#include <QTextStream>
+#include <QThread>
+#include <QUrl>
 
 #ifdef Q_OS_UNIX
 #include <sys/statvfs.h>
@@ -45,46 +46,15 @@
 #include <unistd.h>
 #endif
 
+#include <cstring>
 #include <math.h>
 #include <stdarg.h>
-#include <cstring>
 
-#if defined(Q_OS_WIN)
-#include "utility_win.cpp"
-#elif defined(Q_OS_MAC)
-#include "utility_mac.cpp"
-#else
-#include "utility_unix.cpp"
-#endif
+using namespace std::chrono;
 
 namespace OCC {
 
-Q_LOGGING_CATEGORY(lcUtility, "sync.utility", QtInfoMsg)
-
-bool Utility::writeRandomFile(const QString &fname, int size)
-{
-    int maxSize = 10 * 10 * 1024;
-    qsrand(QDateTime::currentMSecsSinceEpoch());
-
-    if (size == -1)
-        size = qrand() % maxSize;
-
-    QString randString;
-    for (int i = 0; i < size; i++) {
-        int r = qrand() % 128;
-        randString.append(QChar(r));
-    }
-
-    QFile file(fname);
-    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-        QTextStream out(&file);
-        out << randString;
-        // optional, as QFile destructor will already do it:
-        file.close();
-        return true;
-    }
-    return false;
-}
+Q_LOGGING_CATEGORY(lcUtility, "sync.utility")
 
 QString Utility::formatFingerprint(const QByteArray &fmhash, bool colonSeparated)
 {
@@ -104,11 +74,6 @@ QString Utility::formatFingerprint(const QByteArray &fmhash, bool colonSeparated
     return fp;
 }
 
-void Utility::setupFavLink(const QString &folder)
-{
-    setupFavLink_private(folder);
-}
-
 QString Utility::octetsToString(qint64 octets)
 {
 #define THE_FACTOR 1024
@@ -179,7 +144,7 @@ QByteArray Utility::userAgentString()
 {
     return QStringLiteral("Mozilla/5.0 (%1) mirall/%2 (%3, %4-%5 ClientArchitecture: %6 OsArchitecture: %7)")
         .arg(platform(),
-            QStringLiteral(MIRALL_VERSION_STRING),
+            OCC::Version::displayString(),
             // accessing the theme to fetch the string is rather difficult
             // since this is only needed server-side to identify clients, the app name (as of 2.9, the short name) is good enough
             qApp->applicationName(),
@@ -190,26 +155,6 @@ QByteArray Utility::userAgentString()
         .toLatin1();
 }
 
-bool Utility::hasSystemLaunchOnStartup(const QString &appName)
-{
-#if defined(Q_OS_WIN)
-    return hasSystemLaunchOnStartup_private(appName);
-#else
-    Q_UNUSED(appName)
-    return false;
-#endif
-}
-
-bool Utility::hasLaunchOnStartup(const QString &appName)
-{
-    return hasLaunchOnStartup_private(appName);
-}
-
-void Utility::setLaunchOnStartup(const QString &appName, const QString &guiName, bool enable)
-{
-    setLaunchOnStartup_private(appName, guiName, enable);
-}
-
 qint64 Utility::freeDiskSpace(const QString &path)
 {
 #if defined(Q_OS_MAC) || defined(Q_OS_FREEBSD) || defined(Q_OS_FREEBSD_KERNEL) || defined(Q_OS_NETBSD) || defined(Q_OS_OPENBSD)
@@ -371,34 +316,36 @@ QString Utility::fileNameForGuiUse(const QString &fName)
     return fName;
 }
 
-QByteArray Utility::normalizeEtag(QByteArray etag)
+QString Utility::normalizeEtag(QStringView etag)
 {
-    /* strip "XXXX-gzip" */
-    if(etag.startsWith('"') && etag.endsWith("-gzip\"")) {
-        etag.chop(6);
-        etag.remove(0, 1);
+    if (etag.isEmpty()) {
+        return {};
     }
-    /* strip trailing -gzip */
-    if(etag.endsWith("-gzip")) {
-        etag.chop(5);
+
+    const auto unQuote = [&] {
+        if (etag.startsWith(QLatin1Char('"')) && etag.endsWith(QLatin1Char('"'))) {
+            etag = etag.mid(1);
+            etag.chop(1);
+        }
+    };
+
+    // Weak E-Tags can appear when gzip compression is on, see #3946
+    if (etag.startsWith(QLatin1String("W/"))) {
+        etag = etag.mid(2);
+    }
+    /* strip normal quotes and quotes around "XXXX-gzip" */
+    unQuote();
+
+    // https://github.com/owncloud/client/issues/1195
+    const QLatin1String gzipSuffix("-gzip");
+    if (etag.endsWith(gzipSuffix)) {
+        etag.chop(gzipSuffix.size());
     }
     /* strip normal quotes */
-    if (etag.startsWith('"') && etag.endsWith('"')) {
-        etag.chop(1);
-        etag.remove(0, 1);
-    }
-    etag.squeeze();
-    return etag;
+    unQuote();
+    return etag.toString();
 }
 
-#ifndef TOKEN_AUTH_ONLY
-bool Utility::hasDarkSystray()
-{
-    return hasDarkSystray_private();
-}
-#endif
-
-
 QString Utility::platformName()
 {
     return QSysInfo::prettyProductName();
@@ -419,24 +366,22 @@ void Utility::crash()
 // restarting from the installer.
 QByteArray Utility::versionOfInstalledBinary(const QString &command)
 {
-    QByteArray re;
-    if (isLinux()) {
-        QString binary(command);
-        if (binary.isEmpty()) {
-            binary = qApp->arguments()[0];
-        }
-        QStringList params;
-        params << QStringLiteral("--version");
-        QProcess process;
-        process.start(binary, params);
-        process.waitForFinished(); // sets current thread to sleep and waits for pingProcess end
-        re = process.readAllStandardOutput();
-        int newline = re.indexOf('\n');
-        if (newline > 0) {
-            re.truncate(newline);
-        }
+    QString binary(command);
+    if (binary.isEmpty()) {
+        binary = qApp->arguments()[0];
     }
-    return re;
+    QStringList params;
+    params << QStringLiteral("--version");
+    QProcess process;
+    process.start(binary, params);
+    process.waitForFinished(); // sets current thread to sleep and waits for pingProcess end
+    QByteArray re = process.readAllStandardOutput();
+    qCDebug(lcUtility) << Q_FUNC_INFO << re;
+    int newline = re.indexOf('\n');
+    if (newline > 0) {
+        re.truncate(newline);
+    }
+    return re.trimmed();
 }
 
 QString Utility::timeAgoInWords(const QDateTime &dt, const QDateTime &from)
@@ -551,13 +496,28 @@ QUrl Utility::concatUrlPath(const QUrl &url, const QString &concatPath,
         }
         path += concatPath; // put the complete path together
     }
-
+    Q_ASSERT(!path.contains(QStringLiteral("//")));
+    Q_ASSERT(url.query().isEmpty());
     QUrl tmpUrl = url;
     tmpUrl.setPath(path);
     tmpUrl.setQuery(queryItems);
     return tmpUrl;
 }
 
+bool Utility::urlEqual(QUrl url1, QUrl url2)
+{
+    // ensure https://demo.owncloud.org/ matches https://demo.owncloud.org
+    // the empty path was the legacy formating before 2.9
+    if (url1.path().isEmpty()) {
+        url1.setPath(QStringLiteral("/"));
+    }
+    if (url2.path().isEmpty()) {
+        url2.setPath(QStringLiteral("/"));
+    }
+
+    return url1.matches(url2, QUrl::StripTrailingSlash | QUrl::NormalizePathSegments);
+}
+
 QString Utility::makeConflictFileName(
     const QString &fn, const QDateTime &dt, const QString &user)
 {
@@ -640,4 +600,47 @@ QString Utility::sanitizeForFileName(const QString &name)
     return result;
 }
 
+QString Utility::renderTemplate(QString templ, const QMap<QString, QString> &values)
+{
+    static const QRegularExpression pattern(QStringLiteral("@{([^{}]+)}"));
+    const auto replace = [&templ, &values](QRegularExpressionMatchIterator it) {
+        while (it.hasNext()) {
+            const auto match = it.next();
+            Q_ASSERT(match.lastCapturedIndex() == 1);
+            Q_ASSERT([&] {
+                if (!values.contains(match.captured(1))) {
+                    qCCritical(lcUtility) << "Unknown key:" << match.captured(1);
+                    return false;
+                }
+                return true;
+            }());
+            templ.replace(match.captured(0), values.value(match.captured(1)));
+        }
+    };
+
+    auto matches = pattern.globalMatch(templ);
+    do {
+        replace(matches);
+        // the placeholder can again contain a placeholder
+        matches = pattern.globalMatch(templ);
+    } while (matches.hasNext());
+
+    return templ;
+}
+
 } // namespace OCC
+
+QDebug operator<<(QDebug debug, nanoseconds in)
+{
+    QDebugStateSaver save(debug);
+    debug.nospace();
+    const auto h = duration_cast<hours>(in);
+    const auto min = duration_cast<minutes>(in -= h);
+    const auto s = duration_cast<seconds>(in -= min);
+    const auto ms = duration_cast<milliseconds>(in -= s);
+    return debug << "duration("
+                 << h.count() << "h, "
+                 << min.count() << "min, "
+                 << s.count() << "s, "
+                 << ms.count() << "ms)";
+}
