@@ -15,7 +15,6 @@
 #include "accountmanager.h"
 #include "configfile.h"
 #include "creds/credentialmanager.h"
-#include "sslerrordialog.h"
 #include "proxyauthhandler.h"
 #include "common/asserts.h"
 #include <theme.h>
@@ -26,9 +25,26 @@
 #include <QNetworkAccessManager>
 
 namespace {
-static const char urlC[] = "url";
-static const char userC[] = "user";
-static const char httpUserC[] = "http_user";
+auto urlC()
+{
+    return QStringLiteral("url");
+}
+
+auto userC()
+{
+    return QStringLiteral("user");
+}
+
+auto httpUserC()
+{
+    return QStringLiteral("http_user");
+}
+
+auto defaultSyncRootC()
+{
+    return QStringLiteral("default_sync_root");
+}
+
 const QString davUserC()
 {
     return QStringLiteral("dav_user");
@@ -44,10 +60,25 @@ const QString userUUIDC()
     return QStringLiteral("uuid");
 }
 
-static const char caCertsKeyC[] = "CaCertificates";
-static const char accountsC[] = "Accounts";
-static const char versionC[] = "version";
-static const char serverVersionC[] = "serverVersion";
+auto caCertsKeyC()
+{
+    return QStringLiteral("CaCertificates");
+}
+
+auto accountsC()
+{
+    return QStringLiteral("Accounts");
+}
+
+auto versionC()
+{
+    return QStringLiteral("version");
+}
+
+auto capabilitesC()
+{
+    return QStringLiteral("capabilities");
+}
 
 // The maximum versions that this client can read
 static const int maxAccountsVersion = 2;
@@ -70,7 +101,7 @@ bool AccountManager::restore()
     QStringList skipSettingsKeys;
     backwardMigrationSettingsKeys(&skipSettingsKeys, &skipSettingsKeys);
 
-    auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
+    auto settings = ConfigFile::settingsWithGroup(accountsC());
     if (settings->status() != QSettings::NoError) {
         qCWarning(lcAccountManager) << "Could not read settings from" << settings->fileName()
                                     << settings->status();
@@ -86,7 +117,7 @@ bool AccountManager::restore()
     // If there are no accounts, check the old format.
     const auto &childGroups = settings->childGroups();
     if (childGroups.isEmpty()
-        && !settings->contains(QLatin1String(versionC))) {
+        && !settings->contains(versionC())) {
         restoreFromLegacySettings();
         return true;
     }
@@ -97,10 +128,6 @@ bool AccountManager::restore()
             if (auto acc = loadAccountHelper(*settings)) {
                 acc->_id = accountId;
                 if (auto accState = AccountState::loadFromSettings(acc, *settings)) {
-                    auto jar = qobject_cast<CookieJar*>(acc->_am->cookieJar());
-                    OC_ASSERT(jar);
-                    if (jar)
-                        jar->restore(acc->cookieJarPath());
                     addAccountState(accState);
                 }
             }
@@ -116,13 +143,13 @@ bool AccountManager::restore()
 
 void AccountManager::backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys)
 {
-    auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
-    const int accountsVersion = settings->value(QLatin1String(versionC)).toInt();
+    auto settings = ConfigFile::settingsWithGroup(accountsC());
+    const int accountsVersion = settings->value(versionC()).toInt();
     if (accountsVersion <= maxAccountsVersion) {
         const auto &childGroups = settings->childGroups();
         for (const auto &accountId : childGroups) {
             settings->beginGroup(accountId);
-            const int accountVersion = settings->value(QLatin1String(versionC), 1).toInt();
+            const int accountVersion = settings->value(versionC(), 1).toInt();
             if (accountVersion > maxAccountVersion) {
                 ignoreKeys->append(settings->group());
             }
@@ -150,8 +177,8 @@ bool AccountManager::restoreFromLegacySettings()
         // Now try to open the original ownCloud settings to see if they exist.
         QString oCCfgFile = QDir::fromNativeSeparators(settings->fileName());
         // replace the last two segments with ownCloud/owncloud.cfg
-        oCCfgFile = oCCfgFile.left(oCCfgFile.lastIndexOf('/'));
-        oCCfgFile = oCCfgFile.left(oCCfgFile.lastIndexOf('/'));
+        oCCfgFile = oCCfgFile.left(oCCfgFile.lastIndexOf(QLatin1Char('/')));
+        oCCfgFile = oCCfgFile.left(oCCfgFile.lastIndexOf(QLatin1Char('/')));
         oCCfgFile += QLatin1String("/ownCloud/owncloud.cfg");
 
         qCInfo(lcAccountManager) << logPrefix
@@ -162,17 +189,17 @@ bool AccountManager::restoreFromLegacySettings()
 #endif
         QFileInfo fi(oCCfgFile);
         if (fi.isReadable()) {
-            std::unique_ptr<QSettings> oCSettings(new QSettings(oCCfgFile, QSettings::IniFormat));
-            oCSettings->beginGroup(QLatin1String("ownCloud"));
+            auto oCSettings = std::make_unique<QSettings>(oCCfgFile, QSettings::IniFormat);
+            oCSettings->beginGroup(QStringLiteral("ownCloud"));
 
             // Check the theme url to see if it is the same url that the oC config was for
             QString overrideUrl = Theme::instance()->overrideServerUrlV2();
             if (!overrideUrl.isEmpty()) {
-                if (overrideUrl.endsWith('/')) {
+                if (overrideUrl.endsWith(QLatin1Char('/'))) {
                     overrideUrl.chop(1);
                 }
-                QString oCUrl = oCSettings->value(QLatin1String(urlC)).toString();
-                if (oCUrl.endsWith('/')) {
+                QString oCUrl = oCSettings->value(urlC()).toString();
+                if (oCUrl.endsWith(QLatin1Char('/'))) {
                     oCUrl.chop(1);
                 }
 
@@ -200,8 +227,8 @@ bool AccountManager::restoreFromLegacySettings()
 
 void AccountManager::save(bool saveCredentials)
 {
-    auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
-    settings->setValue(QLatin1String(versionC), maxAccountsVersion);
+    auto settings = ConfigFile::settingsWithGroup(accountsC());
+    settings->setValue(versionC(), maxAccountsVersion);
     for (const auto &acc : qAsConst(_accounts)) {
         settings->beginGroup(acc->account()->id());
         saveAccountHelper(acc->account().data(), *settings, saveCredentials);
@@ -216,7 +243,7 @@ void AccountManager::save(bool saveCredentials)
 void AccountManager::saveAccount(Account *a)
 {
     qCDebug(lcAccountManager) << "Saving account" << a->url().toString();
-    auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
+    auto settings = ConfigFile::settingsWithGroup(accountsC());
     settings->beginGroup(a->id());
     saveAccountHelper(a, *settings, false); // don't save credentials they might not have been loaded yet
     settings->endGroup();
@@ -225,18 +252,6 @@ void AccountManager::saveAccount(Account *a)
     qCDebug(lcAccountManager) << "Saved account settings, status:" << settings->status();
 }
 
-void AccountManager::saveAccountState(AccountState *a)
-{
-    qCDebug(lcAccountManager) << "Saving account state" << a->account()->url().toString();
-    auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
-    settings->beginGroup(a->account()->id());
-    a->writeToSettings(*settings);
-    settings->endGroup();
-
-    settings->sync();
-    qCDebug(lcAccountManager) << "Saved account state settings, status:" << settings->status();
-}
-
 QStringList AccountManager::accountNames() const
 {
     QStringList accounts;
@@ -250,12 +265,17 @@ QStringList AccountManager::accountNames() const
 
 void AccountManager::saveAccountHelper(Account *acc, QSettings &settings, bool saveCredentials)
 {
-    settings.setValue(QLatin1String(versionC), maxAccountVersion);
-    settings.setValue(QLatin1String(urlC), acc->_url.toString());
+    settings.setValue(versionC(), maxAccountVersion);
+    settings.setValue(urlC(), acc->_url.toString());
     settings.setValue(davUserC(), acc->_davUser);
     settings.setValue(davUserDisplyNameC(), acc->_displayName);
     settings.setValue(userUUIDC(), acc->uuid());
-    settings.setValue(QLatin1String(serverVersionC), acc->_serverVersion);
+    if (acc->hasCapabilities()) {
+        settings.setValue(capabilitesC(), acc->capabilities().raw());
+    }
+    if (acc->hasDefaultSyncRoot()) {
+        settings.setValue(defaultSyncRootC(), acc->defaultSyncRoot());
+    }
     if (acc->_credentials) {
         if (saveCredentials) {
             // Only persist the credentials if the parameter is set, on migration from 1.8.x
@@ -270,12 +290,12 @@ void AccountManager::saveAccountHelper(Account *acc, QSettings &settings, bool s
         }
 
         // HACK: Save http_user also as user
-        if (acc->_settingsMap.contains(httpUserC))
-            settings.setValue(userC, acc->_settingsMap.value(httpUserC));
+        if (acc->_settingsMap.contains(httpUserC()))
+            settings.setValue(userC(), acc->_settingsMap.value(httpUserC()));
     }
 
     // Save accepted certificates.
-    settings.beginGroup(QLatin1String("General"));
+    settings.beginGroup(QStringLiteral("General"));
     qCInfo(lcAccountManager) << "Saving " << acc->approvedCerts().count() << " unknown certs.";
     const auto approvedCerts = acc->approvedCerts();
     QByteArray certs;
@@ -283,26 +303,14 @@ void AccountManager::saveAccountHelper(Account *acc, QSettings &settings, bool s
         certs += cert.toPem() + '\n';
     }
     if (!certs.isEmpty()) {
-        settings.setValue(QLatin1String(caCertsKeyC), certs);
+        settings.setValue(caCertsKeyC(), certs);
     }
     settings.endGroup();
-
-    // Save cookies.
-    if (acc->_am) {
-        CookieJar *jar = qobject_cast<CookieJar *>(acc->_am->cookieJar());
-        if (jar) {
-            qCInfo(lcAccountManager) << "Saving cookies." << acc->cookieJarPath();
-            if (!jar->save(acc->cookieJarPath()))
-            {
-                qCWarning(lcAccountManager) << "Failed to save cookies to" << acc->cookieJarPath();
-            }
-        }
-    }
 }
 
 AccountPtr AccountManager::loadAccountHelper(QSettings &settings)
 {
-    auto urlConfig = settings.value(QLatin1String(urlC));
+    auto urlConfig = settings.value(urlC());
     if (!urlConfig.isValid()) {
         // No URL probably means a corrupted entry in the account settings
         qCWarning(lcAccountManager) << "No URL for account " << settings.group();
@@ -311,23 +319,16 @@ AccountPtr AccountManager::loadAccountHelper(QSettings &settings)
 
     auto acc = createAccount();
 
-    QString overrideUrl = Theme::instance()->overrideServerUrlV2();
-    QString forceAuth = Theme::instance()->forceConfigAuthType();
-    if (!forceAuth.isEmpty() && !overrideUrl.isEmpty()) {
-        // If forceAuth is set, this might also mean the overrideURL has changed.
-        // See enterprise issues #1126
-        acc->setUrl(overrideUrl);
-    } else {
-        acc->setUrl(urlConfig.toUrl());
-    }
+    acc->setUrl(urlConfig.toUrl());
 
-    acc->_serverVersion = settings.value(QLatin1String(serverVersionC)).toString();
     acc->_davUser = settings.value(davUserC()).toString();
     acc->_displayName = settings.value(davUserDisplyNameC()).toString();
     acc->_uuid = settings.value(userUUIDC(), acc->_uuid).toUuid();
+    acc->setCapabilities(settings.value(capabilitesC()).value<QVariantMap>());
+    acc->setDefaultSyncRoot(settings.value(defaultSyncRootC()).toString());
 
     // We want to only restore settings for that auth type and the user value
-    acc->_settingsMap.insert(QLatin1String(userC), settings.value(userC));
+    acc->_settingsMap.insert(userC(), settings.value(userC()));
     const QString authTypePrefix = QStringLiteral("http_");
     const auto childKeys = settings.childKeys();
     for (const auto &key : childKeys) {
@@ -338,8 +339,8 @@ AccountPtr AccountManager::loadAccountHelper(QSettings &settings)
     acc->setCredentials(new HttpCredentialsGui);
 
     // now the server cert, it is in the general group
-    settings.beginGroup(QLatin1String("General"));
-    const auto certs = QSslCertificate::fromData(settings.value(caCertsKeyC).toByteArray());
+    settings.beginGroup(QStringLiteral("General"));
+    const auto certs = QSslCertificate::fromData(settings.value(caCertsKeyC()).toByteArray());
     qCInfo(lcAccountManager) << "Restored: " << certs.count() << " unknown certs.";
     acc->setApprovedCerts(certs);
     settings.endGroup();
@@ -361,7 +362,7 @@ AccountStatePtr AccountManager::account(const QUuid uuid) {
     return _accounts.value(uuid);
 }
 
-AccountState *AccountManager::addAccount(const AccountPtr &newAccount)
+AccountStatePtr AccountManager::addAccount(const AccountPtr &newAccount)
 {
     auto id = newAccount->id();
     if (id.isEmpty() || !isAccountIdAvailable(id)) {
@@ -369,26 +370,26 @@ AccountState *AccountManager::addAccount(const AccountPtr &newAccount)
     }
     newAccount->_id = id;
 
-    auto newAccountState = new AccountState(newAccount);
+    AccountStatePtr newAccountState(AccountState::fromNewAccount(newAccount));
     addAccountState(newAccountState);
     return newAccountState;
 }
 
-void AccountManager::deleteAccount(AccountState *account)
+void AccountManager::deleteAccount(AccountStatePtr account)
 {
     auto it = std::find(_accounts.begin(), _accounts.end(), account);
     if (it == _accounts.end()) {
         return;
     }
-    auto copy = *it; // keep a reference to the shared pointer so it does not delete it just yet
+    // The argument keeps a strong reference to the AccountState, so we can safely remove other
+    // AccountStatePtr occurrences:
     _accounts.erase(it);
 
     // Forget account credentials, cookies
     account->account()->credentials()->forgetSensitiveData();
     account->account()->credentialManager()->clear();
-    QFile::remove(account->account()->cookieJarPath());
 
-    auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
+    auto settings = ConfigFile::settingsWithGroup(accountsC());
     settings->remove(account->account()->id());
 
     emit accountRemoved(account);
@@ -397,7 +398,6 @@ void AccountManager::deleteAccount(AccountState *account)
 AccountPtr AccountManager::createAccount()
 {
     AccountPtr acc = Account::create();
-    acc->setSslErrorHandler(new SslDialogErrorHandler);
     connect(acc.data(), &Account::proxyAuthenticationRequired,
         ProxyAuthHandler::instance(), &ProxyAuthHandler::handleProxyAuthenticationRequired);
     return acc;
@@ -407,7 +407,7 @@ void AccountManager::shutdown()
 {
     const auto accounts = std::move(_accounts);
     for (const auto &acc : accounts) {
-        emit accountRemoved(acc.data());
+        emit accountRemoved(acc);
     }
 }
 
@@ -435,13 +435,13 @@ QString AccountManager::generateFreeAccountId() const
     }
 }
 
-void AccountManager::addAccountState(AccountState *accountState)
+void AccountManager::addAccountState(AccountStatePtr accountState)
 {
     QObject::connect(accountState->account().data(),
         &Account::wantsAccountSaved,
         this, &AccountManager::saveAccount);
 
-    _accounts.insert(accountState->account()->uuid(), AccountStatePtr{accountState});
+    _accounts.insert(accountState->account()->uuid(), accountState);
     emit accountAdded(accountState);
 }
 }
