@@ -22,14 +22,16 @@
 #include <QTimer>
 
 #include "folder.h"
-#include "quotainfo.h"
-#include "progressdispatcher.h"
+#include "loginrequireddialog.h"
 #include "owncloudgui.h"
+#include "progressdispatcher.h"
+#include "quotainfo.h"
 
 class QModelIndex;
 class QNetworkReply;
 class QListWidgetItem;
 class QLabel;
+class QSortFilterProxyModel;
 
 namespace OCC {
 
@@ -50,23 +52,23 @@ class FolderStatusModel;
 class AccountSettings : public QWidget
 {
     Q_OBJECT
-    Q_PROPERTY(AccountState* accountState MEMBER _accountState)
+    Q_PROPERTY(AccountStatePtr accountState MEMBER _accountState)
 
 public:
-    explicit AccountSettings(AccountState *accountState, QWidget *parent = nullptr);
+    explicit AccountSettings(const AccountStatePtr &accountState, QWidget *parent = nullptr);
     ~AccountSettings() override;
 
+    AccountStatePtr accountsState() const { return _accountState; }
+
 signals:
     void folderChanged();
-    void showIssuesList(const QString &folderAlias);
+    void showIssuesList();
 
 public slots:
     void slotOpenOC();
     void slotUpdateQuota(qint64, qint64);
     void slotAccountStateChanged();
 
-    AccountState *accountsState() { return _accountState; }
-
 protected slots:
     void slotAddFolder();
     void slotEnableCurrentFolder(bool terminate = false);
@@ -76,12 +78,10 @@ protected slots:
     void slotRemoveCurrentFolder();
     void slotEnableVfsCurrentFolder();
     void slotDisableVfsCurrentFolder();
-    void slotSetCurrentFolderAvailability(PinState state);
     void slotFolderWizardAccepted();
-    void slotFolderWizardRejected();
     void slotDeleteAccount();
     void slotToggleSignInState();
-    void refreshSelectiveSyncStatus();
+    [[deprecated]] void refreshSelectiveSyncStatus();
     void slotCustomContextMenuRequested(const QPoint &);
     void slotFolderListClicked(const QModelIndex &indx);
     void doExpand();
@@ -94,17 +94,21 @@ private:
     void createAccountToolbox();
 
     /// Returns the alias of the selected folder, empty string if none
-    QString selectedFolderAlias() const;
+    Folder *selectedFolder() const;
 
     Ui::AccountSettings *ui;
 
     FolderStatusModel *_model;
+    QSortFilterProxyModel *_sortModel;
     QUrl _OCUrl;
     bool _wasDisabledBefore;
-    AccountState *_accountState;
-    QuotaInfo _quotaInfo;
+    AccountStatePtr _accountState;
+    QuotaInfo *_quotaInfo = nullptr;
     QAction *_toggleSignInOutAction;
     QAction *_addAccountAction;
+
+    // needed to make sure we show only one dialog at a time
+    QPointer<LoginRequiredDialog> _askForOAuthLoginDialog = nullptr;
 };
 
 } // namespace OCC
