@@ -13,25 +13,23 @@
  * for more details.
  */
 
-#include "application.h"
-#include "account.h"
-#include "common/asserts.h"
 #include "creds/httpcredentialsgui.h"
+#include "account.h"
+#include "application.h"
+#include "basicloginwidget.h"
+#include "common/asserts.h"
+#include "loginrequireddialog.h"
 #include "networkjobs.h"
 #include "settingsdialog.h"
 #include "theme.h"
 
 #include <QBuffer>
 #include <QDesktopServices>
-#include <QInputDialog>
-#include <QLabel>
 #include <QMessageBox>
 #include <QNetworkReply>
 #include <QTimer>
 
 
-using namespace QKeychain;
-
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcHttpCredentialsGui, "sync.credentials.http.gui", QtInfoMsg)
@@ -55,29 +53,20 @@ void HttpCredentialsGui::askFromUser()
     // which (indirectly, through HttpCredentials::invalidateToken) schedules
     // a cache wipe of the qnam. We can only execute a network job again once
     // the cache has been cleared, otherwise we'd interfere with the job.
-    QTimer::singleShot(100, this, &HttpCredentialsGui::askFromUserAsync);
+    QTimer::singleShot(0, this, &HttpCredentialsGui::askFromUserAsync);
 }
 
 void HttpCredentialsGui::askFromUserAsync()
 {
-    const auto updateOAuth = [this] {
-        _asyncAuth.reset(new OAuth(_account, this));
-        connect(_asyncAuth.data(), &OAuth::result,
-            this, &HttpCredentialsGui::asyncAuthResult);
-        connect(_asyncAuth.data(), &OAuth::destroyed,
-            this, &HttpCredentialsGui::authorisationLinkChanged);
-        _asyncAuth->startAuthentication();
-        emit authorisationLinkChanged();
-    };
     if (isUsingOAuth()) {
-        updateOAuth();
+        restartOAuth();
     } else {
         // First, we will check what kind of auth we need.
         auto job = new DetermineAuthTypeJob(_account->sharedFromThis(), this);
-        QObject::connect(job, &DetermineAuthTypeJob::authType, this, [updateOAuth, this](DetermineAuthTypeJob::AuthType type) {
+        QObject::connect(job, &DetermineAuthTypeJob::authType, this, [this](DetermineAuthTypeJob::AuthType type) {
             _authType = type;
             if (type == DetermineAuthTypeJob::AuthType::OAuth) {
-                updateOAuth();
+                restartOAuth();
             } else if (type == DetermineAuthTypeJob::AuthType::Basic) {
                 showDialog();
             } else {
@@ -90,7 +79,7 @@ void HttpCredentialsGui::askFromUserAsync()
 }
 
 void HttpCredentialsGui::asyncAuthResult(OAuth::Result r, const QString &user,
-    const QString &token, const QString &refreshToken)
+    const QString &token, const QString &refreshToken, const QString &)
 {
     _asyncAuth.reset();
     switch (r) {
@@ -98,7 +87,7 @@ void HttpCredentialsGui::asyncAuthResult(OAuth::Result r, const QString &user,
         showDialog();
         return;
     case OAuth::Error:
-        emit asked();
+        Q_EMIT oAuthErrorOccurred();
         return;
     case OAuth::LoggedIn:
         break;
@@ -115,68 +104,69 @@ void HttpCredentialsGui::asyncAuthResult(OAuth::Result r, const QString &user,
 
 void HttpCredentialsGui::showDialog()
 {
-    QString msg = tr("Please enter %1 password:<br>"
-                     "<br>"
-                     "User: %2<br>"
-                     "Account: %3<br>")
-                      .arg(Utility::escape(Theme::instance()->appNameGUI()),
-                          Utility::escape(_user),
-                          Utility::escape(_account->displayName()));
-
-    QString reqTxt = requestAppPasswordText(_account);
-    if (!reqTxt.isEmpty()) {
-        msg += QLatin1String("<br>") + reqTxt + QLatin1String("<br>");
-    }
-    if (!_fetchErrorString.isEmpty()) {
-        msg += QLatin1String("<br>")
-            + tr("Reading from keychain failed with error: '%1'")
-                  .arg(Utility::escape(_fetchErrorString))
-            + QLatin1String("<br>");
+    if (_loginRequiredDialog != nullptr) {
+        return;
     }
 
-    QInputDialog *dialog = new QInputDialog(ocApp()->gui()->settingsDialog());
-    dialog->setAttribute(Qt::WA_DeleteOnClose, true);
-    dialog->setWindowTitle(tr("Enter Password"));
-    dialog->setLabelText(msg);
-    dialog->setTextValue(_previousPassword);
-    dialog->setTextEchoMode(QLineEdit::Password);
-    if (QLabel *dialogLabel = dialog->findChild<QLabel *>()) {
-        dialogLabel->setOpenExternalLinks(true);
-        dialogLabel->setTextFormat(Qt::RichText);
-    }
+    auto *dialog = new LoginRequiredDialog(LoginRequiredDialog::Mode::Basic, ocApp()->gui()->settingsDialog());
 
-    connect(dialog, &QDialog::finished, this, [this, dialog](int result) {
+    // make sure it's cleaned up since it's not owned by the account settings (also prevents memory leaks)
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+
+    dialog->setTopLabelText(tr("Please enter your password to log in to your account."));
+
+    auto *contentWidget = qobject_cast<BasicLoginWidget *>(dialog->contentWidget());
+    contentWidget->forceUsername(_account->davUser());
+
+    // in this case, we want to use the login button
+    dialog->addLogInButton();
+
+    connect(dialog, &LoginRequiredDialog::finished, ocApp()->gui()->settingsDialog(), [this, contentWidget](const int result) {
         if (result == QDialog::Accepted) {
-            _password = dialog->textValue();
+            Q_ASSERT(contentWidget->username() == _account->davUser());
+            _password = contentWidget->password();
             _refreshToken.clear();
             _ready = true;
             persist();
+        } else {
+            Q_EMIT requestLogout();
         }
         emit asked();
     });
+
     dialog->open();
     ownCloudGui::raiseDialog(dialog);
+
+    QTimer::singleShot(0, contentWidget, [contentWidget]() {
+        contentWidget->setFocus(Qt::OtherFocusReason);
+    });
+
+    _loginRequiredDialog = dialog;
 }
 
-QString HttpCredentialsGui::requestAppPasswordText(const Account *account)
+QUrl HttpCredentialsGui::authorisationLink() const
 {
-    int version = account->serverVersionInt();
-    QString path;
-
-    // Version may not be available before login on new servers!
-    if (!version || version >= Account::makeServerVersion(10, 0, 0)) {
-        path = QLatin1String("/index.php/settings/personal?sectionid=security#apppasswords");
-    } else if (version >= Account::makeServerVersion(9, 1, 0)) {
-        path = QLatin1String("/index.php/settings/personal?section=apppasswords");
-    } else {
-        // Older server than 9.1 does not have the feature to request App Password
-        return QString();
+    OC_ASSERT(isUsingOAuth());
+    if (isUsingOAuth()) {
+        if (_asyncAuth) {
+            return _asyncAuth->authorisationLink();
+        } else {
+            qCWarning(lcHttpCredentialsGui) << "There is no authentication job running, did the previous attempt fail?";
+            return {};
+        }
     }
-
-    auto baseUrl = account->url().toString();
-    if (baseUrl.endsWith('/'))
-        baseUrl.chop(1);
-    return tr("<a href=\"%1\">Click here</a> to request an app password from the web interface.")
-        .arg(baseUrl + path);
+    return {};
 }
+
+void HttpCredentialsGui::restartOAuth()
+{
+    _asyncAuth.reset(new AccountBasedOAuth(_account->sharedFromThis(), this));
+    connect(_asyncAuth.data(), &OAuth::result,
+        this, &HttpCredentialsGui::asyncAuthResult);
+    connect(_asyncAuth.data(), &OAuth::destroyed,
+        this, &HttpCredentialsGui::authorisationLinkChanged);
+    _asyncAuth->startAuthentication();
+    emit authorisationLinkChanged();
+}
+
 } // namespace OCC
