@@ -26,31 +26,8 @@ class FolderStatusDelegate : public QStyledItemDelegate
 {
     Q_OBJECT
 public:
-    FolderStatusDelegate();
+    FolderStatusDelegate(QObject *parent);
 
-    // TODO: don't use the role but columns...
-    enum datarole { FolderAliasRole = Qt::UserRole + 100,
-        HeaderRole,
-        FolderPathRole, // for a SubFolder it's the complete path
-        FolderSecondPathRole,
-        FolderConflictMsg,
-        FolderErrorMsg,
-        FolderInfoMsg,
-        FolderSyncPaused,
-        FolderStatusIconRole,
-        FolderAccountConnected,
-
-        SyncProgressOverallPercent,
-        SyncProgressOverallString,
-        SyncProgressItemString,
-        WarningCount,
-        SyncRunning,
-
-        AddButton, // 1 = enabled; 2 = disabled
-        FolderSyncText,
-
-        IsReady // boolean
-    };
     void paint(QPainter *, const QStyleOptionViewItem &, const QModelIndex &) const override;
     QSize sizeHint(const QStyleOptionViewItem &, const QModelIndex &) const override;
 
@@ -63,7 +40,7 @@ public:
     static int rootFolderHeightWithoutErrors(const QFontMetrics &fm, const QFontMetrics &aliasFm);
 
 private:
-    static QString addFolderText();
+    static QString addFolderText(bool useSapces);
 };
 
 } // namespace OCC
