@@ -12,25 +12,28 @@
  * for more details.
  */
 
-#include "application.h"
 #include "owncloudgui.h"
-#include "theme.h"
-#include "folderman.h"
-#include "configfile.h"
-#include "progressdispatcher.h"
-#include "owncloudsetupwizard.h"
-#include "sharedialog.h"
-#include "settingsdialog.h"
-#include "logger.h"
-#include "logbrowser.h"
-#include "account.h"
-#include "accountstate.h"
-#include "openfilemanager.h"
-#include "accountmanager.h"
 #include "aboutdialog.h"
+#include "account.h"
+#include "accountmanager.h"
+#include "accountstate.h"
+#include "application.h"
 #include "common/syncjournalfilerecord.h"
+#include "configfile.h"
 #include "creds/abstractcredentials.h"
+#include "folderman.h"
+#include "folderwizard/folderwizard.h"
+#include "graphapi/drives.h"
+#include "gui/accountsettings.h"
 #include "guiutility.h"
+#include "logbrowser.h"
+#include "logger.h"
+#include "openfilemanager.h"
+#include "progressdispatcher.h"
+#include "settingsdialog.h"
+#include "setupwizardcontroller.h"
+#include "sharedialog.h"
+#include "theme.h"
 
 #include <QDesktopServices>
 #include <QDir>
@@ -41,8 +44,68 @@
 
 #ifdef WITH_LIBCLOUDPROVIDERS
 #include "libcloudproviders/libcloudproviders.h"
+#include "selectivesyncdialog.h"
 #endif
 
+using namespace std::chrono_literals;
+
+namespace {
+
+using namespace OCC;
+
+void setUpInitialSyncFolder(AccountStatePtr accountStatePtr, bool useVfs)
+{
+    auto folderMan = FolderMan::instance();
+
+    // saves a bit of duplicate code
+    auto addFolder = [folderMan, accountStatePtr, useVfs](const QString &localFolder, const QString &remotePath, const QUrl &webDavUrl, const QString &displayName = {}) {
+        return folderMan->addFolderFromWizard(accountStatePtr, localFolder, remotePath, webDavUrl, displayName, useVfs);
+    };
+
+    auto finalize = [accountStatePtr] {
+        accountStatePtr->checkConnectivity();
+        FolderMan::instance()->setSyncEnabled(true);
+        FolderMan::instance()->scheduleAllFolders();
+    };
+
+    if (accountStatePtr->supportsSpaces()) {
+        auto *drive = new GraphApi::Drives(accountStatePtr->account());
+
+        QObject::connect(drive, &GraphApi::Drives::finishedSignal, [accountStatePtr, drive, addFolder, finalize] {
+            if (drive->parseError().error == QJsonParseError::NoError) {
+                auto drives = drive->drives();
+
+                // we do not want to set up folder sync connections for disabled spaces (#10173)
+                drives.erase(std::remove_if(drives.begin(), drives.end(), &GraphApi::isDriveDisabled), drives.end());
+
+                if (!drives.isEmpty()) {
+                    const QDir localDir(accountStatePtr->account()->defaultSyncRoot());
+                    FileSystem::setFolderMinimumPermissions(localDir.path());
+                    Folder::prepareFolder(localDir.path());
+                    Utility::setupFavLink(localDir.path());
+                    for (const auto &d : drives) {
+                        const QString name = GraphApi::Drives::getDriveDisplayName(d);
+                        const QString folderName = FolderMan::instance()->findGoodPathForNewSyncFolder(localDir.filePath(name));
+                        auto folder = addFolder(folderName, {}, QUrl::fromEncoded(d.getRoot().getWebDavUrl().toUtf8()), name);
+                        folder->setPriority(GraphApi::Drives::getDrivePriority(d));
+                        // save the new priority
+                        folder->saveToSettings();
+                    }
+                    finalize();
+                }
+            }
+        });
+
+        drive->start();
+
+        return;
+    } else {
+        addFolder(accountStatePtr->account()->defaultSyncRoot(), Theme::instance()->defaultServerFolder(), accountStatePtr->account()->davUrl());
+        finalize();
+    }
+}
+}
+
 namespace OCC {
 
 const char propertyAccountC[] = "oc_account";
@@ -146,13 +209,6 @@ void ownCloudGui::slotSyncStateChange(Folder *folder)
 
     qCInfo(lcApplication) << "Sync state changed for folder " << folder->remoteUrl().toString() << ": " << result.statusString();
 
-    if (result.status() == SyncResult::Success
-        || result.status() == SyncResult::Problem
-        || result.status() == SyncResult::SyncAbortRequested
-        || result.status() == SyncResult::Error) {
-        Logger::instance()->enterNextLogFile();
-    }
-
     if (result.status() == SyncResult::NotYetStarted) {
         _settingsDialog->slotRefreshActivity(folder->accountState());
     }
@@ -177,13 +233,13 @@ void ownCloudGui::slotAccountStateChanged()
 
 void ownCloudGui::slotTrayMessageIfServerUnsupported(Account *account)
 {
-    if (account->serverVersionUnsupported()) {
+    if (account->serverSupportLevel() != Account::ServerSupportLevel::Supported) {
         slotShowTrayMessage(
             tr("Unsupported Server Version"),
             tr("The server on account %1 runs an unsupported version %2. "
                "Using this client with unsupported server versions is untested and "
                "potentially dangerous. Proceed at your own risk.")
-                .arg(account->displayName(), account->serverVersion()));
+                .arg(account->displayName(), account->capabilities().status().versionString()));
     }
 }
 
@@ -200,7 +256,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
         _actionStatus->setText(text);
     };
 
-    foreach (auto a, AccountManager::instance()->accounts()) {
+    for (const auto &a : AccountManager::instance()->accounts()) {
         if (!a->isSignedOut()) {
             allSignedOut = false;
         }
@@ -210,7 +266,9 @@ void ownCloudGui::slotComputeOverallSyncStatus()
             allDisconnected = false;
         }
     }
-    foreach (Folder *f, FolderMan::instance()->map()) {
+
+    const auto &map = FolderMan::instance()->folders();
+    for (auto *f : map) {
         if (!f->syncPaused()) {
             allPaused = false;
         }
@@ -226,14 +284,14 @@ void ownCloudGui::slotComputeOverallSyncStatus()
 #ifdef Q_OS_WIN
         // Windows has a 128-char tray tooltip length limit.
         QStringList accountNames;
-        foreach (AccountStatePtr a, problemAccounts) {
+        for (const auto &a : qAsConst(problemAccounts)) {
             accountNames.append(a->account()->displayName());
         }
         _tray->setToolTip(tr("Disconnected from %1").arg(accountNames.join(QLatin1String(", "))));
 #else
         QStringList messages;
         messages.append(tr("Disconnected from accounts:"));
-        foreach (AccountStatePtr a, problemAccounts) {
+        for (const auto &a : qAsConst(problemAccounts)) {
             QString message = tr("Account %1: %2").arg(a->account()->displayName(), a->stateString(a->state()));
             if (!a->connectionErrors().empty()) {
                 message += QLatin1String("\n");
@@ -260,60 +318,51 @@ void ownCloudGui::slotComputeOverallSyncStatus()
 
     // display the info of the least successful sync (eg. do not just display the result of the latest sync)
     QString trayMessage;
-    FolderMan *folderMan = FolderMan::instance();
-    Folder::Map map = folderMan->map();
 
-    SyncResult::Status overallStatus = SyncResult::Undefined;
-    bool hasUnresolvedConflicts = false;
-    FolderMan::trayOverallStatus(map.values(), &overallStatus, &hasUnresolvedConflicts);
+    auto trayOverallStatusResult = FolderMan::trayOverallStatus(map);
 
-    // If the sync succeeded but there are unresolved conflicts,
-    // show the problem icon!
-    auto iconStatus = overallStatus;
-    if (iconStatus == SyncResult::Success && hasUnresolvedConflicts) {
-        iconStatus = SyncResult::Problem;
-    }
-
-    // If we don't get a status for whatever reason, that's a Problem
-    if (iconStatus == SyncResult::Undefined) {
-        iconStatus = SyncResult::Problem;
-    }
-
-    QIcon statusIcon = Theme::instance()->syncStateIcon(iconStatus, true, contextMenuVisible());
+    const QIcon statusIcon = Theme::instance()->syncStateIcon(trayOverallStatusResult.overallStatus(), true, contextMenuVisible());
     _tray->setIcon(statusIcon);
 
     // create the tray blob message, check if we have an defined state
-    if (map.count() > 0) {
 #ifdef Q_OS_WIN
-        // Windows has a 128-char tray tooltip length limit.
-        trayMessage = folderMan->trayTooltipStatusString(overallStatus, hasUnresolvedConflicts, false);
+    // Windows has a 128-char tray tooltip length limit.
+    trayMessage = FolderMan::instance()->trayTooltipStatusString(trayOverallStatusResult.overallStatus(), false);
 #else
-        QStringList allStatusStrings;
-        foreach (Folder *folder, map.values()) {
-            QString folderMessage = FolderMan::trayTooltipStatusString(
-                folder->syncResult().status(),
-                folder->syncResult().hasUnresolvedConflicts(),
-                folder->syncPaused());
-            allStatusStrings += tr("Folder %1: %2").arg(folder->shortGuiLocalPath(), folderMessage);
-        }
-        trayMessage = allStatusStrings.join(QLatin1String("\n"));
+    QStringList allStatusStrings;
+    for (auto *folder : map) {
+        QString folderMessage = FolderMan::trayTooltipStatusString(
+            folder->syncResult(),
+            folder->syncPaused());
+        allStatusStrings += tr("Folder %1: %2").arg(folder->shortGuiLocalPath(), folderMessage);
+    }
+    trayMessage = allStatusStrings.join(QLatin1String("\n"));
 #endif
-        _tray->setToolTip(trayMessage);
+    _tray->setToolTip(trayMessage);
 
-        if (overallStatus == SyncResult::Success || overallStatus == SyncResult::Problem) {
-            if (hasUnresolvedConflicts) {
-                setStatusText(tr("Unresolved conflicts"));
-            } else {
-                setStatusText(tr("Up to date"));
-            }
-        } else if (overallStatus == SyncResult::Paused) {
-            setStatusText(tr("Synchronization is paused"));
-        } else {
-            setStatusText(tr("Error during synchronization"));
+    switch (trayOverallStatusResult.overallStatus().status()) {
+    case SyncResult::Problem:
+        if (trayOverallStatusResult.overallStatus().hasUnresolvedConflicts()) {
+            setStatusText(tr("Unresolved %1 conflicts").arg(QString::number(trayOverallStatusResult.overallStatus().numNewConflictItems())));
+        } else if (trayOverallStatusResult.overallStatus().numBlacklistErrors() != 0) {
+            setStatusText(tr("Ignored errors %1").arg(QString::number(trayOverallStatusResult.overallStatus().numBlacklistErrors())));
         }
-    } else {
+        break;
+    case SyncResult::Success: {
+        QString lastSyncDoneString;
+        // display only the time in case the last sync was today
+        if (QDateTime::currentDateTime().date() == trayOverallStatusResult.lastSyncDone.date()) {
+            lastSyncDoneString = QLocale().toString(trayOverallStatusResult.lastSyncDone.time());
+        } else {
+            lastSyncDoneString = QLocale().toString(trayOverallStatusResult.lastSyncDone);
+        }
+        setStatusText(tr("Up to date (%1)").arg(lastSyncDoneString));
+    } break;
+    case SyncResult::Undefined:
         _tray->setToolTip(tr("There are no sync folders configured."));
         setStatusText(tr("No sync folders configured"));
+    default:
+        setStatusText(FolderMan::instance()->trayTooltipStatusString(trayOverallStatusResult.overallStatus(), false));
     }
 }
 
@@ -331,10 +380,11 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
 
     FolderMan *folderMan = FolderMan::instance();
     bool firstFolder = true;
-    bool singleSyncFolder = folderMan->map().size() == 1 && Theme::instance()->singleSyncFolder();
+    const auto &map = folderMan->folders();
+    bool singleSyncFolder = map.size() == 1 && Theme::instance()->singleSyncFolder();
     bool onePaused = false;
     bool allPaused = true;
-    foreach (Folder *folder, folderMan->map()) {
+    for (auto *folder : map) {
         if (folder->accountState() != accountState.data()) {
             continue;
         }
@@ -352,8 +402,7 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
         }
 
         QAction *action = menu->addAction(tr("Open folder '%1'").arg(folder->shortGuiLocalPath()));
-        auto alias = folder->alias();
-        connect(action, &QAction::triggered, this, [this, alias] { this->slotFolderOpenAction(alias); });
+        connect(action, &QAction::triggered, this, [this, folder] { this->slotFolderOpenAction(folder); });
     }
 
     menu->addSeparator();
@@ -531,7 +580,7 @@ void ownCloudGui::setupContextMenu()
         || desktopSession.contains("plasma")
         || desktopSession.contains("kde");
     QObject *platformMenu = reinterpret_cast<QObject *>(_tray->contextMenu()->platformMenu());
-    if (isKde && platformMenu && platformMenu->metaObject()->className() == QLatin1String("QDBusPlatformMenu")) {
+    if (isKde && platformMenu && platformMenu->metaObject()->className() == QByteArrayLiteral("QDBusPlatformMenu")) {
         _workaroundManualVisibility = true;
         _workaroundNoAboutToShowUpdate = true;
     }
@@ -550,12 +599,12 @@ void ownCloudGui::setupContextMenu()
 
 
     connect(&_delayedTrayUpdateTimer, &QTimer::timeout, this, &ownCloudGui::updateContextMenu);
-    _delayedTrayUpdateTimer.setInterval(2 * 1000);
+    _delayedTrayUpdateTimer.setInterval(2s);
     _delayedTrayUpdateTimer.setSingleShot(true);
 
-    connect(_contextMenu.data(), SIGNAL(aboutToShow()), SLOT(slotContextMenuAboutToShow()));
+    connect(_contextMenu.data(), &QMenu::aboutToShow, this, &ownCloudGui::slotContextMenuAboutToShow);
     // unfortunately aboutToHide is unreliable, it seems to work on OSX though
-    connect(_contextMenu.data(), SIGNAL(aboutToHide()), SLOT(slotContextMenuAboutToHide()));
+    connect(_contextMenu.data(), &QMenu::aboutToHide, this, &ownCloudGui::slotContextMenuAboutToHide);
 
     // Populate the context menu now.
     updateContextMenu();
@@ -592,13 +641,13 @@ void ownCloudGui::updateContextMenu()
     slotRebuildRecentMenus();
 
     // We must call deleteLater because we might be called from the press in one of the actions.
-    foreach (auto menu, _accountMenus) {
+    for (auto *menu : qAsConst(_accountMenus)) {
         menu->deleteLater();
     }
     _accountMenus.clear();
 
 
-    auto accountList = AccountManager::instance()->accounts();
+    const auto &accountList = AccountManager::instance()->accounts();
 
     bool isConfigured = (!accountList.isEmpty());
     bool atLeastOneConnected = false;
@@ -606,7 +655,7 @@ void ownCloudGui::updateContextMenu()
     bool atLeastOneSignedIn = false;
     bool atLeastOnePaused = false;
     bool atLeastOneNotPaused = false;
-    foreach (auto a, accountList) {
+    for (const auto &a : accountList) {
         if (a->isConnected()) {
             atLeastOneConnected = true;
         }
@@ -616,7 +665,8 @@ void ownCloudGui::updateContextMenu()
             atLeastOneSignedIn = true;
         }
     }
-    foreach (auto f, FolderMan::instance()->map()) {
+
+    for (auto *f : FolderMan::instance()->folders()) {
         if (f->syncPaused()) {
             atLeastOnePaused = true;
         } else {
@@ -625,7 +675,7 @@ void ownCloudGui::updateContextMenu()
     }
 
     if (accountList.count() > 1) {
-        foreach (AccountStatePtr account, accountList) {
+        for (const auto &account : accountList) {
             QMenu *accountMenu = new QMenu(account->account()->displayName(), _contextMenu.data());
             _accountMenus.append(accountMenu);
             _contextMenu->addMenu(accountMenu);
@@ -749,9 +799,8 @@ void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QStrin
 /*
  * open the folder with the given Alias
  */
-void ownCloudGui::slotFolderOpenAction(const QString &alias)
+void ownCloudGui::slotFolderOpenAction(Folder *f)
 {
-    Folder *f = FolderMan::instance()->folder(alias);
     if (f) {
         qCInfo(lcApplication) << "opening local url " << f->path();
         QUrl url = QUrl::fromLocalFile(f->path());
@@ -773,7 +822,7 @@ void ownCloudGui::setupActions()
 {
     _actionStatus = new QAction(tr("Unknown status"), this);
     _actionStatus->setEnabled(false);
-    _actionSettings = new QAction(tr("Settings..."), this);
+    _actionSettings = new QAction(tr("Show %1").arg(Theme::instance()->appNameGUI()), this);
     _actionNewAccountWizard = new QAction(tr("New account..."), this);
     _actionRecent = new QAction(tr("Details..."), this);
     _actionRecent->setEnabled(true);
@@ -786,7 +835,7 @@ void ownCloudGui::setupActions()
     _actionAbout = new QAction(tr("About %1").arg(Theme::instance()->appNameGUI()), this);
     QObject::connect(_actionAbout, &QAction::triggered, this, &ownCloudGui::slotAbout);
     _actionQuit = new QAction(tr("Quit %1").arg(Theme::instance()->appNameGUI()), this);
-    QObject::connect(_actionQuit, SIGNAL(triggered(bool)), _app, SLOT(quit()));
+    QObject::connect(_actionQuit, &QAction::triggered, _app, &QApplication::quit);
 
     _actionLogin = new QAction(tr("Log in..."), this);
     connect(_actionLogin, &QAction::triggered, this, &ownCloudGui::slotLogin);
@@ -794,11 +843,11 @@ void ownCloudGui::setupActions()
     connect(_actionLogout, &QAction::triggered, this, &ownCloudGui::slotLogout);
 
     if (_app->debugMode()) {
-        _actionCrash = new QAction("Crash now - Div by zero", this);
+        _actionCrash = new QAction(QStringLiteral("Crash now - Div by zero"), this);
         connect(_actionCrash, &QAction::triggered, _app, &Application::slotCrash);
-        _actionCrashEnforce = new QAction("Crash now - ENFORCE()", this);
+        _actionCrashEnforce = new QAction(QStringLiteral("Crash now - ENFORCE()"), this);
         connect(_actionCrashEnforce, &QAction::triggered, _app, &Application::slotCrashEnforce);
-        _actionCrashFatal = new QAction("Crash now - qFatal", this);
+        _actionCrashFatal = new QAction(QStringLiteral("Crash now - qFatal"), this);
         connect(_actionCrashFatal, &QAction::triggered, _app, &Application::slotCrashFatal);
     } else {
         _actionCrash = nullptr;
@@ -811,7 +860,7 @@ void ownCloudGui::slotRebuildRecentMenus()
 {
     _recentActionsMenu->clear();
     if (!_recentItemsActions.isEmpty()) {
-        foreach (QAction *a, _recentItemsActions) {
+        for (auto *a : qAsConst(_recentItemsActions)) {
             _recentActionsMenu->addAction(a);
         }
         _recentActionsMenu->addSeparator();
@@ -831,11 +880,8 @@ static bool shouldShowInRecentsMenu(const SyncFileItem &item)
         && item._instruction != CSYNC_INSTRUCTION_NONE;
 }
 
-
-void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &progress)
+void ownCloudGui::slotUpdateProgress(Folder *folder, const ProgressInfo &progress)
 {
-    Q_UNUSED(folder);
-
     if (progress.status() == ProgressInfo::Discovery) {
         if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
             _actionStatus->setText(tr("Checking for changes in remote '%1'")
@@ -845,7 +891,7 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
                                        .arg(progress._currentDiscoveredLocalFolder));
         }
     } else if (progress.status() == ProgressInfo::Done) {
-        QTimer::singleShot(2000, this, &ownCloudGui::slotComputeOverallSyncStatus);
+        QTimer::singleShot(2s, this, &ownCloudGui::slotComputeOverallSyncStatus);
     }
     if (progress.status() != ProgressInfo::Propagation) {
         return;
@@ -889,17 +935,14 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
         }
 
         QString kindStr = Progress::asResultString(progress._lastCompletedItem);
-        QString timeStr = QTime::currentTime().toString("hh:mm");
+        QString timeStr = QTime::currentTime().toString(QStringLiteral("hh:mm"));
         QString actionText = tr("%1 (%2, %3)").arg(progress._lastCompletedItem._file, kindStr, timeStr);
         QAction *action = new QAction(actionText, this);
-        Folder *f = FolderMan::instance()->folder(folder);
-        if (f) {
-            QString fullPath = f->path() + '/' + progress._lastCompletedItem._file;
-            if (QFile(fullPath).exists()) {
-                connect(action, &QAction::triggered, this, [this, fullPath] { this->slotOpenPath(fullPath); });
-            } else {
-                action->setEnabled(false);
-            }
+        QString fullPath = folder->path() + QLatin1Char('/') + progress._lastCompletedItem._file;
+        if (QFile(fullPath).exists()) {
+            connect(action, &QAction::triggered, this, [this, fullPath] { this->slotOpenPath(fullPath); });
+        } else {
+            action->setEnabled(false);
         }
         if (_recentItemsActions.length() > 5) {
             _recentItemsActions.takeFirst()->deleteLater();
@@ -917,11 +960,9 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
 void ownCloudGui::slotLogin()
 {
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
-        account->account()->resetRejectedCertificates();
         account->signIn();
     } else {
-        auto list = AccountManager::instance()->accounts();
-        foreach (const auto &a, list) {
+        for (const auto &a : AccountManager::instance()->accounts()) {
             a->signIn();
         }
     }
@@ -932,10 +973,10 @@ void ownCloudGui::slotLogout()
     auto list = AccountManager::instance()->accounts();
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
         list.clear();
-        list.append(account);
+        list.insert(account->account()->uuid(), account);
     }
 
-    foreach (const auto &ai, list) {
+    for (const auto &ai : qAsConst(list)) {
         ai->signOutByUi();
     }
 }
@@ -952,27 +993,122 @@ void ownCloudGui::slotPauseAllFolders()
 
 void ownCloudGui::runNewAccountWizard()
 {
-    if (!_wizard) {
-        _wizard = new OwncloudSetupWizard(settingsDialog());
-        connect(_wizard, &OwncloudSetupWizard::ownCloudWizardDone, _wizard, &OwncloudSetupWizard::deleteLater);
-        connect(_wizard, &OwncloudSetupWizard::ownCloudWizardDone, ocApp(), &Application::slotownCloudWizardDone);
+    if (_wizardController.isNull()) {
+        // passing the settings dialog as parent makes sure the wizard will be shown above it
+        // as the settingsDialog's lifetime spans across the entire application but the dialog will live much shorter,
+        // we have to clean it up manually when finished() is emitted
+        _wizardController = new Wizard::SetupWizardController(settingsDialog());
+
+        // while the wizard is shown, new syncs are disabled
         FolderMan::instance()->setSyncEnabled(false);
-        _wizard->startWizard();
+
+        connect(_wizardController, &Wizard::SetupWizardController::finished, ocApp(),
+            [this](AccountPtr newAccount, Wizard::SyncMode syncMode, const QVariantMap &dynamicRegistrationData) {
+                // note: while the wizard is shown, we disable the folder synchronization
+                // previously we could perform this just here, but now we have to postpone this depending on whether selective sync was chosen
+                // see also #9497
+
+                // when the dialog is closed before it has finished, there won't be a new account to set up
+                // the wizard controller signalizes this by passing a null pointer
+                if (!newAccount.isNull()) {
+                    // finally, call the slot that finalizes the setup
+                    auto accountStatePtr = ocApp()->addNewAccount(newAccount);
+
+                    // ensure we are connected and fetch the capabilities
+                    auto validator = new ConnectionValidator(accountStatePtr->account(), accountStatePtr->account().data());
+
+                    QObject::connect(validator, &ConnectionValidator::connectionResult, accountStatePtr.data(), [accountStatePtr, syncMode, dynamicRegistrationData](ConnectionValidator::Status status, const QStringList &errors) {
+                        if (OC_ENSURE(status == ConnectionValidator::Connected || status == ConnectionValidator::ServerVersionMismatch)) {
+                            // saving once after adding makes sure the account is stored in the config in a working state
+                            // this is needed to ensure a consistent state in the config file upon unexpected terminations of the client
+                            // (for instance, when running from a debugger and stopping the process from there)
+                            AccountManager::instance()->save(true);
+
+                            // only now, we can store the dynamic registration data in the keychain
+                            if (!dynamicRegistrationData.isEmpty()) {
+                                OAuth::saveDynamicRegistrationDataForAccount(accountStatePtr->account(), dynamicRegistrationData);
+                            }
+
+                            switch (syncMode) {
+                            case Wizard::SyncMode::SyncEverything:
+                            case Wizard::SyncMode::UseVfs: {
+                                bool useVfs = syncMode == Wizard::SyncMode::UseVfs;
+                                setUpInitialSyncFolder(accountStatePtr, useVfs);
+
+                                break;
+                            }
+                            case Wizard::SyncMode::ConfigureUsingFolderWizard: {
+                                Q_ASSERT(!accountStatePtr->account()->hasDefaultSyncRoot());
+
+                                auto *folderWizard = new FolderWizard(accountStatePtr, ocApp()->gui()->settingsDialog());
+                                folderWizard->resize(ocApp()->gui()->settingsDialog()->sizeHintForChild());
+                                folderWizard->setAttribute(Qt::WA_DeleteOnClose);
+
+                                // TODO: duplication of AccountSettings
+                                // adapted from AccountSettings::slotFolderWizardAccepted()
+                                connect(folderWizard, &QDialog::accepted, [accountStatePtr, folderWizard]() {
+                                    FolderMan *folderMan = FolderMan::instance();
+
+                                    qCInfo(lcApplication) << "Folder wizard completed";
+                                    const auto config = folderWizard->result();
+
+                                    auto folder = folderMan->addFolderFromFolderWizardResult(accountStatePtr, config);
+
+                                    if (!config.selectiveSyncBlackList.isEmpty() && OC_ENSURE(folder && !config.useVirtualFiles)) {
+                                        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, config.selectiveSyncBlackList);
+
+                                        // The user already accepted the selective sync dialog. everything is in the white list
+                                        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, { QLatin1String("/") });
+                                    }
+
+                                    folderMan->setSyncEnabled(true);
+                                    folderMan->scheduleAllFolders();
+                                });
+
+                                connect(folderWizard, &QDialog::rejected, []() {
+                                    qCInfo(lcApplication) << "Folder wizard cancelled";
+                                    FolderMan::instance()->setSyncEnabled(true);
+                                });
+
+                                folderWizard->open();
+                                ocApp()->gui()->raiseDialog(folderWizard);
+
+                                break;
+                            }
+                            default:
+                                Q_UNREACHABLE();
+                            }
+                        }
+                    });
+
+
+                    validator->checkServer();
+                } else {
+                    FolderMan::instance()->setSyncEnabled(true);
+                }
+
+                // make sure the wizard is cleaned up eventually
+                _wizardController->deleteLater();
+            });
+
+        // all we have to do is show the dialog...
+        _wizardController->window()->show();
+        // ... and bring it to the front
+        raiseDialog(_wizardController->window());
     }
-    raiseDialog(settingsDialog());
 }
 
 void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
 {
-    QList<AccountState *> accounts;
+    QList<AccountStatePtr> accounts;
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
-        accounts.append(account.data());
+        accounts.append(account);
     } else {
-        foreach (auto a, AccountManager::instance()->accounts()) {
-            accounts.append(a.data());
+        for (const auto &a : AccountManager::instance()->accounts()) {
+            accounts.append(a);
         }
     }
-    foreach (Folder *f, FolderMan::instance()->map()) {
+    for (auto *f : FolderMan::instance()->folders()) {
         if (accounts.contains(f->accountState())) {
             f->setSyncPaused(pause);
             if (pause) {
@@ -984,7 +1120,7 @@ void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
 
 void ownCloudGui::slotShowGuiMessage(const QString &title, const QString &message)
 {
-    QMessageBox *msgBox = new QMessageBox;
+    QMessageBox *msgBox = new QMessageBox(settingsDialog());
     msgBox->setWindowFlags(msgBox->windowFlags() | Qt::WindowStaysOnTopHint);
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
     msgBox->setText(message);
@@ -1039,6 +1175,7 @@ void ownCloudGui::raiseDialog(QWidget *raiseWidget)
 {
     auto window = ocApp()->gui()->settingsDialog();
     OC_ASSERT(window);
+    OC_ASSERT_X(!qobject_cast<QDialog *>(raiseWidget) || raiseWidget->parentWidget() == window, "raiseDialog should only be called with modal dialogs");
     if (!window) {
         return;
     }
@@ -1076,47 +1213,52 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
         qCWarning(lcApplication) << "Could not open share dialog for" << localPath << "no responsible folder found";
         return;
     }
-
-    const auto accountState = folder->accountState();
-
-    SyncJournalFileRecord fileRecord;
-
-    bool resharingAllowed = true; // lets assume the good
-    if (folder->journalDb()->getFileRecord(file, &fileRecord) && fileRecord.isValid()) {
-        // check the permission: Is resharing allowed?
-        if (!fileRecord._remotePerm.isNull() && !fileRecord._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
-            resharingAllowed = false;
-        }
-    }
-
-    // As a first approximation, set the set of permissions that can be granted
-    // either to everything (resharing allowed) or nothing (no resharing).
-    //
-    // The correct value will be found with a propfind from ShareDialog.
-    // (we want to show the dialog directly, not wait for the propfind first)
-    SharePermissions maxSharingPermissions =
-        SharePermissionRead
-        | SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete
-        | SharePermissionShare;
-    if (!resharingAllowed) {
-        maxSharingPermissions = SharePermission(0);
-    }
-
-
-    ShareDialog *w = nullptr;
-    if (_shareDialogs.contains(localPath) && _shareDialogs[localPath]) {
-        qCInfo(lcApplication) << "Raising share dialog" << sharePath << localPath;
-        w = _shareDialogs[localPath];
+    if (folder->accountState()->account()->capabilities().filesSharing().sharing_roles) {
+        fetchPrivateLinkUrl(folder->accountState()->account(), folder->webDavUrl(), sharePath, this, [](const QUrl &url) {
+            Utility::openBrowser(url, nullptr);
+        });
     } else {
-        qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
-        w = new ShareDialog(accountState, sharePath, localPath, maxSharingPermissions, fileRecord.legacyDeriveNumericFileId(), startPage, settingsDialog());
-        w->setAttribute(Qt::WA_DeleteOnClose, true);
+        const auto accountState = folder->accountState();
 
-        _shareDialogs[localPath] = w;
-        connect(w, &QObject::destroyed, this, &ownCloudGui::slotRemoveDestroyedShareDialogs);
+        SyncJournalFileRecord fileRecord;
+
+        bool resharingAllowed = true; // lets assume the good
+        if (folder->journalDb()->getFileRecord(file, &fileRecord) && fileRecord.isValid()) {
+            // check the permission: Is resharing allowed?
+            if (!fileRecord._remotePerm.isNull() && !fileRecord._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
+                resharingAllowed = false;
+            }
+        }
+
+        // As a first approximation, set the set of permissions that can be granted
+        // either to everything (resharing allowed) or nothing (no resharing).
+        //
+        // The correct value will be found with a propfind from ShareDialog.
+        // (we want to show the dialog directly, not wait for the propfind first)
+        SharePermissions maxSharingPermissions =
+            SharePermissionRead
+            | SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete
+            | SharePermissionShare;
+        if (!resharingAllowed) {
+            maxSharingPermissions = SharePermission(0);
+        }
+
+
+        ShareDialog *w = nullptr;
+        if (_shareDialogs.contains(localPath) && _shareDialogs[localPath]) {
+            qCInfo(lcApplication) << "Raising share dialog" << sharePath << localPath;
+            w = _shareDialogs[localPath];
+        } else {
+            qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
+            w = new ShareDialog(accountState, folder->webDavUrl(), sharePath, localPath, maxSharingPermissions, startPage, settingsDialog());
+            w->setAttribute(Qt::WA_DeleteOnClose, true);
+
+            _shareDialogs[localPath] = w;
+            connect(w, &QObject::destroyed, this, &ownCloudGui::slotRemoveDestroyedShareDialogs);
+        }
+        w->open();
+        raiseDialog(w);
     }
-    w->open();
-    raiseDialog(w);
 }
 
 void ownCloudGui::slotRemoveDestroyedShareDialogs()
