@@ -26,6 +26,8 @@
 #include "protocolitem.h"
 #include "ui_protocolwidget.h"
 
+#include "models/models.h"
+
 class QPushButton;
 class QSortFilterProxyModel;
 
@@ -49,17 +51,18 @@ public:
     ~ProtocolWidget() override;
 
     static void showContextMenu(QWidget *parent, ProtocolItemModel *model, const QModelIndexList &items);
-    static QMenu *showFilterMenu(QWidget *parent, QSortFilterProxyModel *model);
+    static QMenu *showFilterMenu(QWidget *parent, Models::SignalledQSortFilterProxyModel *model, int role, const QString &columnName);
 
 public slots:
-    void slotItemCompleted(const QString &folder, const SyncFileItemPtr &item);
+    void slotItemCompleted(Folder *folder, const SyncFileItemPtr &item);
+    void filterDidChange();
 
 private slots:
     void slotItemContextMenu();
 
 private:
     ProtocolItemModel *_model;
-    QSortFilterProxyModel *_sortModel;
+    Models::SignalledQSortFilterProxyModel *_sortModel;
     Ui::ProtocolWidget *_ui;
 };
 }
