@@ -206,8 +206,7 @@ void ShareLinkWidget::getShares()
 
 void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shares)
 {
-    const QString versionString = _account->serverVersion();
-    qCInfo(lcSharing) << versionString << "Fetched" << shares.count() << "shares";
+    qCInfo(lcSharing) << "Fetched" << shares.count() << "shares";
 
     // Select the share that was previously selected,
     // except if an explicit override was asked for
@@ -230,7 +229,7 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
     auto deleteIcon = QIcon::fromTheme(QStringLiteral("user-trash"),
         Utility::getCoreIcon(QStringLiteral("delete")));
 
-    foreach (auto share, shares) {
+    for (const auto &share : shares) {
         if (share->getShareType() != Share::TypeLink) {
             continue;
         }
@@ -258,7 +257,7 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
         table->setItem(row, 0, nameItem);
 
         auto dotdotdotButton = new QToolButton;
-        dotdotdotButton->setText("...");
+        dotdotdotButton->setText(QStringLiteral("..."));
         dotdotdotButton->setProperty(propertyShareC, QVariant::fromValue(linkShare));
         connect(dotdotdotButton, &QAbstractButton::clicked, this, &ShareLinkWidget::slotContextMenuButtonClicked);
         table->setCellWidget(row, 1, dotdotdotButton);
@@ -363,14 +362,13 @@ void ShareLinkWidget::slotShareSelectionChanged()
             _ui->lineEdit_password->setEnabled(false);
         }
     } else if (!selectionUnchanged) {
+        // we used to disable the checkbox (and thus the line edit) when setting a password failed
+        // now, we leave it enabled but clear the password in any case
+        // only if setting was successful, we want to show a placeholder afterwards to signalize success
         if (share && share->isPasswordSet()) {
             _ui->checkBox_password->setChecked(true);
-            _ui->lineEdit_password->setPlaceholderText("********");
+            _ui->lineEdit_password->setPlaceholderText(QStringLiteral("********"));
             _ui->lineEdit_password->setEnabled(true);
-        } else {
-            _ui->checkBox_password->setChecked(false);
-            _ui->lineEdit_password->setPlaceholderText(QString());
-            _ui->lineEdit_password->setEnabled(false);
         }
         _ui->lineEdit_password->setText(QString());
         _ui->pushButton_setPassword->setEnabled(false);
@@ -452,6 +450,11 @@ void ShareLinkWidget::slotNameEdited(QTableWidgetItem *item)
     auto share = item->data(Qt::UserRole).value<QSharedPointer<LinkShare>>();
     if (share && newName != share->getName() && newName != share->getToken()) {
         share->setName(newName);
+
+        // the server doesn't necessarily apply the desired name but may assign a different one
+        // for instance, when the user removes a custom name by emptying the field, the server assigns the ID as name
+        // therefore, we need to fetch the name(s) from the server again to display accurate information
+        getShares();
     }
 }
 
@@ -478,7 +481,7 @@ void ShareLinkWidget::slotPasswordSet()
     _ui->checkBox_password->setEnabled(true);
     _ui->lineEdit_password->setText(QString());
     if (share->isPasswordSet()) {
-        _ui->lineEdit_password->setPlaceholderText("********");
+        _ui->lineEdit_password->setPlaceholderText(QStringLiteral("********"));
         _ui->lineEdit_password->setEnabled(true);
     } else {
         _ui->lineEdit_password->setPlaceholderText(QString());
@@ -569,7 +572,7 @@ void ShareLinkWidget::slotCheckBoxExpireClicked()
 
 void ShareLinkWidget::emailShareLink(const QUrl &url)
 {
-    QString fileName = _sharePath.mid(_sharePath.lastIndexOf('/') + 1);
+    QString fileName = _sharePath.mid(_sharePath.lastIndexOf(QLatin1Char('/')) + 1);
     Utility::openEmailComposer(
         tr("I shared %1 with you").arg(fileName),
         url.toString(),
