@@ -64,7 +64,7 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
     , _disableCompleterActivated(false)
 {
     setAttribute(Qt::WA_DeleteOnClose);
-    setObjectName("SharingDialogUG"); // required as group for saveGeometry call
+    setObjectName(QStringLiteral("SharingDialogUG")); // required as group for saveGeometry call
 
     _ui->setupUi(this);
 
@@ -73,7 +73,7 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
 
     _completer = new QCompleter(this);
     _completerModel = new ShareeModel(_account,
-        _isFile ? QLatin1String("file") : QLatin1String("folder"),
+        _isFile ? QStringLiteral("file") : QStringLiteral("folder"),
         _completer);
     connect(_completerModel, &ShareeModel::shareesReady, this, &ShareUserGroupWidget::slotShareesReady);
     connect(_completerModel, &ShareeModel::displayErrorMessage, this, &ShareUserGroupWidget::displayError);
@@ -90,6 +90,10 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
     connect(_ui->shareeLineEdit, &QLineEdit::returnPressed, this, &ShareUserGroupWidget::slotLineEditReturn);
     connect(_ui->privateLinkText, &QLabel::linkActivated, this, &ShareUserGroupWidget::slotPrivateLinkShare);
 
+    if (account->capabilities().privateLinkPropertyAvailable()) {
+        _ui->privateLinkText->setText(tr("You can direct people to this shared file or folder %1 by giving them a private link")
+                                          .arg(QStringLiteral("<a href=\"%1\"><span style=\"text-decoration: underline\">").arg(_privateLinkUrl)));
+    }
     // By making the next two QueuedConnections we can override
     // the strings the completer sets on the line edit.
     connect(_completer, qOverload<const QModelIndex &>(&QCompleter::activated), this, &ShareUserGroupWidget::slotCompleterActivated,
@@ -161,7 +165,7 @@ void ShareUserGroupWidget::searchForSharees()
     ShareeModel::ShareeSet blacklist;
 
     // Add the current user to _sharees since we can't share with ourself
-    QSharedPointer<Sharee> currentUser(new Sharee(_account->credentials()->user(), "", Sharee::Type::User));
+    QSharedPointer<Sharee> currentUser(new Sharee(_account->credentials()->user(), QString(), Sharee::Type::User));
     blacklist << currentUser;
 
     const auto &shareUserLines = _ui->scrollArea->findChildren<ShareUserLine *>();
@@ -291,20 +295,8 @@ void ShareUserGroupWidget::slotCompleterActivated(const QModelIndex &index)
      * https://github.com/owncloud/client/issues/4996
      */
     const SharePermissions defaultPermissions = static_cast<SharePermissions>(_account->capabilities().defaultPermissions());
-    if (sharee->type() == Sharee::Federated
-        && _account->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        SharePermissions permissions = SharePermissionRead | SharePermissionUpdate;
-        if (!_isFile) {
-            permissions |= SharePermissionCreate | SharePermissionDelete;
-        }
-        permissions &= defaultPermissions;
-        _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), permissions);
-    } else {
-        _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), _maxSharingPermissions & defaultPermissions);
-    }
-
+    _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
+        sharee->shareWith(), _maxSharingPermissions & defaultPermissions);
     _ui->shareeLineEdit->setEnabled(false);
     _ui->shareeLineEdit->setText(QString());
 }
@@ -331,7 +323,7 @@ void ShareUserGroupWidget::displayError(int code, const QString &message)
 
 void ShareUserGroupWidget::slotPrivateLinkOpenBrowser()
 {
-    Utility::openBrowser(_privateLinkUrl, this);
+    Utility::openBrowser(QUrl(_privateLinkUrl), this);
 }
 
 void ShareUserGroupWidget::slotPrivateLinkCopy()
@@ -402,17 +394,6 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
     connect(_ui->permissionShare, &QAbstractButton::clicked, this, &ShareUserLine::slotPermissionsChanged);
     connect(_ui->permissionsEdit, &QAbstractButton::clicked, this, &ShareUserLine::slotEditPermissionsChanged);
 
-    /*
-     * We don't show permssion share for federated shares with server <9.1
-     * https://github.com/owncloud/core/issues/22122#issuecomment-185637344
-     * https://github.com/owncloud/client/issues/4996
-     */
-    if (share->getShareType() == Share::TypeRemote
-        && share->account()->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        _ui->permissionShare->setVisible(false);
-        _ui->permissionToolButton->setVisible(false);
-    }
-
     connect(share.data(), &Share::permissionsSet, this, &ShareUserLine::slotPermissionsSet);
     connect(share.data(), &Share::shareDeleted, this, &ShareUserLine::slotShareDeleted);
 
@@ -446,14 +427,15 @@ void ShareUserLine::loadAvatar()
 
     // See core/js/placeholder.js for details on colors and styling
     const QColor bg = QColor::fromHslF(hue, 0.7, 0.68);
-    const QString style = QString(R"(* {
+    const QString style = QStringLiteral(R"(* {
         color: #fff;
         background-color: %1;
         border-radius: %2px;
         text-align: center;
         line-height: %2px;
         font-size: %2px;
-    })").arg(bg.name(), QString::number(avatarSize / 2));
+    })")
+                              .arg(bg.name(), QString::number(avatarSize / 2));
     _ui->avatar->setStyleSheet(style);
 
     // The avatar label is the first character of the user name.
@@ -465,9 +447,13 @@ void ShareUserLine::loadAvatar()
      * Currently only regular users can have avatars.
      */
     if (_share->getShareWith()->type() == Sharee::User) {
-        AvatarJob *job = new AvatarJob(_share->account(), _share->getShareWith()->shareWith(), avatarSize, this);
-        connect(job, &AvatarJob::avatarPixmap, this, &ShareUserLine::slotAvatarLoaded);
-        job->start();
+        auto account = _share->account();
+        auto capabilities = account->capabilities();
+        if (capabilities.isValid() && capabilities.avatarsAvailable()) {
+            AvatarJob *job = new AvatarJob(account, _share->getShareWith()->shareWith(), avatarSize, this);
+            connect(job, &AvatarJob::avatarPixmap, this, &ShareUserLine::slotAvatarLoaded);
+            job->start();
+        }
     }
 }
 
@@ -480,7 +466,7 @@ void ShareUserLine::slotAvatarLoaded(const QPixmap &avatar)
     _ui->avatar->setPixmap(_avatar);
 
     // Remove the stylesheet for the fallback avatar
-    _ui->avatar->setStyleSheet("");
+    _ui->avatar->setStyleSheet(QString());
 }
 
 void ShareUserLine::on_deleteShareButton_clicked()
