@@ -57,9 +57,11 @@ public:
     ~SocketApi() override;
 
 public slots:
+    void registerAccount(const AccountPtr &a);
+    void unregisterAccount(const AccountPtr &a);
     void slotUpdateFolderView(Folder *f);
-    void slotUnregisterPath(const QString &alias);
-    void slotRegisterPath(const QString &alias);
+    void slotUnregisterPath(Folder *f);
+    void slotRegisterPath(Folder *f);
     void broadcastStatusPushMessage(const QString &systemPath, SyncFileStatus fileStatus);
 
 signals:
@@ -69,9 +71,9 @@ private slots:
     void slotNewConnection();
     void slotReadSocket();
 
-    static void copyUrlToClipboard(const QString &link);
-    static void emailPrivateLink(const QString &link);
-    static void openPrivateLink(const QString &link);
+    static void copyUrlToClipboard(const QUrl &link);
+    static void emailPrivateLink(const QUrl &link);
+    static void openPrivateLink(const QUrl &link);
 
 private:
     // Helper structure for getting information on a file
@@ -94,6 +96,8 @@ private:
         QString folderRelativePath;
         // Path of the file on the server (In case of virtual file, it points to the actual file)
         QString serverRelativePath;
+
+        bool isSyncFolder() const;
     };
 
     void broadcastMessage(const QString &msg, bool doWait = false);
@@ -121,12 +125,19 @@ private:
     Q_INVOKABLE void command_DELETE_ITEM(const QString &localFile, SocketListener *listener);
     Q_INVOKABLE void command_MOVE_ITEM(const QString &localFile, SocketListener *listener);
 
+    Q_INVOKABLE void command_OPEN_APP_LINK(const QString &localFile, SocketListener *listener);
     // External sync
     Q_INVOKABLE void command_V2_LIST_ACCOUNTS(const QSharedPointer<SocketApiJobV2> &job) const;
-    Q_INVOKABLE void command_V2_UPLOAD_FILES_FROM(const QSharedPointer<SocketApiJobV2> &job) const;
+
+    // Sends the id and the client icon as PNG image (base64 encoded) in Json key "png"
+    // e.g. { "id" : "1", "arguments" : { "png" : "hswehs343dj8..." } } or an error message in key "error"
+    //
+    // Argument is a SocketApiJobV2 job which contains an id and the required icon size in Json format
+    // e.g. { "id" : "1", "arguments" : { "size" : 16 } }
+    Q_INVOKABLE void command_V2_GET_CLIENT_ICON(const QSharedPointer<SocketApiJobV2> &job) const;
 
     // Fetch the private link and call targetFun
-    void fetchPrivateLinkUrlHelper(const QString &localFile, const std::function<void(const QString &url)> &targetFun);
+    void fetchPrivateLinkUrlHelper(const QString &localFile, const std::function<void(const QUrl &url)> &targetFun);
 
     /** Sends translated/branded strings that may be useful to the integration */
     Q_INVOKABLE void command_GET_STRINGS(const QString &argument, SocketListener *listener);
@@ -143,19 +154,10 @@ private:
      */
     Q_INVOKABLE void command_GET_MENU_ITEMS(const QString &argument, SocketListener *listener);
 
-#if GUI_TESTING
-    Q_INVOKABLE void command_ASYNC_ASSERT_ICON_IS_EQUAL(const QSharedPointer<SocketApiJob> &job);
-    Q_INVOKABLE void command_ASYNC_LIST_WIDGETS(const QSharedPointer<SocketApiJob> &job);
-    Q_INVOKABLE void command_ASYNC_INVOKE_WIDGET_METHOD(const QSharedPointer<SocketApiJob> &job);
-    Q_INVOKABLE void command_ASYNC_GET_WIDGET_PROPERTY(const QSharedPointer<SocketApiJob> &job);
-    Q_INVOKABLE void command_ASYNC_SET_WIDGET_PROPERTY(const QSharedPointer<SocketApiJob> &job);
-    Q_INVOKABLE void command_ASYNC_WAIT_FOR_WIDGET_SIGNAL(const QSharedPointer<SocketApiJob> &job);
-    Q_INVOKABLE void command_ASYNC_TRIGGER_MENU_ACTION(const QSharedPointer<SocketApiJob> &job);
-#endif
-
     QString buildRegisterPathMessage(const QString &path);
 
-    QSet<QString> _registeredAliases;
+    QSet<Folder *> _registeredFolders;
+    QSet<AccountPtr> _registeredAccounts;
     QMap<SocketApiSocket *, QSharedPointer<SocketListener>> _listeners;
     SocketApiServer _localServer;
 };
