@@ -17,6 +17,8 @@
 #include <QVariantMap>
 #include <QDebug>
 
+using namespace std::chrono;
+
 namespace OCC {
 
 
@@ -25,9 +27,18 @@ Capabilities::Capabilities(const QVariantMap &capabilities)
     , _fileSharingCapabilities(_capabilities.value(QStringLiteral("files_sharing")).toMap())
     , _fileSharingPublicCapabilities(_fileSharingCapabilities.value(QStringLiteral("public"), {}).toMap())
     , _tusSupport(_capabilities.value(QStringLiteral("files")).toMap().value(QStringLiteral("tus_support")).toMap())
+    , _spaces(_capabilities.value(QStringLiteral("spaces")).toMap())
+    , _status(_capabilities.value(QStringLiteral("core")).toMap().value(QStringLiteral("status")).toMap())
+    , _appProviders(AppProviders::findVersion(_capabilities.value(QStringLiteral("files")).toMap().value(QStringLiteral("app_providers")).toList(), QVersionNumber({ 1, 1, 0 })))
+    , _filesSharing(_fileSharingCapabilities)
 {
 }
 
+QVariantMap Capabilities::raw() const
+{
+    return _capabilities;
+}
+
 bool Capabilities::shareAPI() const
 {
     // This was later added so if it is not present just assume the API is enabled.
@@ -105,7 +116,7 @@ int Capabilities::defaultPermissions() const
 
 std::chrono::seconds Capabilities::remotePollInterval() const
 {
-    return std::chrono::duration_cast<std::chrono::seconds>(std::chrono::milliseconds(_capabilities.value(QStringLiteral("core")).toMap().value(QStringLiteral("pollinterval")).toInt()));
+    return duration_cast<seconds>(milliseconds(_capabilities.value(QStringLiteral("core")).toMap().value(QStringLiteral("pollinterval")).toInt()));
 }
 
 bool Capabilities::notificationsAvailable() const
@@ -119,32 +130,36 @@ bool Capabilities::isValid() const
     return !_capabilities.isEmpty();
 }
 
-QList<QByteArray> Capabilities::supportedChecksumTypes() const
+QList<CheckSums::Algorithm> Capabilities::supportedChecksumTypes() const
 {
-    QList<QByteArray> list;
+    // TODO: compute once
+    QList<CheckSums::Algorithm> list;
     const auto &supportedTypes = _capabilities.value(QStringLiteral("checksums")).toMap().value(QStringLiteral("supportedTypes")).toList();
     for (const auto &t : supportedTypes) {
-        list.push_back(t.toByteArray());
+        list.push_back(CheckSums::fromByteArray(t.toByteArray()));
     }
     return list;
 }
 
-QByteArray Capabilities::preferredUploadChecksumType() const
+CheckSums::Algorithm Capabilities::preferredUploadChecksumType() const
 {
-    return qEnvironmentVariable("OWNCLOUD_CONTENT_CHECKSUM_TYPE",
-                                _capabilities.value(QStringLiteral("checksums")).toMap()
-                                .value(QStringLiteral("preferredUploadType"), QStringLiteral("SHA1")).toString()).toUtf8();
+    static auto envType = CheckSums::fromByteArray(qgetenv("OWNCLOUD_CONTENT_CHECKSUM_TYPE"));
+    if (envType != CheckSums::Algorithm::NONE && envType != CheckSums::Algorithm::PARSE_ERROR) {
+        return envType;
+    }
+    const auto val = _capabilities.value(QStringLiteral("checksums")).toMap().value(QStringLiteral("preferredUploadType"), QStringLiteral("SHA1")).toString().toUpper().toUtf8();
+    return CheckSums::fromByteArray(val);
 }
 
-QByteArray Capabilities::uploadChecksumType() const
+CheckSums::Algorithm Capabilities::uploadChecksumType() const
 {
-    QByteArray preferred = preferredUploadChecksumType();
-    if (!preferred.isEmpty())
+    auto preferred = preferredUploadChecksumType();
+    if (preferred != CheckSums::Algorithm::PARSE_ERROR)
         return preferred;
-    QList<QByteArray> supported = supportedChecksumTypes();
+    QList<CheckSums::Algorithm> supported = supportedChecksumTypes();
     if (!supported.isEmpty())
         return supported.first();
-    return QByteArray();
+    return CheckSums::Algorithm::PARSE_ERROR;
 }
 
 bool Capabilities::chunkingNg() const
@@ -172,11 +187,21 @@ bool Capabilities::bigfilechunkingEnabled() const
     return _capabilities.value(QStringLiteral("files")).toMap().value(QStringLiteral("bigfilechunking"), true).toBool();
 }
 
+const Status &Capabilities::status() const
+{
+    return _status;
+}
+
 const TusSupport &Capabilities::tusSupport() const
 {
     return _tusSupport;
 }
 
+const SpaceSupport &Capabilities::spacesSupport() const
+{
+    return _spaces;
+}
+
 bool Capabilities::chunkingParallelUploadDisabled() const
 {
     return _capabilities.value(QStringLiteral("dav")).toMap().value(QStringLiteral("chunkingParallelUploadDisabled")).toBool();
@@ -222,11 +247,38 @@ bool Capabilities::versioningEnabled() const
     return _capabilities.value(QStringLiteral("files")).toMap().value(QStringLiteral("versioning")).toBool();
 }
 
+bool Capabilities::avatarsAvailable() const
+{
+    auto userCaps = _fileSharingCapabilities.value(QStringLiteral("user")).toMap();
+    // true by default for older servers, because turning off profile pictures was introduced in later versions
+    return userCaps.value(QStringLiteral("profile_picture"), true).toBool();
+}
+
 QStringList Capabilities::blacklistedFiles() const
 {
     return _capabilities.value(QStringLiteral("files")).toMap().value(QStringLiteral("blacklisted_files")).toStringList();
 }
 
+Status::Status(const QVariantMap &status)
+{
+    legacyVersion = QVersionNumber::fromString(status.value(QStringLiteral("version")).toString());
+    legacyVersionString = status.value(QStringLiteral("versionstring")).toString();
+    edition = status.value(QStringLiteral("edition")).toString();
+    productname = status.value(QStringLiteral("productname")).toString();
+    product = status.value(QStringLiteral("product")).toString();
+    productversion = status.value(QStringLiteral("productversion")).toString();
+}
+
+QVersionNumber Status::version() const
+{
+    return productversion.isEmpty() ? legacyVersion : QVersionNumber::fromString(productversion);
+}
+
+QString Status::versionString() const
+{
+    return productversion.isEmpty() ? legacyVersionString : productversion;
+}
+
 TusSupport::TusSupport(const QVariantMap &tus_support)
 {
     if (tus_support.isEmpty() || qEnvironmentVariableIsSet("OWNCLOUD_NO_TUS")) {
@@ -234,7 +286,8 @@ TusSupport::TusSupport(const QVariantMap &tus_support)
     }
     version = QVersionNumber::fromString(tus_support.value(QStringLiteral("version")).toString());
     resumable = QVersionNumber::fromString(tus_support.value(QStringLiteral("resumable")).toString());
-    extensions = tus_support.value(QStringLiteral("extension")).toString().split(QLatin1Char(','), QString::SkipEmptyParts);
+
+    extensions = tus_support.value(QStringLiteral("extension")).toString().split(QLatin1Char(','), Qt::SkipEmptyParts);
     max_chunk_size = tus_support.value(QStringLiteral("max_chunk_size")).value<quint64>();
     http_method_override = tus_support.value(QStringLiteral("http_method_override")).toString();
 }
@@ -243,4 +296,53 @@ bool TusSupport::isValid() const
 {
     return !version.isNull();
 }
+
+SpaceSupport::SpaceSupport(const QVariantMap &spaces_support)
+{
+    if (spaces_support.isEmpty()) {
+        return;
+    }
+    enabled = spaces_support.value(QStringLiteral("enabled")).toBool();
+    version = QVersionNumber::fromString(spaces_support.value(QStringLiteral("version")).toString());
 }
+
+bool SpaceSupport::isValid() const
+{
+    return !version.isNull();
+}
+
+Capabilities::AppProviders::AppProviders(const QVariantMap &appProviders)
+    : enabled(appProviders.value(QStringLiteral("enabled")).toBool())
+    , version(QVersionNumber::fromString(appProviders.value(QStringLiteral("version")).toString()))
+    , appsUrl(appProviders.value(QStringLiteral("apps_url")).toString())
+    , openUrl(appProviders.value(QStringLiteral("open_url")).toString())
+    , newUrl(appProviders.value(QStringLiteral("new_url")).toString())
+    , openWebUrl(appProviders.value(QStringLiteral("open_web_url")).toString())
+{
+}
+
+const Capabilities::AppProviders &Capabilities::appProviders() const
+{
+    return _appProviders;
+}
+
+Capabilities::AppProviders Capabilities::AppProviders::findVersion(const QVariantList &list, const QVersionNumber &v)
+{
+    auto it = std::find_if(list.cbegin(), list.cend(), [&v](const auto &it) {
+        return QVersionNumber::fromString(it.toMap().value(QStringLiteral("version")).toString()) == v;
+    });
+    return it != list.cend() ? Capabilities::AppProviders { it->toMap() }
+                             : Capabilities::AppProviders();
+}
+
+
+const FilesSharing &Capabilities::filesSharing() const
+{
+    return _filesSharing;
+}
+
+FilesSharing::FilesSharing(const QVariantMap &filesSharing)
+    : sharing_roles(filesSharing.value(QStringLiteral("sharing_roles"), false).toBool())
+{
+}
+} // namespace OCC
