@@ -18,14 +18,44 @@
 
 #include "owncloudlib.h"
 
+#include "common/checksumalgorithms.h"
+
 #include <QVariantMap>
 #include <QStringList>
 #include <QVersionNumber>
 
 namespace OCC {
 
+struct OWNCLOUDSYNC_EXPORT Status
+{
+    /**
+     <installed>1</installed>
+    <maintenance>0</maintenance>
+    <needsDbUpgrade>0</needsDbUpgrade>
+    <version>10.11.0.0</version>
+    <versionstring>10.11.0</versionstring>
+    <edition>Community</edition>
+    <productname>Infinite Scale</productname>
+    <product>Infinite Scale</product>
+    <productversion>2.0.0-beta1+7c2e3201b</productversion>
+    */
 
-struct TusSupport
+    Status(const QVariantMap &status);
+    // legacy version
+    QVersionNumber legacyVersion;
+    // legacy version
+    QString legacyVersionString;
+
+    QString edition;
+    QString productname;
+    QString product;
+    QString productversion;
+
+    QVersionNumber version() const;
+    QString versionString() const;
+};
+
+struct OWNCLOUDSYNC_EXPORT TusSupport
 {
     /**
     <tus_support>
@@ -46,6 +76,40 @@ struct TusSupport
     bool isValid() const;
 };
 
+struct OWNCLOUDSYNC_EXPORT SpaceSupport
+{
+    /**
+        "spaces": {
+          "version": "0.0.1",
+          "enabled": true
+        }
+    */
+    SpaceSupport(const QVariantMap &spaces_support);
+    bool enabled = false;
+    QVersionNumber version;
+
+    bool isValid() const;
+};
+
+struct OWNCLOUDSYNC_EXPORT FilesSharing
+{
+    /**
+     api_enabled": true,
+    "resharing": true,
+    "group_sharing": true,
+    "sharing_roles": true,
+    "auto_accept_share": true,
+    "share_with_group_members_only": true,
+    "share_with_membership_groups_only": true,
+    "search_min_length": 3,
+    "default_permissions": 22,
+    */
+    FilesSharing(const QVariantMap &filesSharing);
+
+    // TODO: add more
+    bool sharing_roles = false;
+};
+
 /**
  * @brief The Capabilities class represents the capabilities of an ownCloud
  * server
@@ -54,6 +118,31 @@ struct TusSupport
 class OWNCLOUDSYNC_EXPORT Capabilities
 {
 public:
+    struct OWNCLOUDSYNC_EXPORT AppProviders
+    {
+        /**
+         "app_providers": [
+                {
+                  "enabled": true,
+                  "version": "1.0.0",
+                  "apps_url": "/app/list",
+                  "open_url": "/app/open",
+                  "new_url": "/app/new"
+                }
+        */
+        AppProviders() = default;
+        static AppProviders findVersion(const QVariantList &list, const QVersionNumber &v);
+        AppProviders(const QVariantMap &appProviders);
+
+        bool enabled = false;
+        QVersionNumber version;
+        QString appsUrl;
+        QString openUrl;
+        QString newUrl;
+        QString openWebUrl;
+    };
+
+
     Capabilities(const QVariantMap &capabilities);
 
     bool shareAPI() const;
@@ -90,7 +179,9 @@ public:
     /// Wheter to use chunking
     bool bigfilechunkingEnabled() const;
 
+    const Status &status() const;
     const TusSupport &tusSupport() const;
+    const SpaceSupport &spacesSupport() const;
 
     /// disable parallel upload in chunking
     bool chunkingParallelUploadDisabled() const;
@@ -118,24 +209,21 @@ public:
      * Default: []
      * Possible entries: "Adler32", "MD5", "SHA1"
      */
-    QList<QByteArray> supportedChecksumTypes() const;
+    QList<OCC::CheckSums::Algorithm> supportedChecksumTypes() const;
 
     /**
      * The checksum algorithm that the server recommends for file uploads.
      * This is just a preference, any algorithm listed in supportedTypes may be used.
      *
      * Path: checksums/preferredUploadType
-     * Default: empty, meaning "no preference"
-     * Possible values: empty or any of the supportedTypes
      */
-    QByteArray preferredUploadChecksumType() const;
+    OCC::CheckSums::Algorithm preferredUploadChecksumType() const;
 
     /**
      * Helper that returns the preferredUploadChecksumType() if set, or one
-     * of the supportedChecksumTypes() if it isn't. May return an empty
-     * QByteArray if no checksum types are supported.
+     * of the supportedChecksumTypes() if it isn't.
      */
-    QByteArray uploadChecksumType() const;
+    CheckSums::Algorithm uploadChecksumType() const;
 
     /**
      * List of HTTP error codes should be guaranteed to eventually reset
@@ -180,11 +268,25 @@ public:
     /** Is versioning available? */
     bool versioningEnabled() const;
 
+    /** Are avatars (profile pictures) available? */
+    bool avatarsAvailable() const;
+
+    const AppProviders &appProviders() const;
+
+    const FilesSharing &filesSharing() const;
+
+
+    QVariantMap raw() const;
+
 private:
     QVariantMap _capabilities;
     QVariantMap _fileSharingCapabilities;
     QVariantMap _fileSharingPublicCapabilities;
     TusSupport _tusSupport;
+    SpaceSupport _spaces;
+    Status _status;
+    AppProviders _appProviders;
+    FilesSharing _filesSharing;
 };
 }
 
