@@ -44,6 +44,8 @@
 #include <QTimerEvent>
 #include <qmath.h>
 
+using namespace std::chrono_literals;
+
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcPropagator, "sync.propagator", QtInfoMsg)
@@ -82,9 +84,7 @@ OwncloudPropagator::~OwncloudPropagator()
 
 int OwncloudPropagator::maximumActiveTransferJob()
 {
-    if (_downloadLimit != 0
-        || _uploadLimit != 0
-        || !_syncOptions._parallelNetworkJobs) {
+    if (_bandwidthManager || !_syncOptions._parallelNetworkJobs) {
         // disable parallelism when there is a network limit.
         return 1;
     }
@@ -150,7 +150,7 @@ static SyncJournalErrorBlacklistRecord createBlacklistEntry(
     entry._file = item._file;
     entry._errorString = item._errorString;
     entry._lastTryModtime = item._modtime;
-    entry._lastTryEtag = item._etag;
+    entry._lastTryEtag = item._etag.toUtf8();
     entry._lastTryTime = Utility::qDateTimeToTime_t(QDateTime::currentDateTimeUtc());
     entry._renameTarget = item._renameTarget;
     entry._retryCount = old._retryCount + 1;
@@ -258,6 +258,7 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
     case SyncFileItem::FatalError:
     case SyncFileItem::NormalError:
     case SyncFileItem::DetailError:
+    case SyncFileItem::Message:
         // Check the blacklist, possibly adjusting the item (including its status)
         blacklistUpdate(propagator()->_journal, *_item);
         break;
@@ -516,12 +517,6 @@ const SyncOptions &OwncloudPropagator::syncOptions() const
     return _syncOptions;
 }
 
-void OwncloudPropagator::setSyncOptions(const SyncOptions &syncOptions)
-{
-    _syncOptions = syncOptions;
-    _chunkSize = syncOptions._initialChunkSize;
-}
-
 Result<QString, bool> OwncloudPropagator::localFileNameClash(const QString &relFile)
 {
     OC_ASSERT(!relFile.isEmpty());
@@ -611,7 +606,7 @@ void OwncloudPropagator::scheduleNextJob()
 {
     if (_jobScheduled) return; // don't schedule more than 1
     _jobScheduled = true;
-    QTimer::singleShot(3, this, &OwncloudPropagator::scheduleNextJobImpl);
+    QTimer::singleShot(0, this, &OwncloudPropagator::scheduleNextJobImpl);
 }
 
 void OwncloudPropagator::scheduleNextJobImpl()
@@ -652,6 +647,29 @@ void OwncloudPropagator::reportFileTotal(const SyncFileItem &item, qint64 newSiz
     emit updateFileTotal(item, newSize);
 }
 
+void OwncloudPropagator::abort()
+{
+    if (_abortRequested)
+        return;
+    if (_rootJob) {
+        // Connect to abortFinished  which signals that abort has been asynchronously finished
+        connect(_rootJob.data(), &PropagateDirectory::abortFinished, this, &OwncloudPropagator::emitFinished);
+
+        // Use Queued Connection because we're possibly already in an item's finished stack
+        QMetaObject::invokeMethod(
+            _rootJob.data(), [this] {
+                _rootJob->abort(PropagatorJob::AbortType::Asynchronous);
+            },
+            Qt::QueuedConnection);
+
+        // Give asynchronous abort 5 sec to finish on its own
+        QTimer::singleShot(5s, this, &OwncloudPropagator::abortTimeout);
+    } else {
+        // No root job, call emitFinished
+        emitFinished(SyncFileItem::NormalError);
+    }
+}
+
 void OwncloudPropagator::reportProgress(const SyncFileItem &item, qint64 bytes)
 {
     emit progress(item, bytes);
@@ -1181,13 +1199,9 @@ void OCC::PropagateUpdateMetaDataJob::start()
 
     const QString filePath = propagator()->fullLocalPath(_item->destination());
     if (_item->_direction == SyncFileItem::Down) {
-        // If the 'W' remote permission changed, update the local filesystem
         SyncJournalFileRecord prev;
         if (propagator()->_journal->getFileRecord(_item->_file, &prev)
-            && prev.isValid()
-            && prev._remotePerm.hasPermission(RemotePermissions::CanWrite) != _item->_remotePerm.hasPermission(RemotePermissions::CanWrite)) {
-            const bool isReadOnly = !_item->_remotePerm.isNull() && !_item->_remotePerm.hasPermission(RemotePermissions::CanWrite);
-            FileSystem::setFileReadOnlyWeak(filePath, isReadOnly);
+            && prev.isValid()) {
             if (_item->_checksumHeader.isEmpty()) {
                 _item->_checksumHeader = prev._checksumHeader;
             }
