@@ -18,14 +18,13 @@
 #include <QHash>
 #include <QObject>
 #include <QMap>
-#include <QLinkedList>
 #include <QElapsedTimer>
 #include <QTimer>
 #include <QPointer>
 #include <QIODevice>
 #include <QMutex>
 
-#include "csync_util.h"
+#include "csync.h"
 #include "syncfileitem.h"
 #include "common/syncjournaldb.h"
 #include "bandwidthmanager.h"
@@ -70,12 +69,15 @@ public:
         Synchronous,
         Asynchronous
     };
+    Q_ENUM(AbortType)
 
     enum JobState {
         NotYetStarted,
         Running,
         Finished
     };
+    Q_ENUM(JobState)
+
     JobState _state;
 
     enum JobParallelism {
@@ -88,6 +90,7 @@ public:
             are executed. */
         WaitForFinished,
     };
+    Q_ENUM(JobParallelism)
 
     virtual JobParallelism parallelism() { return FullParallelism; }
 
@@ -158,25 +161,8 @@ class PropagateItemJob : public PropagatorJob
 protected:
     virtual void done(SyncFileItem::Status status, const QString &errorString = QString());
 
-    /*
-     * set a custom restore job message that is used if the restore job succeeded.
-     * It is displayed in the activity view.
-     */
-    QString restoreJobMsg() const
-    {
-        return _item->_isRestoration ? _item->_errorString : QString();
-    }
-    void setRestoreJobMsg(const QString &msg = QString())
-    {
-        _item->_isRestoration = true;
-        _item->_errorString = msg;
-    }
-
-protected slots:
-    void slotRestoreJobFinished(SyncFileItem::Status status);
-
-private:
-    QScopedPointer<PropagateItemJob> _restoreJob;
+    SyncFileItemPtr _item;
+    friend class PropagateDirectory;
 
 public:
     PropagateItemJob(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
@@ -185,21 +171,7 @@ public:
     {
     }
     ~PropagateItemJob() override;
-
-    bool scheduleSelfOrChild() override
-    {
-        if (_state != NotYetStarted) {
-            return false;
-        }
-        qCInfo(lcPropagator) << "Starting" << _item->_instruction << "propagation of" << _item->destination() << "by" << this;
-
-        _state = Running;
-        QMetaObject::invokeMethod(this, "start"); // We could be in a different thread (neon jobs)
-        return true;
-    }
-
-    SyncFileItemPtr _item;
-
+    bool scheduleSelfOrChild() override;
 public slots:
     virtual void start() = 0;
 };
@@ -213,7 +185,7 @@ class PropagatorCompositeJob : public PropagatorJob
     Q_OBJECT
 public:
     QVector<PropagatorJob *> _jobsToDo;
-    SyncFileItemVector _tasksToDo;
+    SyncFileItemSet _tasksToDo;
     QVector<PropagatorJob *> _runningJobs;
     SyncFileItem::Status _hasError; // NoStatus,  or NormalError / SoftError if there was an error
     quint64 _abortsCount;
@@ -234,7 +206,7 @@ public:
     void appendJob(PropagatorJob *job);
     void appendTask(const SyncFileItemPtr &item)
     {
-        _tasksToDo.append(item);
+        _tasksToDo.insert(item);
     }
 
     bool scheduleSelfOrChild() override;
@@ -249,7 +221,7 @@ public:
     {
         if (!_runningJobs.empty()) {
             _abortsCount = _runningJobs.size();
-            foreach (PropagatorJob *j, _runningJobs) {
+            for (auto *j : qAsConst(_runningJobs)) {
                 if (abortType == AbortType::Asynchronous) {
                     connect(j, &PropagatorJob::abortFinished,
                             this, &PropagatorCompositeJob::slotSubJobAbortFinished);
@@ -281,11 +253,10 @@ private slots:
  * @brief Propagate a directory, and all its sub entries.
  * @ingroup libsync
  */
-class OWNCLOUDSYNC_EXPORT PropagateDirectory : public PropagatorJob
+class OWNCLOUDSYNC_EXPORT PropagateDirectory : public PropagateItemJob
 {
     Q_OBJECT
 public:
-    SyncFileItemPtr _item;
     // e.g: create the directory
     QScopedPointer<PropagateItemJob> _firstJob;
 
@@ -323,17 +294,20 @@ public:
         _firstJob->_item->_affectedItems++;
     }
 
-
     qint64 committedDiskSpace() const override
     {
         return _subJobs.committedDiskSpace();
     }
 
+    SyncFileItemPtr &item()
+    {
+        return _item;
+    }
+
 private slots:
-
+    void start() override {};
     void slotFirstJobFinished(SyncFileItem::Status status);
-    virtual void slotSubJobsFinished(SyncFileItem::Status status);
-
+    virtual void slotSubJobsFinished(const SyncFileItem::Status status);
 };
 
 /**
@@ -397,30 +371,28 @@ public:
     bool _finishedEmited; // used to ensure that finished is only emitted once
 
 public:
-    OwncloudPropagator(AccountPtr account, const QString &localDir,
+    OwncloudPropagator(AccountPtr account, const SyncOptions &options, const QUrl &baseUrl, const QString &localDir,
         const QString &remoteFolder, SyncJournalDb *progressDb)
-        : _localDir((localDir.endsWith(QLatin1Char('/'))) ? localDir : localDir + QLatin1Char('/'))
-        , _remoteFolder((remoteFolder.endsWith(QLatin1Char('/'))) ? remoteFolder : remoteFolder + QLatin1Char('/'))
-        , _journal(progressDb)
+        : _journal(progressDb)
         , _finishedEmited(false)
-        , _bandwidthManager(this)
         , _anotherSyncNeeded(false)
-        , _chunkSize(10 * 1000 * 1000) // 10 MB, overridden in setSyncOptions
+        , _chunkSize(options._initialChunkSize)
         , _account(account)
+        , _syncOptions(options)
+        , _localDir((localDir.endsWith(QLatin1Char('/'))) ? localDir : localDir + QLatin1Char('/'))
+        , _remoteFolder((remoteFolder.endsWith(QLatin1Char('/'))) ? remoteFolder : remoteFolder + QLatin1Char('/'))
+        , _webDavUrl(baseUrl)
     {
         qRegisterMetaType<PropagatorJob::AbortType>("PropagatorJob::AbortType");
     }
 
     ~OwncloudPropagator() override;
 
-    void start(SyncFileItemVector &&_syncedItems);
+    void start(SyncFileItemSet &&_syncedItems);
 
     const SyncOptions &syncOptions() const;
-    void setSyncOptions(const SyncOptions &syncOptions);
 
-    int _downloadLimit = 0;
-    int _uploadLimit = 0;
-    BandwidthManager _bandwidthManager;
+    QPointer<BandwidthManager> _bandwidthManager;
 
     bool _abortRequested = false;
 
@@ -465,8 +437,9 @@ public:
 
     /** Check whether a download would clash with an existing file
      * in filesystems that are only case-preserving.
+     * Returns the path of the clashed file
      */
-    bool localFileNameClash(const QString &relfile);
+    Result<QString, bool> localFileNameClash(const QString &relfile);
 
     /** Check whether a file is properly accessible for upload.
      *
@@ -497,33 +470,21 @@ public:
     void reportProgress(const SyncFileItem &, qint64 bytes);
     void reportFileTotal(const SyncFileItem &item, qint64 newSize);
 
-    void abort()
-    {
-        if (_abortRequested)
-            return;
-        if (_rootJob) {
-            // Connect to abortFinished  which signals that abort has been asynchronously finished
-            connect(_rootJob.data(), &PropagateDirectory::abortFinished, this, &OwncloudPropagator::emitFinished);
-
-            // Use Queued Connection because we're possibly already in an item's finished stack
-            QMetaObject::invokeMethod(_rootJob.data(), "abort", Qt::QueuedConnection,
-                                      Q_ARG(PropagatorJob::AbortType, PropagatorJob::AbortType::Asynchronous));
-
-            // Give asynchronous abort 5000 msec to finish on its own
-            QTimer::singleShot(5000, this, SLOT(abortTimeout()));
-        } else {
-            // No root job, call emitFinished
-            emitFinished(SyncFileItem::NormalError);
-        }
-    }
+    void abort();
 
     AccountPtr account() const;
 
+    QUrl webDavUrl() const
+    {
+        return _webDavUrl;
+    }
+
     enum DiskSpaceResult {
         DiskSpaceOk,
         DiskSpaceFailure,
         DiskSpaceCritical
     };
+    Q_ENUM(DiskSpaceResult)
 
     /** Checks whether there's enough disk space available to complete
      *  all jobs that are currently running.
@@ -543,7 +504,8 @@ public:
         PropagatorCompositeJob *composite, QString *error);
 
     // Map original path (as in the DB) to target final path
-    QMap<QString, QString> _renamedDirectories;
+    // TODO: no public members...
+    QHash<QString, QString> _renamedDirectories;
     QString adjustRenamedPath(const QString &original) const;
 
     /** Update the database for an item.
@@ -551,11 +513,16 @@ public:
      * Typically after a sync operation succeeded. Updates the inode from
      * the filesystem.
      *
-     * Will also trigger a Vfs::convertToPlaceholder.
+     * Will also trigger updatePlaceholder.
      */
-    static bool updateMetadata(const SyncFileItem &item, const QString &localFolderPath, SyncJournalDb &journal, Vfs &vfs);
-    bool updateMetadata(const SyncFileItem &item); // convenience for the above
+    Result<Vfs::ConvertToPlaceholderResult, QString> updateMetadata(const SyncFileItem &item);
 
+
+    /** Update the the placeholder and takes over some metadata from replacesFile
+     *
+     * Will also trigger a Vfs::updateMetadata.
+     */
+    Result<Vfs::ConvertToPlaceholderResult, QString> updatePlaceholder(const SyncFileItem &item, const QString &fileName, const QString &replacesFile);
 private slots:
 
     void abortTimeout()
@@ -583,7 +550,7 @@ signals:
     void finished(bool success);
 
     /** Emitted when propagation has problems with a locked file. */
-    void seenLockedFile(const QString &fileName);
+    void seenLockedFile(const QString &fileName, FileSystem::LockMode mode);
 
     /** Emitted when propagation touches a file.
      *
@@ -603,46 +570,18 @@ private:
 
     const QString _localDir; // absolute path to the local directory. ends with '/'
     const QString _remoteFolder; // remote folder, ends with '/'
+    const QUrl _webDavUrl; // full webdav url, might be the same as in the account
 };
 
-
 /**
- * @brief Job that wait for all the poll jobs to be completed
- * @ingroup libsync
+ * @brief Update MetaData (Permissions and etag) of files
  */
-class CleanupPollsJob : public QObject
+class PropagateUpdateMetaDataJob : public PropagateItemJob
 {
     Q_OBJECT
-    QVector<SyncJournalDb::PollInfo> _pollInfos;
-    AccountPtr _account;
-    SyncJournalDb *_journal;
-    QString _localPath;
-    QSharedPointer<Vfs> _vfs;
-
 public:
-    explicit CleanupPollsJob(const QVector<SyncJournalDb::PollInfo> &pollInfos, AccountPtr account,
-        SyncJournalDb *journal, const QString &localPath, const QSharedPointer<Vfs> &vfs, QObject *parent = nullptr)
-        : QObject(parent)
-        , _pollInfos(pollInfos)
-        , _account(account)
-        , _journal(journal)
-        , _localPath(localPath)
-        , _vfs(vfs)
-    {
-    }
-
-    ~CleanupPollsJob() override;
-
-    /**
-     * Start the job.  After the job is completed, it will emit either finished or aborted, and it
-     * will destroy itself.
-     */
-    void start();
-signals:
-    void finished();
-    void aborted(const QString &error);
-private slots:
-    void slotPollFinished();
+    PropagateUpdateMetaDataJob(OwncloudPropagator *propagator, const SyncFileItemPtr &item);
+    void start() override;
 };
 }
 
