@@ -26,16 +26,20 @@
 #include "common/asserts.h"
 #include "common/vfs.h"
 
+#include <QDir>
+#include <QFileInfo>
 #include <QLoggingCategory>
 #include <QNetworkAccessManager>
-#include <QFileInfo>
-#include <QDir>
+#include <QRandomGenerator>
+
 #include <cmath>
 
 #ifdef Q_OS_UNIX
 #include <unistd.h>
 #endif
 
+using namespace std::chrono_literals;
+
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcGetJob, "sync.networkjob.get", QtInfoMsg)
@@ -48,52 +52,47 @@ QString OWNCLOUDSYNC_EXPORT createDownloadTmpFileName(const QString &previous)
 {
     QString tmpFileName;
     QString tmpPath;
-    int slashPos = previous.lastIndexOf(QLatin1Char('/'));
+    const int slashPos = previous.lastIndexOf(QLatin1Char('/'));
     // work with both pathed filenames and only filenames
     if (slashPos == -1) {
         tmpFileName = previous;
-        tmpPath = QString();
     } else {
         tmpFileName = previous.mid(slashPos + 1);
         tmpPath = previous.left(slashPos);
     }
-    int overhead = 1 + 1 + 2 + 8; // slash dot dot-tilde ffffffff"
-    int spaceForFileName = qMin(254, tmpFileName.length() + overhead) - overhead;
+
+    auto rg = QRandomGenerator::global();
+    const int overhead = 1 + 1 + 2 + 8; // slash dot dot-tilde ffffffff"
+    const int spaceForFileName = qMin(254, tmpFileName.length() + overhead) - overhead;
     if (tmpPath.length() > 0) {
-        return QStringLiteral("%1/.%2.~%3").arg(tmpPath, tmpFileName.left(spaceForFileName), QString::number(uint(qrand() % 0xFFFFFFFF), 16));
+        return QStringLiteral("%1/.%2.~%3").arg(tmpPath, tmpFileName.left(spaceForFileName), QString::number(uint(rg->generate() % 0xFFFFFFFF), 16));
     } else {
-        return QStringLiteral(".%1.~%2").arg(tmpFileName.left(spaceForFileName), QString::number(uint(qrand() % 0xFFFFFFFF), 16));
+        return QStringLiteral(".%1.~%2").arg(tmpFileName.left(spaceForFileName), QString::number(uint(rg->generate() % 0xFFFFFFFF), 16));
     }
 }
 
 // DOES NOT take ownership of the device.
-GETFileJob::GETFileJob(AccountPtr account, const QString &path, QIODevice *device,
-    const QMap<QByteArray, QByteArray> &headers, const QByteArray &expectedEtagForResume,
+GETFileJob::GETFileJob(AccountPtr account, const QUrl &url, const QString &path, QIODevice *device,
+    const QMap<QByteArray, QByteArray> &headers, const QString &expectedEtagForResume,
     qint64 resumeStart, QObject *parent)
-    : GETJob(account, path, parent)
+    : AbstractNetworkJob(account, url, path, parent)
     , _device(device)
     , _headers(headers)
     , _expectedEtagForResume(expectedEtagForResume)
     , _expectedContentLength(-1)
     , _contentLength(-1)
     , _resumeStart(resumeStart)
-    , _hasEmittedFinishedSignal(false)
 {
-}
+    connect(this, &GETFileJob::networkError, this, [this] {
+        if (timedOut()) {
+            qCWarning(lcGetJob) << this << "timeout";
+            _errorString = tr("Connection Timeout");
+            _errorStatus = SyncFileItem::FatalError;
+        }
+    });
 
-GETFileJob::GETFileJob(AccountPtr account, const QUrl &url, QIODevice *device,
-    const QMap<QByteArray, QByteArray> &headers, const QByteArray &expectedEtagForResume,
-    qint64 resumeStart, QObject *parent)
-    : GETJob(account, url.toString(QUrl::FullyEncoded), parent)
-    , _device(device)
-    , _headers(headers)
-    , _expectedEtagForResume(expectedEtagForResume)
-    , _expectedContentLength(-1)
-    , _contentLength(-1)
-    , _resumeStart(resumeStart)
-    , _directDownloadUrl(url)
-    , _hasEmittedFinishedSignal(false)
-{
+    // Long downloads must not block non-propagation jobs.
+    setPriority(QNetworkRequest::LowPriority);
 }
 
 void GETFileJob::start()
@@ -109,31 +108,32 @@ void GETFileJob::start()
         req.setRawHeader(it.key(), it.value());
     }
 
-    req.setPriority(QNetworkRequest::LowPriority); // Long downloads must not block non-propagation jobs.
-
-    if (_directDownloadUrl.isEmpty()) {
-        sendRequest("GET", makeDavUrl(path()), req);
-    } else {
-        // Use direct URL
-        sendRequest("GET", _directDownloadUrl, req);
-    }
+    sendRequest("GET", req);
 
     qCDebug(lcGetJob) << _bandwidthManager << _bandwidthChoked << _bandwidthLimited;
     if (_bandwidthManager) {
         _bandwidthManager->registerDownloadJob(this);
     }
-
-    connect(this, &AbstractNetworkJob::networkActivity, account().data(), &Account::propagatorNetworkActivity);
-
     AbstractNetworkJob::start();
 }
 
+void GETFileJob::finished()
+{
+    if (_bandwidthManager) {
+        _bandwidthManager->unregisterDownloadJob(this);
+    }
+    if (reply()->bytesAvailable() && _httpOk) {
+        // we were throttled, write out the remaining data
+        slotReadyRead();
+        Q_ASSERT(!reply()->bytesAvailable());
+    }
+}
+
 void GETFileJob::newReplyHook(QNetworkReply *reply)
 {
     reply->setReadBufferSize(16 * 1024); // keep low so we can easier limit the bandwidth
 
     connect(reply, &QNetworkReply::metaDataChanged, this, &GETFileJob::slotMetaDataChanged);
-    connect(reply, &QIODevice::readyRead, this, &GETFileJob::slotReadyRead);
     connect(reply, &QNetworkReply::finished, this, &GETFileJob::slotReadyRead);
     connect(reply, &QNetworkReply::downloadProgress, this, &GETFileJob::downloadProgress);
 }
@@ -151,9 +151,8 @@ void GETFileJob::slotMetaDataChanged()
         // Redirects and auth failures (oauth token renew) are handled by AbstractNetworkJob and
         // will end up restarting the job. We do not want to process further data from the initial
         // request. newReplyHook() will reestablish signal connections for the follow-up request.
-        bool ok = disconnect(reply(), &QNetworkReply::finished, this, &GETFileJob::slotReadyRead)
-            && disconnect(reply(), &QNetworkReply::readyRead, this, &GETFileJob::slotReadyRead);
-        OC_ASSERT(ok);
+        disconnect(reply(), &QNetworkReply::finished, this, &GETFileJob::slotReadyRead);
+        disconnect(reply(), &QNetworkReply::readyRead, this, &GETFileJob::slotReadyRead);
         return;
     }
 
@@ -170,23 +169,18 @@ void GETFileJob::slotMetaDataChanged()
     }
     _etag = getEtagFromReply(reply());
 
-    if (!_directDownloadUrl.isEmpty() && !_etag.isEmpty()) {
-        qCInfo(lcGetJob) << "Direct download used, ignoring server ETag" << _etag;
-        _etag = QByteArray(); // reset received ETag
-    } else if (!_directDownloadUrl.isEmpty()) {
-        // All fine, ETag empty and directDownloadUrl used
-    } else if (_etag.isEmpty()) {
+    if (_etag.isEmpty()) {
         qCWarning(lcGetJob) << "No E-Tag reply by server, considering it invalid";
         _errorString = tr("No E-Tag received from server, check Proxy/Gateway");
         _errorStatus = SyncFileItem::NormalError;
-        reply()->abort();
+        abort();
         return;
     } else if (!_expectedEtagForResume.isEmpty() && _expectedEtagForResume != _etag) {
         qCWarning(lcGetJob) << "We received a different E-Tag for resuming!"
                             << _expectedEtagForResume << "vs" << _etag;
         _errorString = tr("We received a different E-Tag for resuming. Retrying next time.");
         _errorStatus = SyncFileItem::NormalError;
-        reply()->abort();
+        abort();
         return;
     }
 
@@ -197,16 +191,17 @@ void GETFileJob::slotMetaDataChanged()
                             << _expectedContentLength << "vs" << _contentLength;
         _errorString = tr("We received an unexpected download Content-Length.");
         _errorStatus = SyncFileItem::NormalError;
-        reply()->abort();
+        abort();
         return;
     }
 
     qint64 start = 0;
-    QString ranges = QString::fromUtf8(reply()->rawHeader("Content-Range"));
+    const QString ranges = QString::fromUtf8(reply()->rawHeader("Content-Range"));
     if (!ranges.isEmpty()) {
-        QRegExp rx(QStringLiteral("bytes (\\d+)-"));
-        if (rx.indexIn(ranges) >= 0) {
-            start = rx.cap(1).toLongLong();
+        static QRegularExpression rx(QStringLiteral("bytes (\\d+)-"));
+        const auto match = rx.match(ranges);
+        if (match.hasMatch()) {
+            start = match.captured(1).toLongLong();
         }
     }
     if (start != _resumeStart) {
@@ -217,14 +212,14 @@ void GETFileJob::slotMetaDataChanged()
             if (!_device->open(QIODevice::WriteOnly)) {
                 _errorString = _device->errorString();
                 _errorStatus = SyncFileItem::NormalError;
-                reply()->abort();
+                abort();
                 return;
             }
             _resumeStart = 0;
         } else {
             _errorString = tr("Server returned wrong content-range");
             _errorStatus = SyncFileItem::NormalError;
-            reply()->abort();
+            abort();
             return;
         }
     }
@@ -233,32 +228,36 @@ void GETFileJob::slotMetaDataChanged()
     if (!lastModified.isNull()) {
         _lastModified = Utility::qDateTimeToTime_t(lastModified.toDateTime());
     }
-
-    _saveBodyToFile = true;
+    _httpOk = true;
+    connect(reply(), &QIODevice::readyRead, this, &GETFileJob::slotReadyRead);
 }
 
-void GETJob::setBandwidthManager(BandwidthManager *bwm)
+void GETFileJob::setBandwidthManager(BandwidthManager *bwm)
 {
     _bandwidthManager = bwm;
 }
 
-void GETJob::setChoked(bool c)
+void GETFileJob::setChoked(bool c)
 {
-    _bandwidthChoked = c;
-    QMetaObject::invokeMethod(this, "slotReadyRead", Qt::QueuedConnection);
+    if (c != _bandwidthChoked) {
+        _bandwidthChoked = c;
+        QMetaObject::invokeMethod(this, &GETFileJob::slotReadyRead, Qt::QueuedConnection);
+    }
 }
 
-void GETJob::setBandwidthLimited(bool b)
+void GETFileJob::setBandwidthLimited(bool b)
 {
-    _bandwidthLimited = b;
-    QMetaObject::invokeMethod(this, "slotReadyRead", Qt::QueuedConnection);
+    if (_bandwidthLimited != b) {
+        _bandwidthLimited = b;
+        QMetaObject::invokeMethod(this, &GETFileJob::slotReadyRead, Qt::QueuedConnection);
+    }
 }
 
-void GETJob::giveBandwidthQuota(qint64 q)
+void GETFileJob::giveBandwidthQuota(qint64 q)
 {
     _bandwidthQuota = q;
     qCDebug(lcGetJob) << "Got" << q << "bytes";
-    QMetaObject::invokeMethod(this, "slotReadyRead", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &GETFileJob::slotReadyRead, Qt::QueuedConnection);
 }
 
 qint64 GETFileJob::currentDownloadPosition()
@@ -271,73 +270,58 @@ qint64 GETFileJob::currentDownloadPosition()
 
 void GETFileJob::slotReadyRead()
 {
-    if (!reply())
+    Q_ASSERT(reply());
+    if (!_httpOk) {
         return;
-    int bufferSize = qMin(1024 * 8ll, reply()->bytesAvailable());
+    }
+
+    const qint64 bufferSize = std::min<qint64>(1024 * 8, reply()->bytesAvailable());
     QByteArray buffer(bufferSize, Qt::Uninitialized);
 
-    while (reply()->bytesAvailable() > 0 && _saveBodyToFile) {
+    while (reply()->bytesAvailable() > 0) {
         if (_bandwidthChoked) {
             qCWarning(lcGetJob) << "Download choked";
             break;
         }
         qint64 toRead = bufferSize;
         if (_bandwidthLimited) {
-            toRead = qMin(qint64(bufferSize), _bandwidthQuota);
+            toRead = std::min<qint64>(bufferSize, _bandwidthQuota);
             if (toRead == 0) {
-                qCWarning(lcGetJob) << "Out of quota";
+                qCWarning(lcGetJob) << "Out of badnwidth quota";
                 break;
             }
             _bandwidthQuota -= toRead;
         }
 
-        qint64 r = reply()->read(buffer.data(), toRead);
-        if (r < 0) {
+        const qint64 read = reply()->read(buffer.data(), toRead);
+        if (read < 0) {
             _errorString = networkReplyErrorString(*reply());
             _errorStatus = SyncFileItem::NormalError;
             qCWarning(lcGetJob) << "Error while reading from device: " << _errorString;
-            reply()->abort();
+            abort();
             return;
         }
 
-        qint64 w = _device->write(buffer.constData(), r);
-        if (w != r) {
+        const qint64 written = _device->write(buffer.constData(), read);
+        if (written != read) {
             _errorString = _device->errorString();
             _errorStatus = SyncFileItem::NormalError;
-            qCWarning(lcGetJob) << "Error while writing to file" << w << r << _errorString;
-            reply()->abort();
+            qCWarning(lcGetJob) << "Error while writing to file" << written << read << _errorString;
+            abort();
             return;
         }
     }
-
-    if (reply()->isFinished() && (reply()->bytesAvailable() == 0 || !_saveBodyToFile)) {
-        qCDebug(lcGetJob) << "Actually finished!";
-        if (_bandwidthManager) {
-            _bandwidthManager->unregisterDownloadJob(this);
-        }
-        if (!_hasEmittedFinishedSignal) {
-            qCInfo(lcGetJob) << "GET of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
-                             << replyStatusString()
-                             << reply()->rawHeader("Content-Range") << reply()->rawHeader("Content-Length");
-
-            emit finishedSignal();
-        }
-        _hasEmittedFinishedSignal = true;
-        deleteLater();
-    }
 }
 
-void GETJob::onTimedOut()
+
+GETFileJob::~GETFileJob()
 {
-    qCWarning(lcGetJob) << this << "timeout";
-    if (reply()) {
-        _errorString = tr("Connection Timeout");
-        _errorStatus = SyncFileItem::FatalError;
+    if (_bandwidthManager) {
+        _bandwidthManager->unregisterDownloadJob(this);
     }
-    AbstractNetworkJob::onTimedOut();
 }
 
-QString GETJob::errorString() const
+QString GETFileJob::errorString() const
 {
     if (!_errorString.isEmpty()) {
         return _errorString;
@@ -359,7 +343,7 @@ void PropagateDownloadFile::start()
     // For virtual files just dehydrate or create the file and be done
     if (_item->_type == ItemTypeVirtualFileDehydration) {
         const QString fsPath = propagator()->fullLocalPath(_item->_file);
-        if (!FileSystem::verifyFileUnchanged(fsPath, _item->_previousSize, _item->_previousModtime)) {
+        if (FileSystem::fileChanged(fsPath, _item->_previousSize, _item->_previousModtime)) {
             propagator()->_anotherSyncNeeded = true;
             done(SyncFileItem::SoftError, tr("File has changed since discovery"));
             return;
@@ -408,11 +392,17 @@ void PropagateDownloadFile::start()
     // If the hashes are collision safe and identical, we assume the content is too.
     // For weak checksums, we only do that if the mtimes are also identical.
 
-    const auto csync_is_collision_safe_hash = [](const QByteArray &checksum_header)
-    {
-        return checksum_header.startsWith("SHA")
-            || checksum_header.startsWith("MD5:");
+    const auto csync_is_collision_safe_hash = [](const QByteArray &checksum_header) {
+        const bool safe = std::any_of(CheckSums::SafeAlgorithms.begin(), CheckSums::SafeAlgorithms.end(), [checksum_header = checksum_header.toUpper()](auto &it) {
+            return checksum_header.startsWith(it.second.data());
+        });
+        if (!safe) {
+            qWarning(lcPropagateDownload) << checksum_header << "is considered unsave";
+            return false;
+        }
+        return true;
     };
+
     if (_item->_instruction == CSYNC_INSTRUCTION_CONFLICT
         && _item->_size == _item->_previousSize
         && !_item->_checksumHeader.isEmpty()
@@ -420,7 +410,8 @@ void PropagateDownloadFile::start()
             || _item->_modtime == _item->_previousModtime)) {
         qCDebug(lcPropagateDownload) << _item->_file << "may not need download, computing checksum";
         auto computeChecksum = new ComputeChecksum(this);
-        computeChecksum->setChecksumType(parseChecksumHeaderType(_item->_checksumHeader));
+        const auto checksumHeader = ChecksumHeader::parseChecksumHeader(_item->_checksumHeader);
+        computeChecksum->setChecksumType(checksumHeader.type());
         connect(computeChecksum, &ComputeChecksum::done,
             this, &PropagateDownloadFile::conflictChecksumComputed);
         propagator()->_activeJobList.append(this);
@@ -431,10 +422,11 @@ void PropagateDownloadFile::start()
     startDownload();
 }
 
-void PropagateDownloadFile::conflictChecksumComputed(const QByteArray &checksumType, const QByteArray &checksum)
+void PropagateDownloadFile::conflictChecksumComputed(CheckSums::Algorithm checksumType, const QByteArray &checksum)
 {
     propagator()->_activeJobList.removeOne(this);
-    if (makeChecksumHeader(checksumType, checksum) == _item->_checksumHeader) {
+    const auto checksumHeader = ChecksumHeader::parseChecksumHeader(_item->_checksumHeader);
+    if (checksumHeader == ChecksumHeader(checksumType, checksum)) {
         // No download necessary, just update fs and journal metadata
         qCDebug(lcPropagateDownload) << _item->_file << "remote and local checksum match";
 
@@ -476,12 +468,12 @@ void PropagateDownloadFile::startDownload()
     const SyncJournalDb::DownloadInfo progressInfo = propagator()->_journal->getDownloadInfo(_item->_file);
     if (progressInfo._valid) {
         // if the etag has changed meanwhile, remove the already downloaded part.
-        if (progressInfo._etag != _item->_etag) {
+        if (progressInfo._etag != _item->_etag.toUtf8()) {
             FileSystem::remove(propagator()->fullLocalPath(progressInfo._tmpfile));
             propagator()->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
         } else {
             tmpFileName = progressInfo._tmpfile;
-            _expectedEtagForResume = progressInfo._etag;
+            _expectedEtagForResume = QString::fromUtf8(progressInfo._etag);
         }
     }
 
@@ -534,7 +526,7 @@ void PropagateDownloadFile::startDownload()
 
     {
         SyncJournalDb::DownloadInfo pi;
-        pi._etag = _item->_etag;
+        pi._etag = _item->_etag.toUtf8();
         pi._tmpfile = tmpFileName;
         pi._valid = true;
         propagator()->_journal->setDownloadInfo(_item->_file, pi);
@@ -550,7 +542,7 @@ void PropagateDownloadFile::startFullDownload()
 
     if (_item->_directDownloadUrl.isEmpty()) {
         // Normal job, download from oC instance
-        _job = new GETFileJob(propagator()->account(),
+        _job = new GETFileJob(propagator()->account(), propagator()->webDavUrl(),
             propagator()->fullRemotePath(_item->_file),
             &_tmpFile, headers, _expectedEtagForResume, _resumeStart, this);
     } else {
@@ -564,10 +556,11 @@ void PropagateDownloadFile::startFullDownload()
         QUrl url = QUrl::fromUserInput(_item->_directDownloadUrl);
         _job = new GETFileJob(propagator()->account(),
             url,
+            {},
             &_tmpFile, headers, _expectedEtagForResume, _resumeStart, this);
     }
-    _job->setBandwidthManager(&propagator()->_bandwidthManager);
-    connect(_job.data(), &GETJob::finishedSignal, this, &PropagateDownloadFile::slotGetFinished);
+    _job->setBandwidthManager(propagator()->_bandwidthManager);
+    connect(_job.data(), &GETFileJob::finishedSignal, this, &PropagateDownloadFile::slotGetFinished);
     connect(qobject_cast<GETFileJob *>(_job.data()), &GETFileJob::downloadProgress,
         this, &PropagateDownloadFile::slotDownloadProgress);
     propagator()->_activeJobList.append(this);
@@ -592,7 +585,7 @@ void PropagateDownloadFile::slotGetFinished()
 {
     propagator()->_activeJobList.removeOne(this);
 
-    GETJob *job = _job;
+    GETFileJob *job = _job;
     OC_ASSERT(job);
 
     _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
@@ -669,7 +662,8 @@ void PropagateDownloadFile::slotGetFinished()
     if (!job->etag().isEmpty()) {
         // The etag will be empty if we used a direct download URL.
         // (If it was really empty by the server, the GETFileJob will have errored
-        _item->_etag = parseEtag(job->etag());
+        Q_ASSERT(job->etag() == Utility::normalizeEtag(job->etag()));
+        _item->_etag = job->etag();
     }
     if (job->lastModified()) {
         // It is possible that the file was modified on the server since we did the discovery phase
@@ -693,7 +687,7 @@ void PropagateDownloadFile::slotGetFinished()
     // of the compressed data. See QTBUG-73364.
     const auto contentEncoding = job->reply()->rawHeader("content-encoding").toLower();
     if ((contentEncoding == "gzip" || contentEncoding == "deflate")
-        && (job->reply()->attribute(QNetworkRequest::HTTP2WasUsedAttribute).toBool()
+        && (job->reply()->attribute(QNetworkRequest::Http2WasUsedAttribute).toBool()
             || job->reply()->attribute(QNetworkRequest::SpdyWasUsedAttribute).toBool())) {
         bodySize = 0;
         hasSizeHeader = false;
@@ -708,8 +702,8 @@ void PropagateDownloadFile::slotGetFinished()
         return;
     }
 
-    if (bodySize > 0 && bodySize != _tmpFile.size() - job->resumeStart()) {
-        qCDebug(lcPropagateDownload) << bodySize << _tmpFile.size() << job->resumeStart();
+    if (bodySize > 0 && (bodySize != (_tmpFile.size() - job->resumeStart()))) {
+        qCDebug(lcPropagateDownload) << bodySize << "!=" << (_tmpFile.size() - job->resumeStart()) << _tmpFile.size() << job->resumeStart();
         propagator()->_anotherSyncNeeded = true;
         done(SyncFileItem::SoftError, tr("The file could not be downloaded completely."));
         return;
@@ -862,15 +856,15 @@ namespace { // Anonymous namespace for the recall feature
     }
 } // end namespace
 
-void PropagateDownloadFile::transmissionChecksumValidated(const QByteArray &checksumType, const QByteArray &checksum)
+void PropagateDownloadFile::transmissionChecksumValidated(CheckSums::Algorithm checksumType, const QByteArray &checksum)
 {
-    const QByteArray theContentChecksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
+    const CheckSums::Algorithm theContentChecksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
 
     // Reuse transmission checksum as content checksum.
     //
     // We could do this more aggressively and accept both MD5 and SHA1
     // instead of insisting on the exactly correct checksum type.
-    if (theContentChecksumType == checksumType || theContentChecksumType.isEmpty()) {
+    if (theContentChecksumType == checksumType || theContentChecksumType != CheckSums::Algorithm::PARSE_ERROR) {
         return contentChecksumComputed(checksumType, checksum);
     }
 
@@ -883,9 +877,9 @@ void PropagateDownloadFile::transmissionChecksumValidated(const QByteArray &chec
     computeChecksum->start(_tmpFile.fileName());
 }
 
-void PropagateDownloadFile::contentChecksumComputed(const QByteArray &checksumType, const QByteArray &checksum)
+void PropagateDownloadFile::contentChecksumComputed(CheckSums::Algorithm checksumType, const QByteArray &checksum)
 {
-    _item->_checksumHeader = makeChecksumHeader(checksumType, checksum);
+    _item->_checksumHeader = ChecksumHeader(checksumType, checksum).makeChecksumHeader();
 
     downloadFinished();
 }
@@ -927,9 +921,6 @@ void PropagateDownloadFile::downloadFinished()
         }
     }
 
-    // Apply the remote permissions
-    FileSystem::setFileReadOnlyWeak(_tmpFile.fileName(), !_item->_remotePerm.isNull() && !_item->_remotePerm.hasPermission(RemotePermissions::CanWrite));
-
     bool isConflict = _item->_instruction == CSYNC_INSTRUCTION_CONFLICT
         && (QFileInfo(fn).isDir() || !FileSystem::fileEquals(fn, _tmpFile.fileName()));
     if (isConflict) {
@@ -948,7 +939,7 @@ void PropagateDownloadFile::downloadFinished()
         // the discovery phase and now.
         const qint64 expectedSize = _item->_previousSize;
         const time_t expectedMtime = _item->_previousModtime;
-        if (!FileSystem::verifyFileUnchanged(fn, expectedSize, expectedMtime)) {
+        if (FileSystem::fileChanged(fn, expectedSize, expectedMtime)) {
             propagator()->_anotherSyncNeeded = true;
             done(SyncFileItem::SoftError, tr("File has changed since discovery"));
             return;
@@ -966,7 +957,7 @@ void PropagateDownloadFile::downloadFinished()
     emit propagator()->touchedFile(fn);
     // The fileChanged() check is done above to generate better error messages.
     if (!FileSystem::uncheckedRenameReplace(_tmpFile.fileName(), fn, &error)) {
-        qCWarning(lcPropagateDownload) << "Rename failed:" << _tmpFile.fileName() << "=>" << fn;
+        qCWarning(lcPropagateDownload) << "Rename failed:" << _tmpFile.fileName() << "=>" << fn << "with error:" << error;
         propagator()->_anotherSyncNeeded = true;
         done(SyncFileItem::SoftError, error);
         return;
@@ -1039,9 +1030,9 @@ void PropagateDownloadFile::updateMetadata(bool isConflict)
         handleRecallFile(fn, propagator()->localPath(), *propagator()->_journal);
     }
 
-    qint64 duration = _stopwatch.elapsed();
-    if (isLikelyFinishedQuickly() && duration > 5 * 1000) {
-        qCWarning(lcPropagateDownload) << "WARNING: Unexpectedly slow connection, took" << duration << "msec for" << _item->_size - _resumeStart << "bytes for" << _item->_file;
+    const auto duration = std::chrono::milliseconds(_stopwatch.elapsed());
+    if (isLikelyFinishedQuickly() && duration > 5s) {
+        qCWarning(lcPropagateDownload) << "WARNING: Unexpectedly slow connection, took" << duration.count() << "ms for" << _item->_size - _resumeStart << "bytes for" << _item->_file;
     }
 }
 
@@ -1057,9 +1048,9 @@ void PropagateDownloadFile::slotDownloadProgress(qint64 received, qint64)
 
 void PropagateDownloadFile::abort(PropagatorJob::AbortType abortType)
 {
-    if (_job && _job->reply())
-        _job->reply()->abort();
-
+    if (_job) {
+        _job->abort();
+    }
     if (abortType == AbortType::Asynchronous) {
         emit abortFinished();
     }
