@@ -22,8 +22,11 @@
 #include <QFile>
 #include <QLoggingCategory>
 #include <QtConcurrent>
+
+using namespace std::chrono_literals;
+
 namespace {
-auto UpdateMetaDataRetyTimeOut = std::chrono::seconds(30);
+constexpr auto UpdateMetaDataRetyTimeOut = 30s;
 }
 namespace OCC {
 
@@ -42,7 +45,7 @@ void PropagateRemoteMkdir::start()
         return slotStartMkcolJob();
     }
 
-    _job = new DeleteJob(propagator()->account(),
+    _job = new DeleteJob(propagator()->account(), propagator()->webDavUrl(),
         propagator()->fullRemotePath(_item->_file),
         this);
     connect(qobject_cast<DeleteJob *>(_job), &DeleteJob::finishedSignal, this, &PropagateRemoteMkdir::slotStartMkcolJob);
@@ -56,8 +59,8 @@ void PropagateRemoteMkdir::slotStartMkcolJob()
 
     qCDebug(lcPropagateRemoteMkdir) << _item->_file;
 
-    _job = new MkColJob(propagator()->account(),
-        propagator()->fullRemotePath(_item->_file),
+    _job = new MkColJob(propagator()->account(), propagator()->webDavUrl(),
+        propagator()->fullRemotePath(_item->_file), {},
         this);
     connect(qobject_cast<MkColJob *>(_job), &MkColJob::finishedWithError, this, &PropagateRemoteMkdir::slotMkcolJobFinished);
     connect(qobject_cast<MkColJob *>(_job), &MkColJob::finishedWithoutError, this, &PropagateRemoteMkdir::slotMkcolJobFinished);
@@ -66,9 +69,9 @@ void PropagateRemoteMkdir::slotStartMkcolJob()
 
 void PropagateRemoteMkdir::abort(PropagatorJob::AbortType abortType)
 {
-    if (_job && _job->reply())
-        _job->reply()->abort();
-
+    if (_job) {
+        _job->abort();
+    }
     if (abortType == AbortType::Asynchronous) {
         emit abortFinished();
     }
@@ -111,14 +114,14 @@ void PropagateRemoteMkdir::slotMkcolJobFinished()
     _item->_fileId = _job->reply()->rawHeader("OC-FileId");
 
     propagator()->_activeJobList.append(this);
-    auto propfindJob = new PropfindJob(_job->account(), _job->path(), this);
+    auto propfindJob = new PropfindJob(_job->account(), _job->baseUrl(), _job->path(), PropfindJob::Depth::Zero, this);
     propfindJob->setProperties({"http://owncloud.org/ns:permissions"});
-    connect(propfindJob, &PropfindJob::result, this, [this](const QMap<QString, QString> &result) {
+    connect(propfindJob, &PropfindJob::directoryListingIterated, this, [this](const QString &, const QMap<QString, QString> &result) {
         propagator()->_activeJobList.removeOne(this);
         _item->_remotePerm = RemotePermissions::fromServerString(result.value(QStringLiteral("permissions")));
         success();
     });
-    connect(propfindJob, &PropfindJob::finishedWithError, this, [this]{
+    connect(propfindJob, &PropfindJob::finishedWithError, this, [this] {
         // ignore the PROPFIND error
         propagator()->_activeJobList.removeOne(this);
         done(SyncFileItem::NormalError);
@@ -130,13 +133,12 @@ void PropagateRemoteMkdir::success()
 {
     // Never save the etag on first mkdir.
     // Only fully propagated directories should have the etag set.
-    auto itemCopy = *_item;
-    itemCopy._etag.clear();
+    auto itemCopy = SyncFileItemPtr::create(*_item);
+    itemCopy->_etag.clear();
 
     // save the file id already so we can detect rename or remove
     // also convert to a placeholder for the proper behaviour of the file
-    Q_ASSERT(thread() == qApp->thread());
-    const auto result = propagator()->updateMetadata(itemCopy);
+    const auto result = propagator()->updateMetadata(*itemCopy);
     if (!result) {
         done(SyncFileItem::FatalError, tr("Error writing metadata to the database: %1").arg(result.error()));
         return;
@@ -148,51 +150,41 @@ void PropagateRemoteMkdir::success()
         // in case the file lock is active.
         // Retry the conversion for UpdateMetaDataRetyTimeOut
         // TODO: make update updateMetadata async in general
-
-        // QtConcurrent does not support moves
-        // use shared_ptr to manage the Result object
-        using resultType = decltype(result);
-        using resultTypePtr = decltype(std::shared_ptr<resultType>());
-
-        auto *watcher = new QFutureWatcher<resultTypePtr>(this);
-        connect(watcher, &QFutureWatcherBase::finished, this, [watcher, this] {
-            const auto *result = watcher->result().get();
-            if (*result) {
-                switch (result->get()) {
-                case Vfs::ConvertToPlaceholderResult::Ok:
-                    done(SyncFileItem::Success);
-                    return;
-                case Vfs::ConvertToPlaceholderResult::Locked:
-                    done(SyncFileItem::SoftError, tr("Setting file status failed due to file lock"));
-                    return;
-                case Vfs::ConvertToPlaceholderResult::Error:
-                    Q_UNREACHABLE();
-                }
-            }
-            done(SyncFileItem::FatalError, result->error());
-        });
-
-        watcher->setFuture(QtConcurrent::run([p = propagator(), itemCopy] {
-            // Try to update the meta data with a 30s timeout
-            const auto start = std::chrono::steady_clock::now();
-            while ((std::chrono::steady_clock::now() - start) < UpdateMetaDataRetyTimeOut) {
-                QThread::sleep(1);
-                auto result = std::make_shared<resultType>(p->updateMetadata(itemCopy));
-                if (*result) {
-                    if (result->get() == Vfs::ConvertToPlaceholderResult::Locked) {
-                        continue;
-                    } else {
-                        return result;
-                    }
-                } else {
-                    return result;
-                }
-            }
-            return std::make_shared<resultType>(Vfs::ConvertToPlaceholderResult::Locked);
-        }));
+        retryUpdateMetadata(std::chrono::steady_clock::now(), itemCopy);
         return;
     }
+#else
+    Q_UNUSED(UpdateMetaDataRetyTimeOut);
 #endif
     done(SyncFileItem::Success);
 }
+
+#ifdef Q_OS_WIN
+void PropagateRemoteMkdir::retryUpdateMetadata(std::chrono::steady_clock::time_point start, SyncFileItemPtr item)
+{
+    // Try to update the meta data with a 30s timeout
+    if ((std::chrono::steady_clock::now() - start) < UpdateMetaDataRetyTimeOut) {
+        auto result = propagator()->updateMetadata(*item);
+        if (result) {
+            switch (*result) {
+            case Vfs::ConvertToPlaceholderResult::Ok:
+                done(SyncFileItem::Success);
+                return;
+            case Vfs::ConvertToPlaceholderResult::Locked:
+                // retry in 1s
+                QTimer::singleShot(1s, this, [start, item, this] {
+                    retryUpdateMetadata(start, item);
+                });
+                return;
+            case Vfs::ConvertToPlaceholderResult::Error:
+                Q_UNREACHABLE();
+            }
+        } else {
+            done(SyncFileItem::FatalError, result.error());
+            return;
+        }
+    }
+    done(SyncFileItem::SoftError, tr("Setting file status failed due to file lock"));
+};
+#endif
 }
