@@ -12,7 +12,9 @@
  * for more details.
  */
 
+#include "propagateuploadtus.h"
 #include "account.h"
+#include "capabilities.h"
 #include "common/asserts.h"
 #include "common/checksums.h"
 #include "common/syncjournaldb.h"
@@ -24,22 +26,18 @@
 #include "owncloudpropagator_p.h"
 #include "propagateremotedelete.h"
 #include "propagateupload.h"
-#include "propagateuploadtus.h"
 #include "propagatorjobs.h"
 #include "syncengine.h"
 
 #include <QNetworkAccessManager>
 #include <QFileInfo>
 #include <QDir>
+
 #include <cmath>
 #include <cstring>
 #include <memory>
 
 namespace {
-QUrl uploadURL(const OCC::AccountPtr &account)
-{
-    return OCC::Utility::concatUrlPath(account->url(), QStringLiteral("remote.php/dav/files/%1/").arg(account->davUser()));
-}
 
 QByteArray uploadOffset()
 {
@@ -67,7 +65,7 @@ UploadDevice *PropagateUploadFileTUS::prepareDevice(const quint64 &chunkSize)
         abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(localFileName));
         return nullptr;
     }
-    auto device = std::make_unique<UploadDevice>(localFileName, _currentOffset, chunkSize, &propagator()->_bandwidthManager);
+    auto device = std::make_unique<UploadDevice>(localFileName, _currentOffset, chunkSize, propagator()->_bandwidthManager);
     if (!device->open(QIODevice::ReadOnly)) {
         qCWarning(lcPropagateUploadTUS) << "Could not prepare upload device: " << device->errorString();
 
@@ -83,12 +81,13 @@ SimpleNetworkJob *PropagateUploadFileTUS::makeCreationWithUploadJob(QNetworkRequ
 {
     Q_ASSERT(propagator()->account()->capabilities().tusSupport().extensions.contains(QStringLiteral("creation-with-upload")));
     // in difference to the old protocol the algrithm and the value are space seperated
-    const auto checkSum = _transmissionChecksumHeader.replace(':', ' ').toBase64();
-    qCDebug(lcPropagateUploadTUS) << "FullPath:" << propagator()->fullRemotePath(_item->_file);
-    request->setRawHeader(QByteArrayLiteral("Upload-Metadata"), "filename " + propagator()->fullRemotePath(_item->_file).toUtf8().toBase64() + ",checksum " + checkSum);
+    const auto checksumHeader = ChecksumHeader::parseChecksumHeader(_transmissionChecksumHeader);
+    const QByteArray checkSum = QByteArray(CheckSums::toQString(checksumHeader.type()).toUtf8() + ' ' + checksumHeader.checksum()).toBase64();
+    const QByteArray base64Path = propagator()->fullRemotePath(_item->_file).toUtf8().toBase64();
+    qCDebug(lcPropagateUploadTUS) << "FullPath:" << propagator()->fullRemotePath(_item->_file) << "Base64:" << base64Path;
+    request->setRawHeader(QByteArrayLiteral("Upload-Metadata"), "filename " + base64Path + ",checksum " + checkSum);
     request->setRawHeader(QByteArrayLiteral("Upload-Length"), QByteArray::number(_item->_size));
-    auto job = new SimpleNetworkJob(propagator()->account(), this);
-    job->prepareRequest("POST", uploadURL(propagator()->account()), *request, device);
+    auto job = new SimpleNetworkJob(propagator()->account(), propagator()->webDavUrl(), {}, "POST", device, *request, this);
     return job;
 }
 
@@ -126,7 +125,7 @@ void PropagateUploadFileTUS::startNextChunk()
     const quint64 chunkSize = [&] {
         auto chunkSize = _item->_size - _currentOffset;
         if (propagator()->account()->capabilities().tusSupport().max_chunk_size) {
-            chunkSize = qMin(chunkSize - _currentOffset, propagator()->account()->capabilities().tusSupport().max_chunk_size);
+            chunkSize = std::min<qint64>(chunkSize, propagator()->account()->capabilities().tusSupport().max_chunk_size);
         }
         return chunkSize;
     }();
@@ -140,24 +139,25 @@ void PropagateUploadFileTUS::startNextChunk()
     SimpleNetworkJob *job;
     if (_currentOffset != 0) {
         qCDebug(lcPropagateUploadTUS) << "Starting to patch upload:" << propagator()->fullRemotePath(_item->_file);
-        job = new SimpleNetworkJob(propagator()->account(), this);
-        job->prepareRequest("PATCH", _location, req, device);
+        job = new SimpleNetworkJob(propagator()->account(), _location, {}, "PATCH", device, req, this);
     } else {
         OC_ASSERT(_location.isEmpty());
         qCDebug(lcPropagateUploadTUS) << "Starting creation with upload:" << propagator()->fullRemotePath(_item->_file);
         job = makeCreationWithUploadJob(&req, device);
     }
+
+    job->setPriority(QNetworkRequest::LowPriority);
     qCDebug(lcPropagateUploadTUS) << "Offset:" << _currentOffset << _currentOffset  / (_item->_size + 1) * 100
                                   << "Chunk:" << chunkSize << chunkSize / (_item->_size + 1) * 100;
 
-    _jobs.append(job);
-    connect(job->reply(), &QNetworkReply::uploadProgress, device, &UploadDevice::slotJobUploadProgress);
-    connect(job->reply(), &QNetworkReply::uploadProgress, this,[this](qint64 bytesSent, qint64){
-       propagator()->reportProgress(*_item, _currentOffset + bytesSent);
-
-    });
+    addChildJob(job);
     connect(job, &SimpleNetworkJob::finishedSignal, this, &PropagateUploadFileTUS::slotChunkFinished);
-    connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
+    job->addNewReplyHook([device, this](QNetworkReply *reply) {
+        connect(reply, &QNetworkReply::uploadProgress, device, &UploadDevice::slotJobUploadProgress);
+        connect(reply, &QNetworkReply::uploadProgress, this, [this](qint64 bytesSent, qint64) {
+            propagator()->reportProgress(*_item, _currentOffset + bytesSent);
+        });
+    });
     job->start();
 }
 
@@ -165,7 +165,6 @@ void PropagateUploadFileTUS::slotChunkFinished()
 {
     SimpleNetworkJob *job = qobject_cast<SimpleNetworkJob *>(sender());
     OC_ASSERT(job);
-    slotJobDestroyed(job); // remove it from the _jobs list
     qCDebug(lcPropagateUploadTUS) << propagator()->fullRemotePath(_item->_file) << HttpLogger::requestVerb(*job->reply());
 
     _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
@@ -180,11 +179,9 @@ void PropagateUploadFileTUS::slotChunkFinished()
             qCWarning(lcPropagateUploadTUS) << propagator()->fullRemotePath(_item->_file) << "Encountered a timeout -> get progrss for" << _location;
             QNetworkRequest req;
             setTusVersionHeader(req);
-            auto updateJob = new SimpleNetworkJob(propagator()->account(), this);
-            updateJob->prepareRequest("HEAD", _location, req);
-            _jobs.append(updateJob);
+            auto updateJob = new SimpleNetworkJob(propagator()->account(), propagator()->webDavUrl(), _location.path(), "HEAD", {}, req, this);
+            addChildJob(updateJob);
             connect(updateJob, &SimpleNetworkJob::finishedSignal, this, &PropagateUploadFileTUS::slotChunkFinished);
-            connect(updateJob, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
             updateJob->start();
             return;
 
@@ -216,10 +213,10 @@ void PropagateUploadFileTUS::slotChunkFinished()
     }
 
     // Check whether the file changed since discovery.
-    if (!FileSystem::verifyFileUnchanged(fullFilePath, _item->_size, _item->_modtime)) {
+    if (FileSystem::fileChanged(fullFilePath, _item->_size, _item->_modtime)) {
         propagator()->_anotherSyncNeeded = true;
         if (!_finished) {
-            abortWithError(SyncFileItem::SoftError, tr("Local file changed during sync."));
+            abortWithError(SyncFileItem::Message, fileChangedMessage());
             // FIXME:  the legacy code was retrying for a few seconds.
             //         and also checking that after the last chunk, and removed the file in case of INSTRUCTION_NEW
             return;
@@ -229,28 +226,35 @@ void PropagateUploadFileTUS::slotChunkFinished()
         startNextChunk();
         return;
     }
-    const QByteArray etag = getEtagFromReply(job->reply());
 
-    _finished = !etag.isEmpty();
+    // ==== handling when the upload is finished:
+    const QString etag = getEtagFromReply(job->reply());
+    const QByteArray remPerms = job->reply()->rawHeader("OC-Perm");
+    if (!remPerms.isEmpty()) {
+        _item->_remotePerm = RemotePermissions::fromServerString(QString::fromUtf8(remPerms));
+    }
+
+    _finished = !(etag.isEmpty() || _item->_remotePerm.isNull());
     if (!_finished) {
-        auto check = new PropfindJob(propagator()->account(), propagator()->fullRemotePath(_item->_file));
-        _jobs.append(check);
+        // Either the ETag or the remote Permissions were not in the headers of the reply.
+        // Start a PROPFIND to fetch these data from the server.
+        auto check = new PropfindJob(propagator()->account(), propagator()->webDavUrl(), propagator()->fullRemotePath(_item->_file), PropfindJob::Depth::Zero, this);
+        addChildJob(check);
         check->setProperties({ "http://owncloud.org/ns:fileid", "http://owncloud.org/ns:permissions", "getetag" });
-        connect(check, &PropfindJob::result, this, [this, check](const QMap<QString, QString> &map) {
+        connect(check, &PropfindJob::directoryListingIterated, this, [this](const QString &, const QMap<QString, QString> &map) {
             _finished = true;
             _item->_remotePerm = RemotePermissions::fromServerString(map.value(QStringLiteral("permissions")));
-            finalize(Utility::normalizeEtag(map.value(QStringLiteral("getetag")).toUtf8()), map.value(QStringLiteral("fileid")).toUtf8());
-            slotJobDestroyed(check);
+            finalize(Utility::normalizeEtag(map.value(QStringLiteral("getetag"))), map.value(QStringLiteral("fileid")).toUtf8());
         });
-        connect(check, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
         check->start();
         return;
     }
+    _item->_remotePerm = RemotePermissions::fromServerString(QString::fromUtf8(job->reply()->rawHeader("Oc-Perm")));
     // the file id should only be empty for new files up- or downloaded
     finalize(etag, job->reply()->rawHeader("OC-FileID"));
 }
 
-void PropagateUploadFileTUS::finalize(const QByteArray &etag, const QByteArray &fileId)
+void PropagateUploadFileTUS::finalize(const QString &etag, const QByteArray &fileId)
 {
     OC_ASSERT(_finished);
     qCDebug(lcPropagateUploadTUS) << _item->_etag << etag << fileId;
