@@ -54,6 +54,8 @@
 #include <QElapsedTimer>
 #include <qtextcodec.h>
 
+using namespace std::chrono_literals;
+
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcEngine, "sync.engine", QtInfoMsg)
@@ -76,9 +78,10 @@ static const std::chrono::milliseconds s_touchedFilesMaxAgeMs(3 * 1000);
 // doc in header
 std::chrono::milliseconds SyncEngine::minimumFileAgeForUpload(2000);
 
-SyncEngine::SyncEngine(AccountPtr account, const QString &localPath,
+SyncEngine::SyncEngine(AccountPtr account, const QUrl &baseUrl, const QString &localPath,
     const QString &remotePath, OCC::SyncJournalDb *journal)
     : _account(account)
+    , _baseUrl(baseUrl)
     , _needsUpdate(false)
     , _syncRunning(false)
     , _localPath(localPath)
@@ -106,7 +109,7 @@ SyncEngine::SyncEngine(AccountPtr account, const QString &localPath,
     _syncFileStatusTracker.reset(new SyncFileStatusTracker(this));
 
     _clearTouchedFilesTimer.setSingleShot(true);
-    _clearTouchedFilesTimer.setInterval(30 * 1000);
+    _clearTouchedFilesTimer.setInterval(30s);
     connect(&_clearTouchedFilesTimer, &QTimer::timeout, this, &SyncEngine::slotClearTouchedFiles);
 }
 
@@ -163,7 +166,7 @@ bool SyncEngine::checkErrorBlacklisting(SyncFileItem &item)
         if (item._etag.isEmpty() || entry._lastTryEtag.isEmpty()) {
             qCInfo(lcEngine) << item._file << "one ETag is empty, no blacklisting";
             return false;
-        } else if (item._etag != entry._lastTryEtag) {
+        } else if (item._etag.toUtf8() != entry._lastTryEtag) {
             qCInfo(lcEngine) << item._file << " is blacklisted, but has changed etag!";
             return false;
         }
@@ -244,8 +247,7 @@ void SyncEngine::deleteStaleUploadInfos(const SyncFileItemSet &syncItems)
         for (auto transferId : ids) {
             if (!transferId)
                 continue; // Was not a chunked upload
-            QUrl url = Utility::concatUrlPath(account()->url(), QLatin1String("remote.php/dav/uploads/") + account()->davUser() + QLatin1Char('/') + QString::number(transferId));
-            (new DeleteJob(account(), url, this))->start();
+            (new DeleteJob(account(), account()->url(), QLatin1String("remote.php/dav/uploads/") + account()->davUser() + QLatin1Char('/') + QString::number(transferId), this))->start();
         }
     }
 }
@@ -432,7 +434,7 @@ void SyncEngine::startSync()
 
     _lastLocalDiscoveryStyle = _localDiscoveryStyle;
 
-    if (_syncOptions._vfs->mode() == Vfs::WithSuffix && _syncOptions._vfs->fileSuffix().isEmpty()) {
+    if (syncOptions()._vfs->mode() == Vfs::WithSuffix && syncOptions()._vfs->fileSuffix().isEmpty()) {
         Q_EMIT syncError(tr("Using virtual files with suffix, but suffix is not set"));
         finalize(false);
         return;
@@ -451,19 +453,16 @@ void SyncEngine::startSync()
     }
 
     _stopWatch.start();
-    _progressInfo->_status = ProgressInfo::Starting;
-    emit transmissionProgress(*_progressInfo);
 
     qCInfo(lcEngine) << "#### Discovery start ####################################################";
-    qCInfo(lcEngine) << "Server" << account()->serverVersion()
+    qCInfo(lcEngine) << "Server" << account()->capabilities().status().versionString()
                      << (account()->isHttp2Supported() ? "Using HTTP/2" : "");
     _progressInfo->_status = ProgressInfo::Discovery;
     emit transmissionProgress(*_progressInfo);
 
     // TODO: add a constructor to DiscoveryPhase
     // pass a syncEngine object rather than copying everyhting to another object
-    _discoveryPhase.reset(new DiscoveryPhase);
-    _discoveryPhase->_account = _account;
+    _discoveryPhase.reset(new DiscoveryPhase(_account, syncOptions(), _baseUrl));
     _discoveryPhase->_excludes = _excludedFiles.data();
     _discoveryPhase->_statedb = _journal;
     _discoveryPhase->_localDir = _localPath;
@@ -472,7 +471,6 @@ void SyncEngine::startSync()
     _discoveryPhase->_remoteFolder = _remotePath;
     if (!_discoveryPhase->_remoteFolder.endsWith(QLatin1Char('/')))
         _discoveryPhase->_remoteFolder+=QLatin1Char('/');
-    _discoveryPhase->_syncOptions = _syncOptions;
     _discoveryPhase->_shouldDiscoverLocaly = [this](const QString &s) { return shouldDiscoverLocally(s); };
     _discoveryPhase->setSelectiveSyncBlackList(selectiveSyncBlackList);
     _discoveryPhase->setSelectiveSyncWhiteList(_journal->getSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, &ok));
@@ -483,19 +481,10 @@ void SyncEngine::startSync()
         return;
     }
 
-    // Check for invalid character in old server version
-    QString invalidFilenamePattern = _account->capabilities().invalidFilenameRegex();
-    if (invalidFilenamePattern.isNull()
-        && _account->serverVersionInt() < Account::makeServerVersion(8, 1, 0)) {
-        // Server versions older than 8.1 don't support some characters in filenames.
-        // If the capability is not set, default to a pattern that avoids uploading
-        // files with names that contain these.
-        // It's important to respect the capability also for older servers -- the
-        // version check doesn't make sense for custom servers.
-        invalidFilenamePattern = QLatin1String("[\\\\:?*\"<>|]");
-    }
-    if (!invalidFilenamePattern.isEmpty())
+    const QString invalidFilenamePattern = _account->capabilities().invalidFilenameRegex();
+    if (!invalidFilenamePattern.isEmpty()) {
         _discoveryPhase->_invalidFilenameRx = QRegExp(invalidFilenamePattern);
+    }
     _discoveryPhase->_serverBlacklistedFiles = _account->capabilities().blacklistedFiles();
     _discoveryPhase->_ignoreHiddenFiles = ignoreHiddenFiles();
 
@@ -540,7 +529,7 @@ void SyncEngine::slotFolderDiscovered(bool local, const QString &folder)
     emit transmissionProgress(*_progressInfo);
 }
 
-void SyncEngine::slotRootEtagReceived(const QByteArray &e, const QDateTime &time)
+void SyncEngine::slotRootEtagReceived(const QString &e, const QDateTime &time)
 {
     if (_remoteRootEtag.isEmpty()) {
         qCDebug(lcEngine) << "Root etag:" << e;
@@ -646,22 +635,20 @@ void SyncEngine::slotDiscoveryFinished()
 
         // post update phase script: allow to tweak stuff by a custom script in debug mode.
         if (!qEnvironmentVariableIsEmpty("OWNCLOUD_POST_UPDATE_SCRIPT")) {
-    #ifndef NDEBUG
+#ifndef NDEBUG
             const QString script = qEnvironmentVariable("OWNCLOUD_POST_UPDATE_SCRIPT");
 
             qCDebug(lcEngine) << "Post Update Script: " << script;
-            QProcess::execute(script);
-    #else
+            QProcess::execute(script, {});
+#else
             qCWarning(lcEngine) << "**** Attention: POST_UPDATE_SCRIPT installed, but not executed because compiled with NDEBUG";
-    #endif
+#endif
         }
 
         // do a database commit
         _journal->commit(QStringLiteral("post treewalk"));
 
-        _propagator = QSharedPointer<OwncloudPropagator>(
-            new OwncloudPropagator(_account, _localPath, _remotePath, _journal));
-        _propagator->setSyncOptions(_syncOptions);
+        _propagator = QSharedPointer<OwncloudPropagator>::create(_account, syncOptions(), _baseUrl, _localPath, _remotePath, _journal);
         connect(_propagator.data(), &OwncloudPropagator::itemCompleted,
             this, &SyncEngine::slotItemCompleted);
         connect(_propagator.data(), &OwncloudPropagator::progress,
@@ -729,14 +716,18 @@ void SyncEngine::setNetworkLimits(int upload, int download)
     _uploadLimit = upload;
     _downloadLimit = download;
 
-    if (!_propagator)
-        return;
-
-    _propagator->_uploadLimit = upload;
-    _propagator->_downloadLimit = download;
-
-    if (upload != 0 || download != 0) {
-        qCInfo(lcEngine) << "Network Limits (down/up) " << upload << download;
+    if (_propagator) {
+        if (upload != 0 || download != 0) {
+            qCInfo(lcEngine) << "Network Limits (down/up) " << upload << download;
+            if (!_propagator->_bandwidthManager) {
+                _propagator->_bandwidthManager = new BandwidthManager(_propagator.data());
+            }
+        }
+        // this might set the limit to 0 but only the next sync will have no bandwithd manager set
+        if (_propagator->_bandwidthManager) {
+            _propagator->_bandwidthManager->setCurrentDownloadLimit(download);
+            _propagator->_bandwidthManager->setCurrentUploadLimit(upload);
+        }
     }
 }
 
@@ -898,6 +889,13 @@ void SyncEngine::setLocalDiscoveryOptions(LocalDiscoveryStyle style, std::set<QS
     _localDiscoveryStyle = style;
     _localDiscoveryPaths = std::move(paths);
 
+
+    // add running upload to the local discovery
+    const auto info = _journal->getUploadInfos();
+    for (const auto &i : info) {
+        _localDiscoveryPaths.insert(QString::fromUtf8(i._path));
+    }
+
     // Normalize to make sure that no path is a contained in another.
     // Note: for simplicity, this code consider anything less than '/' as a path separator, so for
     // example, this will remove "foo.bar" if "foo" is in the list. This will mean we might have
@@ -917,8 +915,9 @@ void SyncEngine::setLocalDiscoveryOptions(LocalDiscoveryStyle style, std::set<QS
 
 bool SyncEngine::shouldDiscoverLocally(const QString &path) const
 {
-    if (_localDiscoveryStyle == LocalDiscoveryStyle::FilesystemOnly)
+    if (_localDiscoveryStyle == LocalDiscoveryStyle::FilesystemOnly) {
         return true;
+    }
 
     // The intention is that if "A/X" is in _localDiscoveryPaths:
     // - parent folders like "/", "A" will be discovered (to make sure the discovery reaches the
