@@ -15,6 +15,7 @@
 #pragma once
 
 #include "owncloudlib.h"
+#include "common/filesystembase.h"
 #include "common/vfs.h"
 
 #include <QRegularExpression>
@@ -32,7 +33,7 @@ namespace OCC {
 class OWNCLOUDSYNC_EXPORT SyncOptions
 {
 public:
-    SyncOptions();
+    explicit SyncOptions(QSharedPointer<Vfs> vfs);
     ~SyncOptions();
 
     /** Maximum size (in Bytes) a folder can have without asking for confirmation.
@@ -104,6 +105,15 @@ public:
      */
     void setPathPattern(const QString &pattern);
 
+
+    /**
+     * The required file lock mode
+     */
+    FileSystem::LockMode requiredLockMode() const
+    {
+        return _vfs->mode() == Vfs::WindowsCfApi ? FileSystem::LockMode::Exclusive : FileSystem::LockMode::Shared;
+    }
+
 private:
     /**
      * Only sync files that mathc the expression
