@@ -13,19 +13,17 @@
  */
 
 #include "theme.h"
-#include "config.h"
 #include "common/depreaction.h"
 #include "common/utility.h"
-#include "version.h"
-#include "configfile.h"
+#include "common/version.h"
 #include "common/vfs.h"
+#include "config.h"
+#include "configfile.h"
 
 #include <QtCore>
-#ifndef TOKEN_AUTH_ONLY
 #include <QtGui>
 #include <QStyle>
 #include <QApplication>
-#endif
 #include <QSslSocket>
 
 #include "owncloudtheme.h"
@@ -141,18 +139,11 @@ QString Theme::appName() const
     return QStringLiteral(APPLICATION_SHORTNAME);
 }
 
-QString Theme::version() const
-{
-    return QStringLiteral(MIRALL_VERSION_STRING);
-}
-
 QString Theme::configFileName() const
 {
     return QStringLiteral(APPLICATION_EXECUTABLE ".cfg");
 }
 
-#ifndef TOKEN_AUTH_ONLY
-
 QIcon Theme::applicationIcon() const
 {
     return themeUniversalIcon(QStringLiteral(APPLICATION_ICON_NAME "-icon"));
@@ -231,7 +222,7 @@ QIcon Theme::loadIcon(const QString &flavor, const QString &name, IconType iconT
                 cached.addFile(pixmapName, { size, size });
             } else if (size >= 128) {
                 if (!previousIcon.isEmpty()) {
-                    qWarning() << "Upsacling:" << previousIcon << "to" << size;
+                    qWarning() << "Upscaling:" << previousIcon << "to" << size;
                     cached.addPixmap(QPixmap(previousIcon).scaled({ size, size }, Qt::KeepAspectRatio, Qt::SmoothTransformation));
                 }
             }
@@ -275,8 +266,6 @@ QString Theme::systrayIconFlavor(bool mono, bool sysTrayMenuVisible) const
     return flavor;
 }
 
-#endif
-
 Theme::Theme()
     : QObject(nullptr)
 {
@@ -301,7 +290,7 @@ QString Theme::defaultServerFolder() const
 
 QString Theme::helpUrl() const
 {
-    return QStringLiteral("https://doc.owncloud.org/desktop/%1.%2/").arg(MIRALL_VERSION_MAJOR).arg(MIRALL_VERSION_MINOR);
+    return QStringLiteral("https://doc.owncloud.org/desktop/%1.%2/").arg(OCC::Version::version().majorVersion()).arg(OCC::Version::version().microVersion());
 }
 
 QString Theme::conflictHelpUrl() const
@@ -387,13 +376,11 @@ bool Theme::wizardHideFolderSizeLimitCheckbox() const
 
 QString Theme::gitSHA1(VersionFormat format) const
 {
-#ifdef GIT_SHA1
-    const auto gitSha = QStringLiteral(GIT_SHA1);
-    const auto gitShahSort = gitSha.left(6);
+    const QString gitShahSort = Version::gitSha().left(6);
     if (!aboutShowCopyright()) {
         return gitShahSort;
     }
-    const auto gitUrl = QStringLiteral("https://github.com/owncloud/client/commit/%1").arg(gitSha);
+    const auto gitUrl = QStringLiteral("https://github.com/owncloud/client/commit/%1").arg(Version::gitSha());
     switch (format) {
     case Theme::VersionFormat::OneLiner:
         Q_FALLTHROUGH();
@@ -404,7 +391,6 @@ QString Theme::gitSHA1(VersionFormat format) const
     case Theme::VersionFormat::RichText:
         return QStringLiteral("<a href=\"%1\">%3</a>").arg(gitUrl, gitShahSort);
     }
-#endif
     return QString();
 }
 
@@ -424,25 +410,24 @@ QString Theme::aboutVersions(Theme::VersionFormat format) const
         Q_UNREACHABLE();
     }();
     const QString qtVersion = QString::fromUtf8(qVersion());
-    const QString qtVersionString = (QLatin1String(QT_VERSION_STR) == qtVersion ? qtVersion : QCoreApplication::translate("ownCloudTheme::qtVer", "%1 (Built against Qt %1)").arg(qtVersion, QStringLiteral(QT_VERSION_STR)));
-    QString _version = version();
+    const QString qtVersionString = (QLatin1String(QT_VERSION_STR) == qtVersion ? qtVersion : QCoreApplication::translate("ownCloudTheme::qtVer", "%1 (Built against Qt %2)").arg(qtVersion, QStringLiteral(QT_VERSION_STR)));
+    QString _version = Version::displayString();
     QString gitUrl;
-#ifdef GIT_SHA1
-    if (format != Theme::VersionFormat::Url) {
-        _version = QCoreApplication::translate("ownCloudTheme::versionWithSha", "%1 %2").arg(version(), gitSHA1(format));
-    } else {
-        gitUrl = gitSHA1(format) + br;
+    if (!Version::gitSha().isEmpty()) {
+        if (format != Theme::VersionFormat::Url) {
+            _version = QCoreApplication::translate("ownCloudTheme::versionWithSha", "%1 %2").arg(_version, gitSHA1(format));
+        } else {
+            gitUrl = gitSHA1(format) + br;
+        }
     }
-#endif
     return QCoreApplication::translate("ownCloudTheme::aboutVersions()",
-        "%1 %2 %3%8"
-        "%9"
-        "Libraries Qt %4, %5%8"
-        "Using virtual files plugin: %6%8"
-        "%7")
+        "%1 %2%7"
+        "%8"
+        "Libraries Qt %3, %4%7"
+        "Using virtual files plugin: %5%7"
+        "%6")
         .arg(appName(),
             _version,
-            QStringLiteral(__DATE__ " " __TIME__),
             qtVersionString,
             QSslSocket::sslLibraryVersionString(),
             Vfs::modeToString(bestAvailableVfsMode()),
@@ -460,13 +445,14 @@ QString Theme::about() const
     return tr("<p>Version %1. For more information visit <a href=\"%2\">https://%3</a></p>"
               "<p>For known issues and help, please visit: <a href=\"https://central.owncloud.org/c/desktop-client\">https://central.owncloud.org</a></p>"
               "<p><small>By Klaas Freitag, Daniel Molkentin, Olivier Goffart, Markus Götz, "
-              " Jan-Christoph Borchardt, Thomas Müller, Dominik Schmidt, Michael Stingl, Hannah von Reth, and others.</small></p>"
+              " Jan-Christoph Borchardt, Thomas Müller,<br>"
+              "Dominik Schmidt, Michael Stingl, Hannah von Reth, Fabian Müller and others.</small></p>"
               "<p>Copyright ownCloud GmbH</p>"
               "<p>Distributed by %4 and licensed under the GNU General Public License (GPL) Version 2.0.<br/>"
               "%5 and the %5 logo are registered trademarks of %4 in the "
               "United States, other countries, or both.</p>"
               "<p><small>%6</small></p>")
-        .arg(Utility::escape(version()),
+        .arg(Utility::escape(Version::displayString()),
             Utility::escape(QStringLiteral("https://" APPLICATION_DOMAIN)),
             Utility::escape(QStringLiteral(APPLICATION_DOMAIN)),
             Utility::escape(vendor),
@@ -479,7 +465,6 @@ bool Theme::aboutShowCopyright() const
     return true;
 }
 
-#ifndef TOKEN_AUTH_ONLY
 QIcon Theme::syncStateIcon(SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible) const
 {
     return syncStateIcon(SyncResult { status }, sysTray, sysTrayMenuVisible);
@@ -528,7 +513,7 @@ QIcon Theme::syncStateIcon(const SyncResult &result, bool sysTray, bool sysTrayM
 
 QIcon Theme::folderDisabledIcon() const
 {
-    return themeIcon(QLatin1String("state-pause"));
+    return themeIcon(QStringLiteral("state-pause"));
 }
 
 QIcon Theme::folderOfflineIcon(bool sysTray, bool sysTrayMenuVisible) const
@@ -570,7 +555,6 @@ QPixmap Theme::wizardHeaderBanner(const QSize &size) const
     pix.fill(c);
     return pix;
 }
-#endif
 
 QString Theme::webDavPath() const
 {
@@ -653,6 +637,11 @@ QString Theme::openIdConnectPrompt() const
     return QStringLiteral("select_account consent");
 }
 
+bool Theme::oidcEnableDynamicRegistration() const
+{
+    return true;
+}
+
 QString Theme::versionSwitchOutput() const
 {
     return aboutVersions(Theme::VersionFormat::Url);
@@ -688,4 +677,29 @@ bool Theme::warnOnMultipleDb() const
     return isVanilla();
 }
 
+bool Theme::allowDuplicatedFolderSyncPair() const
+{
+    return true;
+}
+
+bool Theme::wizardEnableWebfinger() const
+{
+    return false;
+}
+
+template <>
+OWNCLOUDSYNC_EXPORT QString Utility::enumToDisplayName(Theme::UserIDType userIdType)
+{
+    switch (userIdType) {
+    case Theme::UserIDUserName:
+        return QCoreApplication::translate("Type of user ID", "Username");
+    case Theme::UserIDEmail:
+        return QCoreApplication::translate("Type of user ID", "E-mail address");
+    case Theme::UserIDCustom:
+        return Theme::instance()->customUserID();
+    default:
+        Q_UNREACHABLE();
+    }
+}
+
 } // end namespace client
