@@ -8,10 +8,11 @@ import re
 import urllib.request
 import json
 import requests
+import builtins
 import shutil
 
-from objectmaphelper import RegularExpression
 from pageObjects.AccountConnectionWizard import AccountConnectionWizard
+from pageObjects.SyncConnectionWizard import SyncConnectionWizard
 from helpers.SetupClientHelper import *
 from helpers.FilesHelper import buildConflictedRegex
 from pageObjects.EnterPassword import EnterPassword
@@ -22,55 +23,156 @@ from pageObjects.Toolbar import Toolbar
 from pageObjects.Activity import Activity
 from pageObjects.AccountStatus import AccountStatus
 
+from helpers.SyncHelper import (
+    SYNC_STATUS,
+    getInitialSyncPatterns,
+    getSyncedPattern,
+    generateSyncPatternFromMessages,
+    filterSyncMessages,
+    filterMessagesForItem,
+    isEqual,
+)
+
 # the script needs to use the system wide python
 # to switch from the built-in interpreter see https://kb.froglogic.com/squish/howto/using-external-python-interpreter-squish-6-6/
 # if the IDE fails to reference the script, add the folder in Edit->Preferences->PyDev->Interpreters->Libraries
 sys.path.append(os.path.realpath('../../../shell_integration/nautilus/'))
-import syncstate
-import functools
+from syncstate import SocketConnect
 
 
-confdir = '/tmp/bdd-tests-owncloud-client/'
-confFilePath = confdir + 'owncloud.cfg'
 socketConnect = None
 
-stateDataFromMiddleware = None
-
-# File syncing in client has the following status
-SYNC_STATUS = {
-    'SYNC': 'STATUS:SYNC',  # sync in process
-    'OK': 'STATUS:OK',  # sync completed
-    'ERROR': 'STATUS:ERROR',  # file sync has error
-    'IGNORE': 'STATUS:IGNORE',  # file is igored
-    'NOP': 'STATUS:NOP',  # file yet to be synced
-}
+createdUsers = {}
 
 
-def getTestStateFromMiddleware(context):
-    global stateDataFromMiddleware
-    if stateDataFromMiddleware is None:
+def readSocketMessages():
+    socket_messages = []
+    socketConnect = getSocketConnection()
+    socketConnect.read_socket_data_with_timeout(0.1)
+    for line in socketConnect.get_available_responses():
+        socket_messages.append(line)
+    return socket_messages
+
+
+def readAndUpdateSocketMessages():
+    messages = readSocketMessages()
+    return updateSocketMessages(messages)
+
+
+def updateSocketMessages(messages):
+    global socket_messages
+    socket_messages.extend(filterSyncMessages(messages))
+    return socket_messages
+
+
+def clearSocketMessages(resource=''):
+    global socket_messages
+    if resource:
+        resource_messages = set(filterMessagesForItem(socket_messages, resource))
+        socket_messages = [
+            msg for msg in socket_messages if msg not in resource_messages
+        ]
+    else:
+        socket_messages.clear()
+
+
+def listenSyncStatusForItem(item, type='FOLDER'):
+    type = type.upper()
+    if type != 'FILE' and type != 'FOLDER':
+        raise Exception("type must be 'FILE' or 'FOLDER'")
+    socketConnect = getSocketConnection()
+    socketConnect.sendCommand("RETRIEVE_" + type + "_STATUS:" + item + "\n")
+
+
+def getCurrentSyncStatus(resource, resourceType):
+    listenSyncStatusForItem(resource, resourceType)
+    messages = filterMessagesForItem(readSocketMessages(), resource)
+    # return the last message from the list
+    return messages[-1]
+
+
+def waitForFileOrFolderToSync(
+    context, resource='', resourceType='FOLDER', patterns=None
+):
+    resource = join(context.userData['currentUserSyncPath'], resource).rstrip('/')
+    listenSyncStatusForItem(resource, resourceType)
+
+    timeout = context.userData['maxSyncTimeout'] * 1000
+
+    if patterns is None:
+        patterns = getSyncedPattern()
+
+    synced = waitFor(
+        lambda: hasSyncPattern(patterns, resource),
+        timeout,
+    )
+    clearSocketMessages(resource)
+    if not synced:
+        # if the sync pattern doesn't match then check the last sync status
+        # and pass the step if the last sync status is STATUS:OK
+        status = getCurrentSyncStatus(resource, resourceType)
+        if status.startswith(SYNC_STATUS['OK']):
+            test.log(
+                "[WARN] Failed to match sync pattern for resource: "
+                + resource
+                + "\nBut its last status is "
+                + "'"
+                + SYNC_STATUS['OK']
+                + "'"
+                + ". So passing the step."
+            )
+            return
+        else:
+            raise Exception(
+                "Timeout while waiting for sync to complete for "
+                + str(timeout)
+                + " milliseconds"
+            )
+
+
+def waitForInitialSyncToComplete(context):
+    waitForFileOrFolderToSync(
+        context,
+        context.userData['currentUserSyncPath'],
+        'FOLDER',
+        getInitialSyncPatterns(),
+    )
+
+
+def hasSyncPattern(patterns, resource=None):
+    if isinstance(patterns[0], str):
+        patterns = [patterns]
+    messages = readAndUpdateSocketMessages()
+    if resource:
+        messages = filterMessagesForItem(messages, resource)
+    for pattern in patterns:
+        pattern_len = len(pattern)
+        for idx, _ in enumerate(messages):
+            actual_pattern = generateSyncPatternFromMessages(
+                messages[idx : idx + pattern_len]
+            )
+            if len(actual_pattern) < pattern_len:
+                break
+            if pattern_len == len(actual_pattern) and isEqual(pattern, actual_pattern):
+                return True
+    # 100 milliseconds polling interval
+    snooze(0.1)
+    return False
+
+
+# gets all users information created in a test scenario
+def getCreatedUsersFromMiddleware(context):
+    createdUsers = {}
+    try:
         res = requests.get(
             os.path.join(context.userData['middlewareUrl'], 'state'),
             headers={"Content-Type": "application/json"},
         )
-        try:
-            stateDataFromMiddleware = res.json()
-        except ValueError:
-            raise Exception("Could not get created users information from middleware")
+        createdUsers = res.json()['created_users']
+    except ValueError:
+        raise Exception("Could not get created users information from middleware")
 
-    return stateDataFromMiddleware
-
-
-@OnScenarioStart
-def hook(context):
-    try:
-        os.makedirs(confdir, 0o0755)
-    except:
-        pass
-    try:
-        os.remove(confFilePath)
-    except:
-        pass
+    return createdUsers
 
 
 @Given(r'the user has added (the first|another) account with', regexp=True)
@@ -101,23 +203,65 @@ def step(context, displayname, host):
     )
 
 
+def getUserInfo(context, username, attribute):
+    # add and update users to the global createdUsers dict if not already there
+    # so that we don't have to request for user information in every scenario
+    # but instead get user information from the global dict
+    global createdUsers
+    if username in createdUsers:
+        return createdUsers[username][attribute]
+    else:
+        createdUsers = {**createdUsers, **getCreatedUsersFromMiddleware(context)}
+        return createdUsers[username][attribute]
+
+
 def getDisplaynameForUser(context, username):
-    usersDataFromMiddleware = getTestStateFromMiddleware(context)
-    return usersDataFromMiddleware['created_users'][username]['displayname']
+    return getUserInfo(context, username, 'displayname')
 
 
 def getPasswordForUser(context, username):
-    usersDataFromMiddleware = getTestStateFromMiddleware(context)
-    return usersDataFromMiddleware['created_users'][username]['password']
+    return getUserInfo(context, username, 'password')
+
+
+def isConnecting():
+    return "Connecting to" in str(
+        waitForObjectExists(names.stack_connectLabel_QLabel).text
+    )
+
+
+def waitUntilConnectionIsConfigured(context):
+    timeout = context.userData['maxSyncTimeout'] * 1000
+
+    result = waitFor(
+        lambda: isConnecting(),
+        timeout,
+    )
+
+    if not result:
+        raise Exception(
+            "Timeout waiting for connection to be configured for "
+            + str(timeout)
+            + " milliseconds"
+        )
 
 
 @Given('user "|any|" has set up a client with default settings')
 def step(context, username):
     password = getPasswordForUser(context, username)
     displayName = getDisplaynameForUser(context, username)
-    setUpClient(context, username, displayName, confFilePath)
-    enterUserPassword = EnterPassword()
-    enterUserPassword.enterPassword(password)
+    setUpClient(context, username, displayName, context.userData['clientConfigFile'])
+
+    if context.userData['ocis']:
+        newAccount = AccountConnectionWizard()
+        newAccount.acceptCertificate()
+        newAccount.oidcLogin(username, password, True)
+    else:
+        waitUntilConnectionIsConfigured(context)
+        enterUserPassword = EnterPassword()
+        enterUserPassword.enterPassword(password)
+
+    # wait for files to sync
+    waitForInitialSyncToComplete(context)
 
 
 @Given('the user has started the client')
@@ -125,7 +269,7 @@ def step(context):
     startClient(context)
 
 
-@When(r'the user adds (the first|another) account with', regexp=True)
+@When(r'^the user adds (the first|another) account with$', regexp=True)
 def step(context, accountType):
     newAccount = AccountConnectionWizard()
     if accountType == 'another':
@@ -138,52 +282,42 @@ def step(context, accountType):
 @Given('the user has added the following account information:')
 def step(context):
     newAccount = AccountConnectionWizard()
-    newAccount.addServer(context)
-    newAccount.addUserCreds(context)
-    newAccount.selectSyncFolder(context)
+    newAccount.addAccountCredential(context)
+
+
+def getSocketConnection():
+    global socketConnect
+    if not socketConnect or not socketConnect.connected:
+        socketConnect = SocketConnect()
+    return socketConnect
 
 
 # Using socket API to check file sync status
-def hasSyncStatus(type, itemName, status):
-    if type != 'FILE' and type != 'FOLDER':
-        raise Exception("type must be 'FILE' or 'FOLDER'")
-    socketConnect = syncstate.SocketConnect()
-    socketConnect.sendCommand("RETRIEVE_" + type + "_STATUS:" + itemName + "\n")
-
-    if not socketConnect.read_socket_data_with_timeout(0.1):
-        return False
-    for line in socketConnect.get_available_responses():
-        if line.startswith(status) and line.endswith(itemName):
+def hasSyncStatus(itemName, status):
+    sync_messages = readAndUpdateSocketMessages()
+    sync_messages = filterMessagesForItem(sync_messages, itemName)
+    for line in sync_messages:
+        if line.startswith(status) and line.rstrip('/').endswith(itemName.rstrip('/')):
             return True
-        elif line.endswith(itemName):
-            return False
-
-
-def folderHasSyncStatus(folderName, status):
-    return hasSyncStatus('FOLDER', folderName, status)
-
-
-def fileHasSyncStatus(fileName, status):
-    return hasSyncStatus('FILE', fileName, status)
+    return False
 
 
+# useful for checking sync status such as 'error', 'ignore'
+# but not quite so reliable for checking 'ok' sync status
 def waitForFileOrFolderToHaveSyncStatus(
     context, resource, resourceType, status=SYNC_STATUS['OK'], timeout=None
 ):
+    resource = sanitizePath(join(context.userData['currentUserSyncPath'], resource))
+
+    listenSyncStatusForItem(resource, resourceType)
+
     if not timeout:
         timeout = context.userData['maxSyncTimeout'] * 1000
 
-    resource = join(context.userData['currentUserSyncPath'], resource)
-    if resourceType.lower() == "file":
-        result = waitFor(
-            lambda: fileHasSyncStatus(sanitizePath(resource), status),
-            timeout,
-        )
-    elif resourceType.lower() == "folder":
-        result = waitFor(
-            lambda: folderHasSyncStatus(sanitizePath(resource), status),
-            timeout,
-        )
+    result = waitFor(
+        lambda: hasSyncStatus(resource, status),
+        timeout,
+    )
 
     if not result:
         if status == SYNC_STATUS['ERROR']:
@@ -203,67 +337,12 @@ def waitForFileOrFolderToHaveSyncStatus(
         )
 
 
-def waitForSyncToStart(context, resource, resourceType):
-    resource = join(context.userData['currentUserSyncPath'], resource)
-
-    hasStatusNOP = hasSyncStatus(resourceType.upper(), resource, SYNC_STATUS['NOP'])
-    hasStatusSYNC = hasSyncStatus(resourceType.upper(), resource, SYNC_STATUS['SYNC'])
-
-    if hasStatusSYNC:
-        return
-
-    try:
-        if hasStatusNOP:
-            waitForFileOrFolderToHaveSyncStatus(
-                context, resource, resourceType, SYNC_STATUS['SYNC']
-            )
-        else:
-            waitForFileOrFolderToHaveSyncStatus(
-                context,
-                resource,
-                resourceType,
-                SYNC_STATUS['SYNC'],
-                context.userData['minSyncTimeout'] * 1000,
-            )
-    except:
-        hasStatusNOP = hasSyncStatus(resourceType.upper(), resource, SYNC_STATUS['NOP'])
-        if hasStatusNOP:
-            raise Exception(
-                "Expected "
-                + resourceType
-                + " '"
-                + resource
-                + "' to have sync started but not."
-            )
-
-
-def waitForFileOrFolderToSync(context, resource, resourceType):
-    waitForSyncToStart(context, resource, resourceType)
-    waitForFileOrFolderToHaveSyncStatus(
-        context, resource, resourceType, SYNC_STATUS['OK']
-    )
-
-
-def waitForRootFolderToSync(context):
-    waitForFileOrFolderToSync(
-        context, context.userData['currentUserSyncPath'], 'folder'
-    )
-
-
 def waitForFileOrFolderToHaveSyncError(context, resource, resourceType):
-    waitForSyncToStart(context, resource, resourceType)
     waitForFileOrFolderToHaveSyncStatus(
         context, resource, resourceType, SYNC_STATUS['ERROR']
     )
 
 
-def waitForFileOrFolderToBeSyncIgnored(context, resource, resourceType):
-    waitForSyncToStart(context, resource, resourceType)
-    waitForFileOrFolderToHaveSyncStatus(
-        context, resource, resourceType, SYNC_STATUS['IGNORE']
-    )
-
-
 def folderExists(folderPath, timeout=1000):
     return waitFor(
         lambda: isdir(sanitizePath(folderPath)),
@@ -283,7 +362,7 @@ def sanitizePath(path):
 
 
 def shareResource(resource):
-    socketConnect = syncstate.SocketConnect()
+    socketConnect = getSocketConnection()
     socketConnect.sendCommand("SHARE:" + resource + "\n")
     if not socketConnect.read_socket_data_with_timeout(0.1):
         return False
@@ -377,7 +456,7 @@ def collaboratorShouldBeListed(
     context, receiver, resource, permissions, receiverCount=0
 ):
     resource = getResourcePath(context, resource)
-    socketConnect = syncstate.SocketConnect()
+    socketConnect = getSocketConnection()
     socketConnect.sendCommand("SHARE:" + resource + "\n")
     permissionsList = permissions.split(',')
 
@@ -420,7 +499,7 @@ def collaboratorShouldBeListed(
 
 @When('the user waits for the files to sync')
 def step(context):
-    waitForRootFolderToSync(context)
+    waitForFileOrFolderToSync(context)
 
 
 @When(r'the user waits for (file|folder) "([^"]*)" to be synced', regexp=True)
@@ -433,45 +512,21 @@ def step(context, type, resource):
     waitForFileOrFolderToHaveSyncError(context, resource, type)
 
 
-@When(r'the user waits for (file|folder) "([^"]*)" to be sync ignored', regexp=True)
-def step(context, type, resource):
-    waitForFileOrFolderToBeSyncIgnored(context, resource, type)
-
-
-@Given(r'the user has waited for (file|folder) "([^"]*)" to be synced', regexp=True)
-def step(context, type, resource):
-    waitForFileOrFolderToSync(context, resource, type)
-
-
-@Given(
-    'user "|any|" has created a file "|any|" with the following content inside the sync folder'
+@When(
+    r'user "([^"]*)" waits for (file|folder) "([^"]*)" to have sync error', regexp=True
 )
-def step(context, username, filename):
-    createFile(context, filename, username)
+def step(context, username, type, resource):
+    resource = join(getUserSyncPath(context, username), resource)
+    waitForFileOrFolderToHaveSyncError(context, resource, type)
 
 
 @When(
     'user "|any|" creates a file "|any|" with the following content inside the sync folder'
 )
 def step(context, username, filename):
-    createFile(context, filename, username)
-
-
-def createFile(context, filename, username=None):
     fileContent = "\n".join(context.multiLineText)
-    syncPath = None
-    if username:
-        syncPath = getUserSyncPath(context, username)
-    else:
-        syncPath = context.userData['currentUserSyncPath']
-
-    # A file is scheduled to be synced but is marked as ignored for 5 seconds. And if we try to sync it, it will fail. So we need to wait for 5 seconds.
-    # https://github.com/owncloud/client/issues/9325
-    snooze(context.userData['minSyncTimeout'])
-
-    f = open(join(syncPath, filename), "w")
-    f.write(fileContent)
-    f.close()
+    syncPath = getUserSyncPath(context, username)
+    waitAndWriteFile(context, join(syncPath, filename), fileContent)
 
 
 @When('user "|any|" creates a folder "|any|" inside the sync folder')
@@ -484,10 +539,14 @@ def step(context, username, foldername):
     createFolder(context, foldername, username)
 
 
-def createFolder(context, foldername, username=None):
+# To create folders in a temporary directory, we set isTempFolder True
+# And if isTempFolder is True, the createFolder function create folders in tempFolderPath
+def createFolder(context, foldername, username=None, isTempFolder=False):
     syncPath = None
-    if username:
+    if username and not isTempFolder:
         syncPath = getUserSyncPath(context, username)
+    elif isTempFolder:
+        syncPath = context.userData['tempFolderPath']
     else:
         syncPath = context.userData['currentUserSyncPath']
     path = join(syncPath, foldername)
@@ -500,6 +559,21 @@ def renameFileFolder(context, source, destination):
     rename(source, destination)
 
 
+@When('user "|any|" creates a file "|any|" with size "|any|" inside the sync folder')
+def step(context, username, filename, filesize):
+    createFileWithSize(context, filename, filesize)
+
+
+def createFileWithSize(context, filename, filesize, isTempFolder=False):
+    if isTempFolder:
+        path = context.userData['tempFolderPath']
+    else:
+        path = context.userData['currentUserSyncPath']
+    file = join(path, filename)
+    cmd = "truncate -s {filesize} {file}".format(filesize=filesize, file=file)
+    os.system(cmd)
+
+
 @When('the user copies the folder "|any|" to "|any|"')
 def step(context, sourceFolder, destinationFolder):
     source_dir = join(context.userData['currentUserSyncPath'], sourceFolder)
@@ -594,33 +668,21 @@ def step(context, resourceType, resource):
 @Given('the user has paused the file sync')
 def step(context):
     syncWizard = SyncWizard()
-    syncWizard.performAction("Pause sync")
+    syncWizard.performAction(context, "Pause sync")
 
 
 @Given('the user has changed the content of local file "|any|" to:')
 def step(context, filename):
     fileContent = "\n".join(context.multiLineText)
-    f = open(context.userData['currentUserSyncPath'] + filename, "w")
-    f.write(fileContent)
-    f.close()
+    waitAndWriteFile(
+        context, join(context.userData['currentUserSyncPath'], filename), fileContent
+    )
 
 
 @When('the user resumes the file sync on the client')
 def step(context):
     syncWizard = SyncWizard()
-    syncWizard.performAction("Resume sync")
-
-
-@When('the user triggers force sync on the client')
-def step(context):
-    mouseClick(
-        waitForObjectItem(names.stack_folderList_QTreeView, "_1"),
-        720,
-        36,
-        Qt.NoModifier,
-        Qt.LeftButton,
-    )
-    activateItem(waitForObjectItem(names.settings_QMenu, "Force sync now"))
+    syncWizard.performAction(context, "Resume sync")
 
 
 @Then(
@@ -629,8 +691,6 @@ def step(context):
 def step(context, filename):
     expected = "\n".join(context.multiLineText)
 
-    namepart = filename.split('.')[0]
-    extpart = filename.split('.')[1]
     onlyfiles = [
         f
         for f in listdir(context.userData['currentUserSyncPath'])
@@ -685,7 +745,11 @@ def step(context, filename):
 @Then('the file "|any|" should be blacklisted')
 def step(context, filename):
     activity = Activity()
-    activity.checkBlackListedFileExist(filename)
+    test.compare(
+        True,
+        activity.checkBlackListedResourceExist(context, filename),
+        "File is blacklisted",
+    )
 
 
 @When('the user selects "|any|" tab in the activity')
@@ -696,6 +760,11 @@ def step(context, tabName):
 
 def openSharingDialog(context, resource, itemType='file'):
     resource = getResourcePath(context, resource)
+    resourceExist = waitFor(
+        lambda: os.path.exists(resource), context.userData['maxSyncTimeout'] * 1000
+    )
+    if not resourceExist:
+        raise Exception("{} doesn't exists".format(resource))
     waitFor(lambda: shareResource(resource), context.userData['maxSyncTimeout'] * 1000)
 
 
@@ -915,7 +984,6 @@ def step(context, username):
 
 @Given('user "|any|" has logged out of the client-UI')
 def step(context, username):
-    waitForRootFolderToSync(context)
     accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     accountStatus.accountAction("Log out")
     isUserSignedOut(context, username)
@@ -926,15 +994,20 @@ def step(context, username):
     accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     accountStatus.accountAction("Log in")
     password = getPasswordForUser(context, username)
-    enterUserPassword = EnterPassword()
-    enterUserPassword.enterPassword(password)
+
+    if context.userData['ocis']:
+        account = AccountConnectionWizard()
+        account.oidcLogin(username, password, True)
+    else:
+        enterUserPassword = EnterPassword()
+        enterUserPassword.enterPassword(password)
+
+    # wait for files to sync
+    waitForInitialSyncToComplete(context)
 
 
 @Then('user "|any|" should be connect to the client-UI')
 def step(context, username):
-    # TODO: find some way to dynamically to check if files are synced
-    # It might take some time for all files to sync and connect to ther server
-    snooze(context.userData['minSyncTimeout'])
     isUserSignedIn(context, username)
 
 
@@ -963,10 +1036,6 @@ def step(context, displayname, host):
 
 @Then('connection wizard should be visible')
 def step(context):
-    test.compare(
-        str(waitForObjectExists(names.owncloudWizard_label_2_QLabel).text),
-        'Ser&ver Address',
-    )
     waitForObject(AccountConnectionWizard.SERVER_ADDRESS_BOX)
 
 
@@ -1027,15 +1096,31 @@ def step(context, resource, group):
     sharingDialog.selectCollaborator(group, True)
 
 
-def overwriteFile(resource, content):
+# performing actions immediately after completing the sync from the server does not work
+# The test should wait for a while before performing the action
+# issue: https://github.com/owncloud/client/issues/8832
+def waitForClientToBeReady(context):
+    global waitedAfterSync
+    if not waitedAfterSync:
+        snooze(context.userData['minSyncTimeout'])
+        waitedAfterSync = True
+
+
+def writeFile(resource, content):
     f = open(resource, "w")
     f.write(content)
     f.close()
 
 
-def tryToOverwriteFile(context, resource, content):
+def waitAndWriteFile(context, path, content):
+    waitForClientToBeReady(context)
+    writeFile(path, content)
+
+
+def waitAndTryToWriteFile(context, resource, content):
+    waitForClientToBeReady(context)
     try:
-        overwriteFile(resource, content)
+        writeFile(resource, content)
     except:
         pass
 
@@ -1044,84 +1129,51 @@ def tryToOverwriteFile(context, resource, content):
 def step(context, resource, content):
     print("starting file overwrite")
     resource = join(context.userData['currentUserSyncPath'], resource)
-
-    # overwriting the file immediately after it has been synced from the server seems to have some problem.
-    # The client does not see the change although the changes have already been made thus we are having a race condition
-    # So for now we add 5sec static wait
-    # an issue https://github.com/owncloud/client/issues/8832 has been created for it
-    snooze(context.userData['minSyncTimeout'])
-
-    overwriteFile(resource, content)
-
+    waitAndWriteFile(context, resource, content)
     print("file has been overwritten")
 
 
 @When('the user tries to overwrite the file "|any|" with content "|any|"')
 def step(context, resource, content):
     resource = context.userData['currentUserSyncPath'] + resource
-    tryToOverwriteFile(context, resource, content)
+    waitAndTryToWriteFile(context, resource, content)
 
 
 @When('user "|any|" tries to overwrite the file "|any|" with content "|any|"')
 def step(context, user, resource, content):
     resource = getResourcePath(context, resource, user)
-    tryToOverwriteFile(context, resource, content)
+    waitAndTryToWriteFile(context, resource, content)
 
 
-def enableVFSSupport(vfsBtnText):
-    # The enabling/disabling VFS button do not have it's own object
-    # But it is inside the "stack_folderList_QTreeView" object.
-    # So we are clicking at (718, 27) of "stack_folderList_QTreeView" object to enable/disable VFS
-    mouseClick(
-        waitForObjectItem(names.stack_folderList_QTreeView, "_1"),
-        718,
-        27,
-        Qt.NoModifier,
-        Qt.LeftButton,
-    )
+def enableVFSSupport(context, vfsBtnText):
+    AccountStatus.openAccountMenu(context)
     activateItem(waitForObjectItem(names.settings_QMenu, vfsBtnText))
     clickButton(
-        waitForObject(names.stack_Enable_experimental_placeholder_mode_QPushButton)
+        waitForObject(
+            names.enable_experimental_feature_Enable_experimental_placeholder_mode_QPushButton
+        )
     )
 
 
 @When("the user enables virtual file support")
 def step(context):
-    enableVFSSupport("Enable virtual file support (experimental)...")
+    enableVFSSupport(context, "Enable virtual file support (experimental)...")
 
 
 @Then('the "|any|" button should be available')
 def step(context, btnText):
-    # The enabling/disabling VFS button do not have it's own object
-    # But it is inside the "stack_folderList_QTreeView" object.
-    # So we are clicking at (718, 27) of "stack_folderList_QTreeView" object to enable/disable VFS
-    mouseClick(
-        waitForObjectItem(names.stack_folderList_QTreeView, "_1"),
-        718,
-        27,
-        Qt.NoModifier,
-        Qt.LeftButton,
-    )
+    AccountStatus.openAccountMenu(context)
     waitForObjectItem(names.settings_QMenu, btnText)
 
 
 @Given("the user has enabled virtual file support")
 def step(context):
-    enableVFSSupport("Enable virtual file support (experimental)...")
+    enableVFSSupport(context, "Enable virtual file support (experimental)...")
 
 
 @When("the user disables virtual file support")
 def step(context):
-    # The enabling/disabling VFS button do not have it's own object
-    # But it is inside the "stack_folderList_QTreeView" object.
-    # So we are clicking at (718, 27) of "stack_folderList_QTreeView" object to enable/disable VFS
-    mouseClick(
-        waitForObjectItem(names.stack_folderList_QTreeView, "_1"),
-        733,
-        27,
-        Qt.NoModifier,
-        Qt.LeftButton,
-    )
+    AccountStatus.openAccountMenu(context)
     activateItem(
         waitForObjectItem(names.settings_QMenu, "Disable virtual file support...")
     )
@@ -1132,8 +1184,8 @@ def step(context):
 
 @When('the user accepts the certificate')
 def step(context):
-    clickButton(waitForObject(names.oCC_SslErrorDialog_cbTrustConnect_QCheckBox))
-    clickButton(waitForObject(names.oCC_SslErrorDialog_OK_QPushButton))
+    newAccount = AccountConnectionWizard()
+    newAccount.acceptCertificate()
 
 
 @Then('the lock shown should be closed')
@@ -1149,10 +1201,7 @@ def step(context, errorMsg):
 
 @When(r'the user deletes the (file|folder) "([^"]*)"', regexp=True)
 def step(context, itemType, resource):
-    # deleting the file immediately after it has been synced from the server seems to have some problem.
-    # issue: https://github.com/owncloud/client/issues/8832
-    if itemType == "file":
-        snooze(context.userData['minSyncTimeout'])
+    waitForClientToBeReady(context)
 
     resourcePath = sanitizePath(context.userData['currentUserSyncPath'] + resource)
     if itemType == 'file':
@@ -1194,7 +1243,7 @@ def step(context):
     newAccount = AccountConnectionWizard()
     newAccount.addServer(context)
     test.compare(
-        waitForObjectExists(newAccount.CREDENTIAL_PAGE).visible,
+        waitForObjectExists(newAccount.BASIC_CREDENTIAL_PAGE).visible,
         True,
         "Assert credentials page is visible",
     )
@@ -1206,90 +1255,45 @@ def step(context):
     newAccount.addServer(context)
 
 
-@Given('the user has added the following user credentials:')
-def step(context):
-    newAccount = AccountConnectionWizard()
-    newAccount.addUserCreds(context)
-    test.compare(
-        waitForObjectExists(newAccount.ADVANCE_SETUP_PAGE).visible,
-        True,
-        "Assert setup page is visible",
-    )
-
-
-@Given('the user has opened chose_what_to_sync dialog')
-def step(context):
-    newAccount = AccountConnectionWizard()
-    newAccount.openSyncDialog()
-    test.compare(
-        waitForObjectExists(newAccount.SELECTIVE_SYNC_DIALOG).visible,
-        True,
-        "Assert selective sync dialog is visible",
-    )
-
-
-@When('the user opens chose_what_to_sync dialog')
-def step(context):
-    newAccount = AccountConnectionWizard()
-    newAccount.openSyncDialog()
-
-
 @When('the user selects the following folders to sync:')
 def step(context):
-    newAccount = AccountConnectionWizard()
-    newAccount.selectFoldersToSync(context)
+    syncConnection = SyncConnectionWizard()
+    syncConnection.selectFoldersToSync(context)
+    syncConnection.addSyncConnection()
 
 
-@When('the user selects manual sync folder option')
+@When('the user selects manual sync folder option in advanced section')
 def step(context):
     newAccount = AccountConnectionWizard()
-    newAccount.selectManualSyncFolder()
-
-
-@When('the user connects the account')
-def step(context):
-    newAccount = AccountConnectionWizard()
-    newAccount.connectAccount()
+    newAccount.selectManualSyncFolderOption()
+    newAccount.nextStep()
 
 
 @When('the user sorts the folder list by "|any|"')
 def step(context, headerText):
     headerText = headerText.capitalize()
     if headerText in ["Size", "Name"]:
-        newAccount = AccountConnectionWizard()
-        newAccount.sortBy(headerText)
+        syncConnection = SyncConnectionWizard()
+        syncConnection.sortBy(headerText)
     else:
         raise Exception("Sorting by '" + headerText + "' is not supported.")
 
 
-@Then('the dialog chose_what_to_sync should be visible')
-def step(context):
-    newAccount = AccountConnectionWizard()
-    test.compare(
-        waitForObjectExists(newAccount.SELECTIVE_SYNC_DIALOG).visible,
-        True,
-        "Assert selective sync dialog is visible",
-    )
-
-
 @Then('the sync all checkbox should be checked')
 def step(context):
-    newAccount = AccountConnectionWizard()
-    test.compare(
-        waitForObjectExists(newAccount.SYNC_DIALOG_ROOT_FOLDER).checkState,
-        "checked",
-        "Assert sync all checkbox is checked",
-    )
+    syncConnection = SyncConnectionWizard()
+    state = waitForObject(syncConnection.SYNC_DIALOG_ROOT_FOLDER)["checkState"]
+    test.compare("checked", state, "Sync all checkbox is checked")
 
 
 @Then("the folders should be in the following order:")
 def step(context):
-    newAccount = AccountConnectionWizard()
+    syncConnection = SyncConnectionWizard()
     rowIndex = 0
     for row in context.table[1:]:
         FOLDER_TREE_ROW = {
             "row": rowIndex,
-            "container": newAccount.SYNC_DIALOG_ROOT_FOLDER,
+            "container": syncConnection.SYNC_DIALOG_ROOT_FOLDER,
             "type": "QModelIndex",
         }
         expectedFolder = row[0]
@@ -1376,7 +1380,6 @@ def step(context):
 
 @Then('the collaborators should be listed in the following order:')
 def step(context):
-    shareItem = SharingDialog()
     for index, collaborator in enumerate(context.table[1:], start=1):
         test.compare(
             str(
@@ -1396,26 +1399,80 @@ def step(context):
 
 @Then('VFS enabled baseline image should match the default screenshot')
 def step(context):
-    test.vp("VP_VFS_enabled")
+    if context.userData['ocis']:
+        test.vp("VP_VFS_enabled_oCIS")
+    else:
+        test.vp("VP_VFS_enabled")
 
 
 @Then('VFS enabled baseline image should not match the default screenshot')
 def step(context):
-    test.xvp("VP_VFS_enabled")
+    if context.userData['ocis']:
+        test.xvp("VP_VFS_enabled_oCIS")
+    else:
+        test.xvp("VP_VFS_enabled")
 
 
-@Given('user "|any|" has created the following files inside the sync folder:')
+@When('user "|any|" creates the following files inside the sync folder:')
 def step(context, username):
-    '''
-    Create files without any content
-    '''
     syncPath = getUserSyncPath(context, username)
 
-    # A file is scheduled to be synced but is marked as ignored for 5 seconds. And if we try to sync it, it will fail. So we need to wait for 5 seconds.
-    # https://github.com/owncloud/client/issues/9325
-    snooze(context.userData['minSyncTimeout'])
+    waitForClientToBeReady(context)
 
     for row in context.table[1:]:
         filename = syncPath + row[0]
-        f = open(join(syncPath, filename), "w")
-        f.close()
+        writeFile(join(syncPath, filename), '')
+
+
+@Given(
+    'the user has created a folder "|any|" with "|any|" files each of size "|any|" bytes in temp folder'
+)
+def step(context, foldername, filenumber, filesize):
+    createFolder(context, foldername, isTempFolder=True)
+    filesize = builtins.int(filesize)
+    for i in range(0, builtins.int(filenumber)):
+        filename = f"file{i}.txt"
+        createFileWithSize(context, join(foldername, filename), filesize, True)
+
+
+@When('user "|any|" moves folder "|any|" from the temp folder into the sync folder')
+def step(context, username, foldername):
+    source_dir = join(context.userData['tempFolderPath'], foldername)
+    destination_dir = getUserSyncPath(context, username)
+    shutil.move(source_dir, destination_dir)
+
+
+@Then("credentials wizard should be visible")
+def step(context):
+    if context.userData['ocis']:
+        waitForObject(AccountConnectionWizard.OAUTH_CREDENTIAL_PAGE)
+    else:
+        waitForObject(AccountConnectionWizard.BASIC_CREDENTIAL_PAGE)
+
+
+@When('the user sets the sync path in sync connection wizard')
+def step(context):
+    syncConnection = SyncConnectionWizard()
+    syncConnection.setSyncPathInSyncConnectionWizard(context)
+
+
+@When('the user selects "|any|" as a remote destination folder')
+def step(context, folderName):
+    syncConnection = SyncConnectionWizard()
+    syncConnection.selectRemoteDestinationFolder(folderName)
+
+
+@When('the user selects vfs option in advanced section')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.selectVFSOption()
+
+
+@When(r'^the user (confirms|cancels) the enable experimental vfs option$', regexp=True)
+def step(context, action):
+    newAccount = AccountConnectionWizard()
+    if action == "confirms":
+        newAccount.confirmEnableExperimentalVFSOption()
+    else:
+        newAccount.cancelEnableExperimentalVFSOption()
+    newAccount.nextStep()
