@@ -58,7 +58,7 @@ private slots:
         auto initialState = fakeFolder.currentLocalState();
         int aboutToRemoveAllFilesCalled = 0;
         QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
-            [&](SyncFileItem::Direction dir, std::function<void(bool)> callback) {
+            [&](SyncFileItem::Direction dir, const std::function<void(bool)> &callback) {
                 QCOMPARE(aboutToRemoveAllFilesCalled, 0);
                 aboutToRemoveAllFilesCalled++;
                 QCOMPARE(dir, deleteOnRemote ? SyncFileItem::Down : SyncFileItem::Up);
@@ -67,11 +67,16 @@ private slots:
             });
 
         auto &modifier = deleteOnRemote ? fakeFolder.remoteModifier() : fakeFolder.localModifier();
-        const auto &children = fakeFolder.currentRemoteState().children;
-        for (auto it = children.cbegin(); it != children.cend(); ++it)
+        const auto children = fakeFolder.currentRemoteState().children; // make sure to take a copy, otherwise it's a use-after-free
+        for (auto it = children.cbegin(); it != children.cend(); ++it) {
             modifier.remove(it.key());
-
+        }
+        // Explicitly check that local modifications are successfully applied, and then that the
+        // sync fails. We could calll `applyLocalModificationsAndSync()` instead, but that could
+        // fail when those local modifications fail to apply correctly.
+        QVERIFY(fakeFolder.applyLocalModificationsWithoutSync());
         QVERIFY(!fakeFolder.syncOnce()); // Should fail because we cancel the sync
+
         QCOMPARE(aboutToRemoveAllFilesCalled, 1);
 
         // Next sync should recover all files
@@ -100,7 +105,7 @@ private slots:
 
         int aboutToRemoveAllFilesCalled = 0;
         QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
-            [&](SyncFileItem::Direction dir, std::function<void(bool)> callback) {
+            [&](SyncFileItem::Direction dir, const std::function<void(bool)> &callback) {
                 QCOMPARE(aboutToRemoveAllFilesCalled, 0);
                 aboutToRemoveAllFilesCalled++;
                 QCOMPARE(dir, deleteOnRemote ? SyncFileItem::Down : SyncFileItem::Up);
@@ -108,9 +113,11 @@ private slots:
             });
 
         auto &modifier = deleteOnRemote ? fakeFolder.remoteModifier() : fakeFolder.localModifier();
-        const auto &children = fakeFolder.currentRemoteState().children;
-        for (auto it = children.cbegin(); it != children.cend(); ++it)
+        const auto children = fakeFolder.currentRemoteState().children; // make sure to take a copy, otherwise it's a use-after-free
+        for (auto it = children.cbegin(); it != children.cend(); ++it) {
             modifier.remove(it.key());
+        }
+        QVERIFY(fakeFolder.applyLocalModificationsWithoutSync());
 
         QVERIFY(fakeFolder.syncOnce()); // Should succeed, and all files must then be deleted
 
@@ -141,13 +148,13 @@ private slots:
         const auto &children = fakeFolder.currentRemoteState().children;
         for (auto it = children.cbegin(); it != children.cend(); ++it)
             fakeFolder.syncJournal().avoidRenamesOnNextSync(it.key()); // clears all the fileid and inodes.
-        fakeFolder.localModifier().remove("A/a1");
+        fakeFolder.localModifier().remove(QStringLiteral("A/a1"));
         auto expectedState = fakeFolder.currentLocalState();
         QVERIFY(fakeFolder.syncOnce());
         QCOMPARE(fakeFolder.currentLocalState(), expectedState);
         QCOMPARE(fakeFolder.currentRemoteState(), expectedState);
 
-        fakeFolder.remoteModifier().remove("B/b1");
+        fakeFolder.remoteModifier().remove(QStringLiteral("B/b1"));
         changeAllFileId(fakeFolder.remoteModifier());
         expectedState = fakeFolder.currentRemoteState();
         QVERIFY(fakeFolder.syncOnce());
@@ -161,7 +168,7 @@ private slots:
 
         int aboutToRemoveAllFilesCalled = 0;
         QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
-            [&](SyncFileItem::Direction dir, std::function<void(bool)> callback) {
+            [&](SyncFileItem::Direction dir, const std::function<void(bool)> &callback) {
                 QCOMPARE(aboutToRemoveAllFilesCalled, 0);
                 aboutToRemoveAllFilesCalled++;
                 QCOMPARE(dir, SyncFileItem::Down);
@@ -169,14 +176,16 @@ private slots:
             });
 
         // Some small changes
-        fakeFolder.localModifier().mkdir("Q");
-        fakeFolder.localModifier().insert("Q/q1");
-        fakeFolder.localModifier().appendByte("B/b1");
+        fakeFolder.localModifier().mkdir(QStringLiteral("Q"));
+        fakeFolder.localModifier().insert(QStringLiteral("Q/q1"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("B/b1"));
+        QVERIFY(fakeFolder.applyLocalModificationsWithoutSync());
         QVERIFY(fakeFolder.syncOnce());
         QCOMPARE(aboutToRemoveAllFilesCalled, 0);
 
         // Do some change localy
-        fakeFolder.localModifier().appendByte("A/a1");
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        QVERIFY(fakeFolder.applyLocalModificationsWithoutSync());
 
         // reset the server.
         fakeFolder.remoteModifier() = FileInfo::A12_B12_C12_S12();
@@ -187,20 +196,20 @@ private slots:
 
     }
 
-    void testDataFingetPrint_data()
+    void testDataFingerPrint_data()
     {
         QTest::addColumn<bool>("hasInitialFingerPrint");
         QTest::newRow("initial finger print") << true;
         QTest::newRow("no initial finger print") << false;
     }
 
-    void testDataFingetPrint()
+    void testDataFingerPrint()
     {
         QFETCH(bool, hasInitialFingerPrint);
         FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.remoteModifier().setContents("C/c1", 'N');
-        fakeFolder.remoteModifier().setModTime("C/c1", QDateTime::currentDateTimeUtc().addDays(-2));
-        fakeFolder.remoteModifier().remove("C/c2");
+        fakeFolder.remoteModifier().setContents(QStringLiteral("C/c1"), FileModifier::DefaultFileSize, 'N');
+        fakeFolder.remoteModifier().setModTime(QStringLiteral("C/c1"), QDateTime::currentDateTimeUtc().addDays(-2));
+        fakeFolder.remoteModifier().remove(QStringLiteral("C/c2"));
         if (hasInitialFingerPrint) {
             fakeFolder.remoteModifier().extraDavProperties = "<oc:data-fingerprint>initial_finger_print</oc:data-fingerprint>";
         } else {
@@ -214,10 +223,11 @@ private slots:
             if (verb == "PROPFIND") {
                 auto data = stream->readAll();
                 if (data.contains("data-fingerprint")) {
-                    if (request.url().path().endsWith("webdav/"))
+                    if (request.url().path().endsWith(fakeFolder.account()->davPath())) {
                         ++fingerprintRequests;
-                    else
+                    } else {
                         fingerprintRequests = -10000; // fingerprint queried on incorrect path
+                    }
                 }
             }
             return nullptr;
@@ -233,18 +243,18 @@ private slots:
         /* Simulate a backup restoration */
 
         // A/a1 is an old file
-        fakeFolder.remoteModifier().setContents("A/a1", 'O');
-        fakeFolder.remoteModifier().setModTime("A/a1", QDateTime::currentDateTimeUtc().addDays(-2));
+        fakeFolder.remoteModifier().setContents(QStringLiteral("A/a1"), FileModifier::DefaultFileSize, 'O');
+        fakeFolder.remoteModifier().setModTime(QStringLiteral("A/a1"), QDateTime::currentDateTimeUtc().addDays(-2));
         // B/b1 did not exist at the time of the backup
-        fakeFolder.remoteModifier().remove("B/b1");
+        fakeFolder.remoteModifier().remove(QStringLiteral("B/b1"));
         // B/b2 was uploaded by another user in the mean time.
-        fakeFolder.remoteModifier().setContents("B/b2", 'N');
-        fakeFolder.remoteModifier().setModTime("B/b2", QDateTime::currentDateTimeUtc().addDays(2));
+        fakeFolder.remoteModifier().setContents(QStringLiteral("B/b2"), FileModifier::DefaultFileSize, 'N');
+        fakeFolder.remoteModifier().setModTime(QStringLiteral("B/b2"), QDateTime::currentDateTimeUtc().addDays(2));
 
         // C/c3 was removed since we made the backup
-        fakeFolder.remoteModifier().insert("C/c3_removed");
+        fakeFolder.remoteModifier().insert(QStringLiteral("C/c3_removed"));
         // C/c4 was moved to A/a2 since we made the backup
-        fakeFolder.remoteModifier().rename("A/a2", "C/old_a2_location");
+        fakeFolder.remoteModifier().rename(QStringLiteral("A/a2"), QStringLiteral("C/old_a2_location"));
 
         // The admin sets the data-fingerprint property
         fakeFolder.remoteModifier().extraDavProperties = "<oc:data-fingerprint>new_finger_print</oc:data-fingerprint>";
@@ -254,7 +264,7 @@ private slots:
         auto currentState = fakeFolder.currentLocalState();
         // Altough the local file is kept as a conflict, the server file is downloaded
         QCOMPARE(currentState.find("A/a1")->contentChar, 'O');
-        auto conflict = findConflict(currentState, "A/a1");
+        auto conflict = findConflict(currentState, QStringLiteral("A/a1"));
         QVERIFY(conflict);
         QCOMPARE(conflict->contentChar, 'W');
         fakeFolder.localModifier().remove(conflict->path());
@@ -263,10 +273,11 @@ private slots:
 
         // b2 has the new content (was not restored), since its mode time goes forward in time
         QCOMPARE(currentState.find("B/b2")->contentChar, 'N');
-        conflict = findConflict(currentState, "B/b2");
+        conflict = findConflict(currentState, QStringLiteral("B/b2"));
         QVERIFY(conflict); // Just to be sure, we kept the old file in a conflict
         QCOMPARE(conflict->contentChar, 'W');
         fakeFolder.localModifier().remove(conflict->path());
+        QVERIFY(fakeFolder.applyLocalModificationsWithoutSync());
 
         // We actually do not remove files that technically should have been removed (we don't want data-loss)
         QVERIFY(currentState.find("C/c3_removed"));
@@ -280,19 +291,19 @@ private slots:
 
         int aboutToRemoveAllFilesCalled = 0;
         QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
-            [&](SyncFileItem::Direction , std::function<void(bool)> ) {
+            [&](SyncFileItem::Direction, const std::function<void(bool)> &) {
                 aboutToRemoveAllFilesCalled++;
                 QFAIL("should not be called");
             });
 
         // add a single file
-        fakeFolder.localModifier().insert("hello.txt");
+        fakeFolder.localModifier().insert(QStringLiteral("hello.txt"));
         QVERIFY(fakeFolder.syncOnce());
         QCOMPARE(aboutToRemoveAllFilesCalled, 0);
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // rename it
-        fakeFolder.localModifier().rename("hello.txt", "goodbye.txt");
+        fakeFolder.localModifier().rename(QStringLiteral("hello.txt"), QStringLiteral("goodbye.txt"));
 
         QVERIFY(fakeFolder.syncOnce());
         QCOMPARE(aboutToRemoveAllFilesCalled, 0);
@@ -307,7 +318,7 @@ private slots:
 
         int aboutToRemoveAllFilesCalled = 0;
         QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
-            [&](SyncFileItem::Direction , std::function<void(bool)>) {
+            [&](SyncFileItem::Direction, const std::function<void(bool)> &) {
                 aboutToRemoveAllFilesCalled++;
                 QFAIL("should not be called");
             });
@@ -317,7 +328,7 @@ private slots:
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         fakeFolder.syncEngine().journal()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList,
-            QStringList() << "A/" << "B/" << "C/" << "S/");
+            QStringList() << QStringLiteral("A/") << QStringLiteral("B/") << QStringLiteral("C/") << QStringLiteral("S/"));
 
         QVERIFY(fakeFolder.syncOnce());
         QCOMPARE(fakeFolder.currentLocalState(), FileInfo{}); // all files should be one localy
