@@ -13,7 +13,7 @@
 using namespace OCC;
 
 static void applyPermissionsFromName(FileInfo &info) {
-    static QRegularExpression rx("_PERM_([^_]*)_[^/]*$");
+    static QRegularExpression rx(QStringLiteral("_PERM_([^_]*)_[^/]*$"));
     auto m = rx.match(info.name);
     if (m.hasMatch()) {
         info.permissions = RemotePermissions::fromServerString(m.captured(1));
@@ -39,6 +39,7 @@ static void assertCsyncJournalOk(SyncJournalDb &journal)
 #if defined(Q_OS_WIN) // Make sure the file does not appear in the FileInfo
     FileSystem::setFileHidden(journal.databaseFilePath() + "-shm", true);
 #endif
+    journal.allowReopen();
 }
 
 SyncFileItemPtr findDiscoveryItem(const SyncFileItemSet &spy, const QString &path)
@@ -68,10 +69,33 @@ class TestPermissions : public QObject
     Q_OBJECT
 
 private slots:
+    void initTestCase_data()
+    {
+        QTest::addColumn<Vfs::Mode>("vfsMode");
+        QTest::addColumn<bool>("filesAreDehydrated");
+
+        QTest::newRow("Vfs::Off") << Vfs::Off << false;
+
+        if (isVfsPluginAvailable(Vfs::WindowsCfApi)) {
+            QTest::newRow("Vfs::WindowsCfApi dehydrated") << Vfs::WindowsCfApi << true;
+
+            // TODO: the hydrated version will fail due to an issue in the winvfs plugin, so leave it disabled for now.
+            // QTest::newRow("Vfs::WindowsCfApi hydrated") << Vfs::WindowsCfApi << false;
+        } else if (Utility::isWindows()) {
+            QWARN("Skipping Vfs::WindowsCfApi");
+        }
+    }
 
     void t7pl()
     {
-        FakeFolder fakeFolder{ FileInfo() };
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        if (vfsMode == Vfs::WindowsCfApi) {
+            QSKIP("Known issue: the winvfs plug-in fails to set file permissions");
+        }
+
+        FakeFolder fakeFolder(FileInfo(), vfsMode, filesAreDehydrated);
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // Some of this test depends on the order of discovery. With threading
@@ -81,37 +105,37 @@ private slots:
         syncOpts._parallelNetworkJobs = 1;
         fakeFolder.syncEngine().setSyncOptions(syncOpts);
 
-        const int cannotBeModifiedSize = 133;
-        const int canBeModifiedSize = 144;
+        const auto cannotBeModifiedSize = 133_b;
+        const auto canBeModifiedSize = 144_b;
 
         //create some files
         auto insertIn = [&](const QString &dir) {
-            fakeFolder.remoteModifier().insert(dir + "normalFile_PERM_WVND_.data", 100 );
-            fakeFolder.remoteModifier().insert(dir + "cannotBeRemoved_PERM_WVN_.data", 101 );
-            fakeFolder.remoteModifier().insert(dir + "canBeRemoved_PERM_D_.data", 102 );
+            fakeFolder.remoteModifier().insert(dir + "normalFile_PERM_WVND_.data", 100_b);
+            fakeFolder.remoteModifier().insert(dir + "cannotBeRemoved_PERM_WVN_.data", 101_b);
+            fakeFolder.remoteModifier().insert(dir + "canBeRemoved_PERM_D_.data", 102_b);
             fakeFolder.remoteModifier().insert(dir + "cannotBeModified_PERM_DVN_.data", cannotBeModifiedSize , 'A');
             fakeFolder.remoteModifier().insert(dir + "canBeModified_PERM_W_.data", canBeModifiedSize );
         };
 
         //put them in some directories
-        fakeFolder.remoteModifier().mkdir("normalDirectory_PERM_CKDNV_");
-        insertIn("normalDirectory_PERM_CKDNV_/");
-        fakeFolder.remoteModifier().mkdir("readonlyDirectory_PERM_M_" );
-        insertIn("readonlyDirectory_PERM_M_/" );
-        fakeFolder.remoteModifier().mkdir("readonlyDirectory_PERM_M_/subdir_PERM_CK_");
-        fakeFolder.remoteModifier().mkdir("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_");
-        fakeFolder.remoteModifier().insert("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data", 100);
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("normalDirectory_PERM_CKDNV_"));
+        insertIn(QStringLiteral("normalDirectory_PERM_CKDNV_/"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("readonlyDirectory_PERM_M_"));
+        insertIn(QStringLiteral("readonlyDirectory_PERM_M_/"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data"), 100_b);
         applyPermissionsFromName(fakeFolder.remoteModifier());
 
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         assertCsyncJournalOk(fakeFolder.syncJournal());
         qInfo("Do some changes and see how they propagate");
 
         //1. remove the file than cannot be removed
         //  (they should be recovered)
-        fakeFolder.localModifier().remove("normalDirectory_PERM_CKDNV_/cannotBeRemoved_PERM_WVN_.data");
-        fakeFolder.localModifier().remove("readonlyDirectory_PERM_M_/cannotBeRemoved_PERM_WVN_.data");
+        fakeFolder.localModifier().remove(QStringLiteral("normalDirectory_PERM_CKDNV_/cannotBeRemoved_PERM_WVN_.data"));
+        fakeFolder.localModifier().remove(QStringLiteral("readonlyDirectory_PERM_M_/cannotBeRemoved_PERM_WVN_.data"));
 
         //2. remove the file that can be removed
         //  (they should properly be gone)
@@ -120,8 +144,8 @@ private slots:
             QFile(fakeFolder.localPath() + file).setPermissions(QFile::WriteOwner | QFile::ReadOwner);
             fakeFolder.localModifier().remove(file);
         };
-        removeReadOnly("normalDirectory_PERM_CKDNV_/canBeRemoved_PERM_D_.data");
-        removeReadOnly("readonlyDirectory_PERM_M_/canBeRemoved_PERM_D_.data");
+        removeReadOnly(QStringLiteral("normalDirectory_PERM_CKDNV_/canBeRemoved_PERM_D_.data"));
+        removeReadOnly(QStringLiteral("readonlyDirectory_PERM_M_/canBeRemoved_PERM_D_.data"));
 
         //3. Edit the files that cannot be modified
         //  (they should be recovered, and a conflict shall be created)
@@ -130,21 +154,21 @@ private slots:
             QFile(fakeFolder.localPath() + file).setPermissions(QFile::WriteOwner | QFile::ReadOwner);
             fakeFolder.localModifier().appendByte(file);
         };
-        editReadOnly("normalDirectory_PERM_CKDNV_/cannotBeModified_PERM_DVN_.data");
-        editReadOnly("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data");
+        editReadOnly(QStringLiteral("normalDirectory_PERM_CKDNV_/cannotBeModified_PERM_DVN_.data"));
+        editReadOnly(QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"));
 
         //4. Edit other files
         //  (they should be uploaded)
-        fakeFolder.localModifier().appendByte("normalDirectory_PERM_CKDNV_/canBeModified_PERM_W_.data");
-        fakeFolder.localModifier().appendByte("readonlyDirectory_PERM_M_/canBeModified_PERM_W_.data");
+        fakeFolder.localModifier().appendByte(QStringLiteral("normalDirectory_PERM_CKDNV_/canBeModified_PERM_W_.data"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("readonlyDirectory_PERM_M_/canBeModified_PERM_W_.data"));
 
         //5. Create a new file in a read write folder
         // (should be uploaded)
-        fakeFolder.localModifier().insert("normalDirectory_PERM_CKDNV_/newFile_PERM_WDNV_.data", 106 );
+        fakeFolder.localModifier().insert(QStringLiteral("normalDirectory_PERM_CKDNV_/newFile_PERM_WDNV_.data"), 106_b);
         applyPermissionsFromName(fakeFolder.remoteModifier());
 
         //do the sync
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         assertCsyncJournalOk(fakeFolder.syncJournal());
         auto currentLocalState = fakeFolder.currentLocalState();
 
@@ -163,15 +187,16 @@ private slots:
         QCOMPARE(currentLocalState.find("normalDirectory_PERM_CKDNV_/cannotBeModified_PERM_DVN_.data")->contentSize, cannotBeModifiedSize);
         QCOMPARE(currentLocalState.find("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data")->contentSize, cannotBeModifiedSize);
         // and conflict created
-        auto c1 = findConflict(currentLocalState, "normalDirectory_PERM_CKDNV_/cannotBeModified_PERM_DVN_.data");
+        auto c1 = findConflict(currentLocalState, QStringLiteral("normalDirectory_PERM_CKDNV_/cannotBeModified_PERM_DVN_.data"));
         QVERIFY(c1);
         QCOMPARE(c1->contentSize, cannotBeModifiedSize + 1);
-        auto c2 = findConflict(currentLocalState, "readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data");
+        auto c2 = findConflict(currentLocalState, QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"));
         QVERIFY(c2);
         QCOMPARE(c2->contentSize, cannotBeModifiedSize + 1);
         // remove the conflicts for the next state comparison
         fakeFolder.localModifier().remove(c1->path());
         fakeFolder.localModifier().remove(c2->path());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         //4. File should be updated, that's tested by assertLocalAndRemoteDir
         QCOMPARE(currentLocalState.find("normalDirectory_PERM_CKDNV_/canBeModified_PERM_W_.data")->contentSize, canBeModifiedSize + 1);
@@ -188,11 +213,11 @@ private slots:
 
         //6. Create a new file in a read only folder
         // (they should not be uploaded)
-        fakeFolder.localModifier().insert("readonlyDirectory_PERM_M_/newFile_PERM_WDNV_.data", 105 );
+        fakeFolder.localModifier().insert(QStringLiteral("readonlyDirectory_PERM_M_/newFile_PERM_WDNV_.data"), 105_b);
 
         applyPermissionsFromName(fakeFolder.remoteModifier());
         // error: can't upload to readonly
-        QVERIFY(!fakeFolder.syncOnce());
+        QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
 
         assertCsyncJournalOk(fakeFolder.syncJournal());
         currentLocalState = fakeFolder.currentLocalState();
@@ -202,7 +227,8 @@ private slots:
         QVERIFY(currentLocalState.find("readonlyDirectory_PERM_M_/newFile_PERM_WDNV_.data"));
         QVERIFY(!fakeFolder.currentRemoteState().find("readonlyDirectory_PERM_M_/newFile_PERM_WDNV_.data"));
         // remove it so next test succeed.
-        fakeFolder.localModifier().remove("readonlyDirectory_PERM_M_/newFile_PERM_WDNV_.data");
+        fakeFolder.localModifier().remove(QStringLiteral("readonlyDirectory_PERM_M_/newFile_PERM_WDNV_.data"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         // Both side should still be the same
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
@@ -210,9 +236,9 @@ private slots:
         //######################################################################
         qInfo( "remove the read only directory" );
         // -> It must be recovered
-        fakeFolder.localModifier().remove("readonlyDirectory_PERM_M_");
+        fakeFolder.localModifier().remove(QStringLiteral("readonlyDirectory_PERM_M_"));
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         assertCsyncJournalOk(fakeFolder.syncJournal());
         currentLocalState = fakeFolder.currentLocalState();
         QVERIFY(currentLocalState.find("readonlyDirectory_PERM_M_/cannotBeRemoved_PERM_WVN_.data"));
@@ -222,10 +248,10 @@ private slots:
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // restore
-        fakeFolder.remoteModifier().mkdir("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_");
-        fakeFolder.remoteModifier().insert("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data");
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data"));
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
 
@@ -234,9 +260,9 @@ private slots:
 
         //Missing directory should be restored
         //new directory should be uploaded
-        fakeFolder.localModifier().rename("readonlyDirectory_PERM_M_/subdir_PERM_CK_", "normalDirectory_PERM_CKDNV_/subdir_PERM_CKDNV_");
+        fakeFolder.localModifier().rename(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_"), QStringLiteral("normalDirectory_PERM_CKDNV_/subdir_PERM_CKDNV_"));
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         currentLocalState = fakeFolder.currentLocalState();
 
         // old name restored
@@ -251,10 +277,10 @@ private slots:
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // restore for further tests
-        fakeFolder.remoteModifier().mkdir("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_");
-        fakeFolder.remoteModifier().insert("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data");
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data"));
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
 
@@ -263,7 +289,7 @@ private slots:
 
         // do a sync to update the database
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         assertCsyncJournalOk(fakeFolder.syncJournal());
 
         QVERIFY(fakeFolder.currentLocalState().find("readonlyDirectory_PERM_M_/subdir_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data" ));
@@ -271,13 +297,13 @@ private slots:
         //1. rename a directory in a read only folder
         //Missing directory should be restored
         //new directory should stay but not be uploaded
-        fakeFolder.localModifier().rename("readonlyDirectory_PERM_M_/subdir_PERM_CK_", "readonlyDirectory_PERM_M_/newname_PERM_CK_"  );
+        fakeFolder.localModifier().rename(QStringLiteral("readonlyDirectory_PERM_M_/subdir_PERM_CK_"), QStringLiteral("readonlyDirectory_PERM_M_/newname_PERM_CK_"));
 
         //2. move a directory from read to read only  (move the directory from previous step)
-        fakeFolder.localModifier().rename("normalDirectory_PERM_CKDNV_/subdir_PERM_CKDNV_", "readonlyDirectory_PERM_M_/moved_PERM_CK_" );
+        fakeFolder.localModifier().rename(QStringLiteral("normalDirectory_PERM_CKDNV_/subdir_PERM_CKDNV_"), QStringLiteral("readonlyDirectory_PERM_M_/moved_PERM_CK_"));
 
         // error: can't upload to readonly!
-        QVERIFY(!fakeFolder.syncOnce());
+        QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
         currentLocalState = fakeFolder.currentLocalState();
 
         //1.
@@ -288,7 +314,7 @@ private slots:
         // new still exist
         QVERIFY(currentLocalState.find("readonlyDirectory_PERM_M_/newname_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data" ));
         // but is not on server: so remove it localy for the future comarison
-        fakeFolder.localModifier().remove("readonlyDirectory_PERM_M_/newname_PERM_CK_");
+        fakeFolder.localModifier().remove(QStringLiteral("readonlyDirectory_PERM_M_/newname_PERM_CK_"));
 
         //2.
         // old removed
@@ -298,8 +324,9 @@ private slots:
         // new still there
         QVERIFY(currentLocalState.find("readonlyDirectory_PERM_M_/moved_PERM_CK_/subsubdir_PERM_CKDNV_/normalFile_PERM_WVND_.data" ));
         //but not on server
-        fakeFolder.localModifier().remove("readonlyDirectory_PERM_M_/moved_PERM_CK_");
-        fakeFolder.remoteModifier().remove("normalDirectory_PERM_CKDNV_/subdir_PERM_CKDNV_");
+        fakeFolder.localModifier().remove(QStringLiteral("readonlyDirectory_PERM_M_/moved_PERM_CK_"));
+        fakeFolder.remoteModifier().remove(QStringLiteral("normalDirectory_PERM_CKDNV_/subdir_PERM_CKDNV_"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
@@ -307,32 +334,33 @@ private slots:
         //######################################################################
         qInfo( "multiple restores of a file create different conflict files" );
 
-        fakeFolder.remoteModifier().insert("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data");
+        fakeFolder.remoteModifier().insert(QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"));
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        editReadOnly("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data");
-        fakeFolder.localModifier().setContents("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data", 's');
+        editReadOnly(QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"));
+        fakeFolder.localModifier().setContents("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data", FileModifier::DefaultFileSize, 's');
         //do the sync
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         assertCsyncJournalOk(fakeFolder.syncJournal());
 
         QThread::sleep(1); // make sure changes have different mtime
-        editReadOnly("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data");
-        fakeFolder.localModifier().setContents("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data", 'd');
+        editReadOnly(QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"));
+        fakeFolder.localModifier().setContents(QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"), FileModifier::DefaultFileSize, 'd');
 
         //do the sync
         applyPermissionsFromName(fakeFolder.remoteModifier());
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         assertCsyncJournalOk(fakeFolder.syncJournal());
 
         // there should be two conflict files
         currentLocalState = fakeFolder.currentLocalState();
         int count = 0;
-        while (auto i = findConflict(currentLocalState, "readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data")) {
+        while (auto i = findConflict(currentLocalState, QStringLiteral("readonlyDirectory_PERM_M_/cannotBeModified_PERM_DVN_.data"))) {
             QVERIFY((i->contentChar == 's') || (i->contentChar == 'd'));
             fakeFolder.localModifier().remove(i->path());
+            QVERIFY(fakeFolder.applyLocalModificationsWithoutSync());
             currentLocalState = fakeFolder.currentLocalState();
             count++;
         }
@@ -350,7 +378,14 @@ private slots:
     // What happens if the source can't be moved or the target can't be created?
     void testForbiddenMoves()
     {
-        FakeFolder fakeFolder{FileInfo{}};
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        if (vfsMode == Vfs::WindowsCfApi) {
+            QSKIP("Known issue: the winvfs plug-in fails to set file permissions");
+        }
+
+        FakeFolder fakeFolder(FileInfo(), vfsMode, filesAreDehydrated);
 
         // Some of this test depends on the order of discovery. With threading
         // that order becomes effectively random, but we want to make sure to test
@@ -361,56 +396,56 @@ private slots:
 
         auto &lm = fakeFolder.localModifier();
         auto &rm = fakeFolder.remoteModifier();
-        rm.mkdir("allowed");
-        rm.mkdir("norename");
-        rm.mkdir("nomove");
-        rm.mkdir("nocreatefile");
-        rm.mkdir("nocreatedir");
-        rm.mkdir("zallowed"); // order of discovery matters
+        rm.mkdir(QStringLiteral("allowed"));
+        rm.mkdir(QStringLiteral("norename"));
+        rm.mkdir(QStringLiteral("nomove"));
+        rm.mkdir(QStringLiteral("nocreatefile"));
+        rm.mkdir(QStringLiteral("nocreatedir"));
+        rm.mkdir(QStringLiteral("zallowed")); // order of discovery matters
 
-        rm.mkdir("allowed/sub");
-        rm.mkdir("allowed/sub2");
-        rm.insert("allowed/file");
-        rm.insert("allowed/sub/file");
-        rm.insert("allowed/sub2/file");
-        rm.mkdir("norename/sub");
-        rm.insert("norename/file");
-        rm.insert("norename/sub/file");
-        rm.mkdir("nomove/sub");
-        rm.insert("nomove/file");
-        rm.insert("nomove/sub/file");
-        rm.mkdir("zallowed/sub");
-        rm.mkdir("zallowed/sub2");
-        rm.insert("zallowed/file");
-        rm.insert("zallowed/sub/file");
-        rm.insert("zallowed/sub2/file");
+        rm.mkdir(QStringLiteral("allowed/sub"));
+        rm.mkdir(QStringLiteral("allowed/sub2"));
+        rm.insert(QStringLiteral("allowed/file"));
+        rm.insert(QStringLiteral("allowed/sub/file"));
+        rm.insert(QStringLiteral("allowed/sub2/file"));
+        rm.mkdir(QStringLiteral("norename/sub"));
+        rm.insert(QStringLiteral("norename/file"));
+        rm.insert(QStringLiteral("norename/sub/file"));
+        rm.mkdir(QStringLiteral("nomove/sub"));
+        rm.insert(QStringLiteral("nomove/file"));
+        rm.insert(QStringLiteral("nomove/sub/file"));
+        rm.mkdir(QStringLiteral("zallowed/sub"));
+        rm.mkdir(QStringLiteral("zallowed/sub2"));
+        rm.insert(QStringLiteral("zallowed/file"));
+        rm.insert(QStringLiteral("zallowed/sub/file"));
+        rm.insert(QStringLiteral("zallowed/sub2/file"));
 
-        setAllPerm(rm.find("norename"), RemotePermissions::fromServerString("WDVCK"));
-        setAllPerm(rm.find("nomove"), RemotePermissions::fromServerString("WDNCK"));
-        setAllPerm(rm.find("nocreatefile"), RemotePermissions::fromServerString("WDNVK"));
-        setAllPerm(rm.find("nocreatedir"), RemotePermissions::fromServerString("WDNVC"));
+        setAllPerm(rm.find("norename"), RemotePermissions::fromServerString(QStringLiteral("WDVCK")));
+        setAllPerm(rm.find("nomove"), RemotePermissions::fromServerString(QStringLiteral("WDNCK")));
+        setAllPerm(rm.find("nocreatefile"), RemotePermissions::fromServerString(QStringLiteral("WDNVK")));
+        setAllPerm(rm.find("nocreatedir"), RemotePermissions::fromServerString(QStringLiteral("WDNVC")));
 
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         // Renaming errors
-        lm.rename("norename/file", "norename/file_renamed");
-        lm.rename("norename/sub", "norename/sub_renamed");
+        lm.rename(QStringLiteral("norename/file"), QStringLiteral("norename/file_renamed"));
+        lm.rename(QStringLiteral("norename/sub"), QStringLiteral("norename/sub_renamed"));
         // Moving errors
-        lm.rename("nomove/file", "allowed/file_moved");
-        lm.rename("nomove/sub", "allowed/sub_moved");
+        lm.rename(QStringLiteral("nomove/file"), QStringLiteral("allowed/file_moved"));
+        lm.rename(QStringLiteral("nomove/sub"), QStringLiteral("allowed/sub_moved"));
         // Createfile errors
-        lm.rename("allowed/file", "nocreatefile/file");
-        lm.rename("zallowed/file", "nocreatefile/zfile");
-        lm.rename("allowed/sub", "nocreatefile/sub"); // TODO: probably forbidden because it contains file children?
+        lm.rename(QStringLiteral("allowed/file"), QStringLiteral("nocreatefile/file"));
+        lm.rename(QStringLiteral("zallowed/file"), QStringLiteral("nocreatefile/zfile"));
+        lm.rename(QStringLiteral("allowed/sub"), QStringLiteral("nocreatefile/sub")); // TODO: probably forbidden because it contains file children?
         // Createdir errors
-        lm.rename("allowed/sub2", "nocreatedir/sub2");
-        lm.rename("zallowed/sub2", "nocreatedir/zsub2");
+        lm.rename(QStringLiteral("allowed/sub2"), QStringLiteral("nocreatedir/sub2"));
+        lm.rename(QStringLiteral("zallowed/sub2"), QStringLiteral("nocreatedir/zsub2"));
 
         // also hook into discovery!!
         SyncFileItemSet discovery;
         connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToPropagate, this, [&discovery](auto v) { discovery = v; });
         ItemCompletedSpy completeSpy(fakeFolder);
-        QVERIFY(!fakeFolder.syncOnce());
+        QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
 
         // if renaming doesn't work, just delete+create
         QVERIFY(itemInstruction(completeSpy, "norename/file", CSYNC_INSTRUCTION_REMOVE));
@@ -449,7 +484,7 @@ private slots:
         // A follow-up sync will restore allowed/file and allowed/sub2 and maintain the nocreatedir/file errors
         completeSpy.clear();
         QCOMPARE(fakeFolder.syncJournal().wipeErrorBlacklist(), 4);
-        QVERIFY(!fakeFolder.syncOnce());
+        QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
 
         QVERIFY(itemInstruction(completeSpy, "nocreatefile/file", CSYNC_INSTRUCTION_ERROR));
         QVERIFY(itemInstruction(completeSpy, "nocreatefile/zfile", CSYNC_INSTRUCTION_ERROR));
@@ -470,7 +505,14 @@ private slots:
 
     // Test for issue #7293
     void testAllowedMoveForbiddenDelete() {
-         FakeFolder fakeFolder{FileInfo{}};
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        if (vfsMode == Vfs::WindowsCfApi) {
+            QSKIP("Known issue: the winvfs plug-in fails to set file permissions");
+        }
+
+        FakeFolder fakeFolder(FileInfo(), vfsMode, filesAreDehydrated);
 
         // Some of this test depends on the order of discovery. With threading
         // that order becomes effectively random, but we want to make sure to test
@@ -481,35 +523,32 @@ private slots:
 
         auto &lm = fakeFolder.localModifier();
         auto &rm = fakeFolder.remoteModifier();
-        rm.mkdir("changeonly");
-        rm.mkdir("changeonly/sub1");
-        rm.insert("changeonly/sub1/file1");
-        rm.insert("changeonly/sub1/filetorname1a");
-        rm.insert("changeonly/sub1/filetorname1z");
-        rm.mkdir("changeonly/sub2");
-        rm.insert("changeonly/sub2/file2");
-        rm.insert("changeonly/sub2/filetorname2a");
-        rm.insert("changeonly/sub2/filetorname2z");
+        rm.mkdir(QStringLiteral("changeonly"));
+        rm.mkdir(QStringLiteral("changeonly/sub1"));
+        rm.insert(QStringLiteral("changeonly/sub1/file1"));
+        rm.insert(QStringLiteral("changeonly/sub1/filetorname1a"));
+        rm.insert(QStringLiteral("changeonly/sub1/filetorname1z"));
+        rm.mkdir(QStringLiteral("changeonly/sub2"));
+        rm.insert(QStringLiteral("changeonly/sub2/file2"));
+        rm.insert(QStringLiteral("changeonly/sub2/filetorname2a"));
+        rm.insert(QStringLiteral("changeonly/sub2/filetorname2z"));
 
-        setAllPerm(rm.find("changeonly"), RemotePermissions::fromServerString("NSV"));
+        setAllPerm(rm.find("changeonly"), RemotePermissions::fromServerString(QStringLiteral("NSV")));
 
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        lm.rename("changeonly/sub1/filetorname1a", "changeonly/sub1/aaa1_renamed");
-        lm.rename("changeonly/sub1/filetorname1z", "changeonly/sub1/zzz1_renamed");
+        lm.rename(QStringLiteral("changeonly/sub1/filetorname1a"), QStringLiteral("changeonly/sub1/aaa1_renamed"));
+        lm.rename(QStringLiteral("changeonly/sub1/filetorname1z"), QStringLiteral("changeonly/sub1/zzz1_renamed"));
 
-        lm.rename("changeonly/sub2/filetorname2a", "changeonly/sub2/aaa2_renamed");
-        lm.rename("changeonly/sub2/filetorname2z", "changeonly/sub2/zzz2_renamed");
+        lm.rename(QStringLiteral("changeonly/sub2/filetorname2a"), QStringLiteral("changeonly/sub2/aaa2_renamed"));
+        lm.rename(QStringLiteral("changeonly/sub2/filetorname2z"), QStringLiteral("changeonly/sub2/zzz2_renamed"));
 
-        lm.rename("changeonly/sub1", "changeonly/aaa");
-        lm.rename("changeonly/sub2", "changeonly/zzz");
+        lm.rename(QStringLiteral("changeonly/sub1"), QStringLiteral("changeonly/aaa"));
+        lm.rename(QStringLiteral("changeonly/sub2"), QStringLiteral("changeonly/zzz"));
 
 
-        auto expectedState = fakeFolder.currentLocalState();
-
-        QVERIFY(fakeFolder.syncOnce());
-        QCOMPARE(fakeFolder.currentLocalState(), expectedState);
-        QCOMPARE(fakeFolder.currentRemoteState(), expectedState);
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 };
 
