@@ -7,24 +7,29 @@
 #pragma once
 
 #include "account.h"
-#include "creds/abstractcredentials.h"
-#include "logger.h"
-#include "filesystem.h"
-#include "folder.h"
-#include "syncengine.h"
+#include "common/filesystembase.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/vfs.h"
+#include "creds/abstractcredentials.h"
 #include "csync_exclude.h"
+#include "filesystem.h"
+#include "folder.h"
+#include "logger.h"
+#include "syncengine.h"
+#include "testutils.h"
 #include <cstring>
 
 #include <QDir>
-#include <QNetworkReply>
 #include <QMap>
+#include <QNetworkReply>
+#include <QTimer>
 #include <QtTest>
 #include <cookiejar.h>
-#include <QTimer>
 
+#include <chrono>
+
+using namespace OCC::FileSystem::SizeLiterals;
 /*
  * TODO: In theory we should use QVERIFY instead of Q_ASSERT for testing, but this
  * only works when directly called from a QTest :-(
@@ -50,11 +55,11 @@ inline QString getFilePathFromUrl(const QUrl &url)
 
 inline QByteArray generateEtag()
 {
-    return QByteArray::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(qrand(), 16);
+    return QByteArray::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(QRandomGenerator::global()->generate(), 16);
 }
 inline QByteArray generateFileId()
 {
-    return QByteArray::number(qrand(), 16);
+    return QByteArray::number(QRandomGenerator::global()->generate(), 16);
 }
 
 class PathComponents : public QStringList
@@ -75,23 +80,54 @@ public:
     QString fileName() const { return last(); }
 };
 
+/**
+ * @brief The FileModifier class defines the interface for both the local on-disk modifier and the
+ * remote in-memory modifier.
+ */
 class FileModifier
 {
 public:
+    static constexpr auto DefaultFileSize = 64_b;
+    static constexpr char DefaultContentChar = 'X';
+
     virtual ~FileModifier() { }
     virtual void remove(const QString &relativePath) = 0;
-    virtual void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') = 0;
-    virtual void setContents(const QString &relativePath, char contentChar) = 0;
-    virtual void appendByte(const QString &relativePath, char contentChar = 0) = 0;
-    virtual void modifyByte(const QString &relativePath, quint64 offset, char contentChar) = 0;
+    virtual void insert(const QString &relativePath, quint64 size = DefaultFileSize, char contentChar = DefaultContentChar) = 0;
+
+    virtual void setContents(const QString &relativePath, quint64 newSize, char contentChar = DefaultContentChar) = 0;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void setContents(const QString &relativePath, T newSize, char contentChar = DefaultContentChar) = delete;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void insert(const QString &relativePath, T size, char contentChar = DefaultContentChar) = delete;
+
+    virtual void appendByte(const QString &relativePath, char contentChar = DefaultContentChar) = 0;
     virtual void mkdir(const QString &relativePath) = 0;
     virtual void rename(const QString &relativePath, const QString &relativeDestinationDirectory) = 0;
     virtual void setModTime(const QString &relativePath, const QDateTime &modTime) = 0;
 };
 
+class FakeFolder;
+
+/**
+ * @brief The DiskFileModifier class is a FileModifier for files on the local disk.
+ *
+ * It uses a helper program (`test_helper`) to do the actual modifications. When running without a
+ * VFS, or when running with suffix-VFS, the changes are done before doing a sync. When running
+ * with a VFS, the OS will do callbacks to the test program. So in this case the modifications are
+ * done while the event loop is running, in order to process those OS callbacks.
+ *
+ * Note: these actions do NOT read content from files on disk: if the file would be dehydrated,
+ * reading will cause a re-hydration, which means network requests, and some tests explicitly check
+ * for the amount of requests.
+ */
 class DiskFileModifier : public FileModifier
 {
     QDir _rootDir;
+    QStringList _processArguments;
 
 public:
     DiskFileModifier(const QString &rootDirPath)
@@ -99,18 +135,44 @@ public:
     {
     }
     void remove(const QString &relativePath) override;
-    void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') override;
-    void setContents(const QString &relativePath, char contentChar) override;
-    void appendByte(const QString &relativePath, char contentChar) override;
-    void modifyByte(const QString &relativePath, quint64 offset, char contentChar) override;
+    void insert(const QString &relativePath, quint64 size = DefaultFileSize, char contentChar = DefaultContentChar) override;
+    void setContents(const QString &relativePath, quint64 newSize, char contentChar = DefaultContentChar) override;
+    void appendByte(const QString &relativePath, char contentChar = DefaultContentChar) override;
 
     void mkdir(const QString &relativePath) override;
     void rename(const QString &from, const QString &to) override;
     void setModTime(const QString &relativePath, const QDateTime &modTime) override;
+
+    bool applyModifications();
+    Q_REQUIRED_RESULT bool applyModificationsAndSync(FakeFolder &ff, OCC::Vfs::Mode mode);
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void setContents(const QString &relativePath, T newSize, char contentChar = DefaultContentChar) = delete;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void insert(const QString &relativePath, T size, char contentChar = DefaultContentChar) = delete;
 };
 
+static inline qint64 defaultLastModified()
+{
+    auto precise = QDateTime::currentDateTimeUtc().addDays(-7);
+    time_t timeInSeconds = OCC::Utility::qDateTimeToTime_t(precise);
+    return timeInSeconds;
+}
+
+/**
+ * @brief The FileInfo class represents the remotely stored (on-server) content. To be able to
+ * modify the content, this class is also the remote \c FileModifier.
+ */
 class FileInfo : public FileModifier
 {
+    /// FIXME: we should make it explicit in the construtor if we're talking about a hydrated or a dehydrated file!
+    /// FIXME: this class is both a remote folder, as the root folder which in turn is the remote FileModifier.
+    ///        This is a mess: unifying remote files/folders is ok, but because it's also the remote root (which
+    ///        should implement the FileModifier), this means that each single remote file/folder also implements
+    ///        this interface.
 public:
     static FileInfo A12_B12_C12_S12();
 
@@ -119,32 +181,32 @@ public:
         : name { name }
     {
     }
-    FileInfo(const QString &name, qint64 size)
+    FileInfo(const QString &name, quint64 size)
         : name { name }
         , isDir { false }
-        , size { size }
+        , fileSize(size)
+        , contentSize { size }
     {
     }
-    FileInfo(const QString &name, qint64 size, char contentChar)
+    FileInfo(const QString &name, quint64 size, char contentChar)
         : name { name }
         , isDir { false }
-        , size { size }
+        , fileSize(size)
+        , contentSize { size }
         , contentChar { contentChar }
     {
     }
     FileInfo(const QString &name, const std::initializer_list<FileInfo> &children);
 
-    void addChild(const FileInfo &info);
+    FileInfo &addChild(const FileInfo &info);
 
     void remove(const QString &relativePath) override;
 
-    void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') override;
+    void insert(const QString &relativePath, quint64 size = DefaultFileSize, char contentChar = DefaultContentChar) override;
 
-    void setContents(const QString &relativePath, char contentChar) override;
+    void setContents(const QString &relativePath, quint64 newSize, char contentChar = DefaultContentChar) override;
 
-    void appendByte(const QString &relativePath, char contentChar = 0) override;
-
-    void modifyByte(const QString &relativePath, quint64 offset, char contentChar) override;
+    void appendByte(const QString &relativePath, char contentChar = DefaultContentChar) override;
 
     void mkdir(const QString &relativePath) override;
 
@@ -152,24 +214,57 @@ public:
 
     void setModTime(const QString &relativePath, const QDateTime &modTime) override;
 
+    /// Return a pointer to the FileInfo, or a nullptr if it doesn't exist
     FileInfo *find(PathComponents pathComponents, const bool invalidateEtags = false);
 
     FileInfo *createDir(const QString &relativePath);
 
-    FileInfo *create(const QString &relativePath, qint64 size, char contentChar);
+    FileInfo *create(const QString &relativePath, quint64 size, char contentChar);
 
     bool operator<(const FileInfo &other) const
     {
         return name < other.name;
     }
 
-    bool operator==(const FileInfo &other) const;
+    enum CompareWhat {
+        CompareLastModified,
+        IgnoreLastModified,
+    };
+
+    bool operator==(const FileInfo &other) const { return equals(other, CompareLastModified); }
+
+    bool equals(const FileInfo &other, CompareWhat compareWhat) const;
 
     bool operator!=(const FileInfo &other) const
     {
         return !operator==(other);
     }
 
+    QDateTime lastModified() const
+    {
+        return QDateTime::fromSecsSinceEpoch(_lastModifiedInSecondsUTC, Qt::LocalTime);
+    }
+
+    QDateTime lastModifiedInUtc() const
+    {
+        return QDateTime::fromSecsSinceEpoch(_lastModifiedInSecondsUTC, Qt::UTC);
+    }
+
+    void setLastModified(const QDateTime &t)
+    {
+        _lastModifiedInSecondsUTC = t.toSecsSinceEpoch();
+    }
+
+    qint64 lastModifiedInSecondsUTC() const
+    {
+        return _lastModifiedInSecondsUTC;
+    }
+
+    void setLastModifiedFromSecondsUTC(qint64 utcSecs)
+    {
+        _lastModifiedInSecondsUTC = utcSecs;
+    }
+
     QString path() const;
     QString absolutePath() const;
 
@@ -179,13 +274,15 @@ public:
     bool isDir = true;
     bool isShared = false;
     OCC::RemotePermissions permissions; // When uset, defaults to everything
-    QDateTime lastModified = QDateTime::currentDateTimeUtc().addDays(-7);
+    qint64 _lastModifiedInSecondsUTC = defaultLastModified();
     QByteArray etag = generateEtag();
     QByteArray fileId = generateFileId();
     QByteArray checksums;
     QByteArray extraDavProperties;
-    qint64 size = 0;
+    quint64 fileSize = 0;
+    quint64 contentSize = 0;
     char contentChar = 'W';
+    bool isDehydratedPlaceholder = false;
 
     // Sorted by name to be able to compare trees
     QMap<QString, FileInfo> children;
@@ -195,16 +292,34 @@ public:
 
     friend inline QDebug operator<<(QDebug dbg, const FileInfo &fi)
     {
-        return dbg << "{ " << fi.path() << ": " << fi.children;
+        return dbg.nospace().noquote()
+            << "{ '" << fi.path() << "': "
+            << ", isDir:" << fi.isDir
+            << QStringLiteral(", lastModified: %1 (%2)").arg(QString::number(fi._lastModifiedInSecondsUTC), fi.lastModifiedInUtc().toString())
+            << ", fileSize:" << fi.fileSize
+            << ", contentSize:" << fi.contentSize
+            << QStringLiteral(", contentChar: 0x%1").arg(QString::number(int(fi.contentChar), 16))
+            << ", isDehydratedPlaceholder:" << fi.isDehydratedPlaceholder
+            << ", children:" << fi.children
+            << " }";
     }
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void setContents(const QString &relativePath, T newSize, char contentChar = DefaultContentChar) = delete;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void insert(const QString &relativePath, T size, char contentChar = DefaultContentChar) = delete;
 };
 
 class FakeReply : public QNetworkReply
 {
     Q_OBJECT
+
 public:
     FakeReply(QObject *parent);
-    virtual ~FakeReply();
+    virtual ~FakeReply() override;
 
     // useful to be public for testing
     using QNetworkReply::setRawHeader;
@@ -213,6 +328,7 @@ public:
 class FakePropfindReply : public FakeReply
 {
     Q_OBJECT
+
 public:
     QByteArray payload;
 
@@ -231,7 +347,6 @@ public:
 class FakePutReply : public FakeReply
 {
     Q_OBJECT
-    FileInfo *fileInfo;
 
 public:
     FakePutReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, const QByteArray &putPayload, QObject *parent);
@@ -242,12 +357,14 @@ public:
 
     void abort() override;
     qint64 readData(char *, qint64) override { return 0; }
+
+private:
+    FileInfo *fileInfo;
 };
 
 class FakeMkcolReply : public FakeReply
 {
     Q_OBJECT
-    FileInfo *fileInfo;
 
 public:
     FakeMkcolReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
@@ -256,11 +373,15 @@ public:
 
     void abort() override { }
     qint64 readData(char *, qint64) override { return 0; }
+
+private:
+    FileInfo *fileInfo;
 };
 
 class FakeDeleteReply : public FakeReply
 {
     Q_OBJECT
+
 public:
     FakeDeleteReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
@@ -273,6 +394,7 @@ public:
 class FakeMoveReply : public FakeReply
 {
     Q_OBJECT
+
 public:
     FakeMoveReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
@@ -285,39 +407,30 @@ public:
 class FakeGetReply : public FakeReply
 {
     Q_OBJECT
+
 public:
+    enum class State {
+        Ok,
+        Aborted,
+        FileNotFound,
+    };
+
     const FileInfo *fileInfo;
     char payload;
-    int size;
-    bool aborted = false;
+    qint64 size;
+    State state = State::Ok;
+    const std::pair<qint64, qint64> _range;
 
     FakeGetReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
     Q_INVOKABLE void respond();
 
     void abort() override;
-    qint64 bytesAvailable() const override;
+    virtual qint64 bytesAvailable() const override;
 
-    qint64 readData(char *data, qint64 maxlen) override;
-};
+    virtual qint64 readData(char *data, qint64 maxlen) override;
 
-class FakeGetWithDataReply : public FakeReply
-{
-    Q_OBJECT
-public:
-    const FileInfo *fileInfo;
-    QByteArray payload;
-    quint64 offset = 0;
-    bool aborted = false;
-
-    FakeGetWithDataReply(FileInfo &remoteRootFileInfo, const QByteArray &data, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
-
-    Q_INVOKABLE void respond();
-
-    void abort() override;
-    qint64 bytesAvailable() const override;
-
-    qint64 readData(char *data, qint64 maxlen) override;
+    static std::pair<qint64, qint64> parseRange(const QNetworkRequest &request);
 };
 
 class FakeChunkMoveReply : public FakeReply
@@ -398,12 +511,12 @@ class DelayedReply : public OriginalReply
 {
 public:
     template <typename... Args>
-    explicit DelayedReply(quint64 delayMS, Args &&...args)
+    explicit DelayedReply(const std::chrono::milliseconds delayMS, Args &&...args)
         : OriginalReply(std::forward<Args>(args)...)
         , _delayMs(delayMS)
     {
     }
-    quint64 _delayMs;
+    std::chrono::milliseconds _delayMs;
 
     void respond() override
     {
@@ -414,7 +527,7 @@ public:
     }
 };
 
-class FakeQNAM : public QNetworkAccessManager
+class FakeAM : public OCC::AccessManager
 {
 public:
     using Override = std::function<QNetworkReply *(Operation, const QNetworkRequest &, QIODevice *)>;
@@ -428,7 +541,7 @@ private:
     Override _override;
 
 public:
-    FakeQNAM(FileInfo initialRoot);
+    FakeAM(FileInfo initialRoot);
     FileInfo &currentRemoteState() { return _remoteRootFileInfo; }
     FileInfo &uploadState() { return _uploadFileInfo; }
 
@@ -443,16 +556,15 @@ protected:
 
 class FakeCredentials : public OCC::AbstractCredentials
 {
-    QNetworkAccessManager *_qnam;
-
 public:
-    FakeCredentials(QNetworkAccessManager *qnam)
-        : _qnam { qnam }
+    FakeCredentials(OCC::AccessManager *am)
+        : _am { am }
     {
     }
+
     QString authType() const override { return QStringLiteral("test"); }
     QString user() const override { return QStringLiteral("admin"); }
-    QNetworkAccessManager *createQNAM() const override { return _qnam; }
+    OCC::AccessManager *createAM() const override { return _am; }
     bool ready() const override { return true; }
     void fetchFromKeychain() override { }
     void askFromUser() override { }
@@ -460,20 +572,23 @@ public:
     void persist() override { }
     void invalidateToken() override { }
     void forgetSensitiveData() override { }
+
+private:
+    OCC::AccessManager *_am;
 };
 
 class FakeFolder
 {
-    QTemporaryDir _tempDir;
+    const QTemporaryDir _tempDir = OCC::TestUtils::createTempDir();
     DiskFileModifier _localModifier;
     // FIXME: Clarify ownership, double delete
-    FakeQNAM *_fakeQnam;
+    FakeAM *_fakeAm;
     OCC::AccountPtr _account;
     std::unique_ptr<OCC::SyncJournalDb> _journalDb;
     std::unique_ptr<OCC::SyncEngine> _syncEngine;
 
 public:
-    FakeFolder(const FileInfo &fileTemplate);
+    FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode = OCC::Vfs::Off, bool filesAreDehydrated = false);
 
     void switchToVfs(QSharedPointer<OCC::Vfs> vfs);
 
@@ -482,24 +597,24 @@ public:
     OCC::SyncJournalDb &syncJournal() const { return *_journalDb; }
 
     FileModifier &localModifier() { return _localModifier; }
-    FileInfo &remoteModifier() { return _fakeQnam->currentRemoteState(); }
+    FileInfo &remoteModifier() { return _fakeAm->currentRemoteState(); }
     FileInfo currentLocalState();
 
-    FileInfo currentRemoteState() { return _fakeQnam->currentRemoteState(); }
-    FileInfo &uploadState() { return _fakeQnam->uploadState(); }
+    FileInfo &currentRemoteState() { return _fakeAm->currentRemoteState(); }
+    FileInfo &uploadState() { return _fakeAm->uploadState(); }
     FileInfo dbState() const;
 
     struct ErrorList
     {
-        FakeQNAM *_qnam;
+        FakeAM *_qnam;
         void append(const QString &path, int error = 500)
         {
             _qnam->errorPaths().insert(path, error);
         }
         void clear() { _qnam->errorPaths().clear(); }
     };
-    ErrorList serverErrorPaths() { return { _fakeQnam }; }
-    void setServerOverride(const FakeQNAM::Override &override) { _fakeQnam->setOverride(override); }
+    ErrorList serverErrorPaths() { return { _fakeAm }; }
+    void setServerOverride(const FakeAM::Override &override) { _fakeAm->setOverride(override); }
 
     QString localPath() const;
 
@@ -523,10 +638,24 @@ public:
         return execUntilFinished();
     }
 
+    Q_REQUIRED_RESULT bool applyLocalModificationsAndSync()
+    {
+        auto mode = _syncEngine->syncOptions()._vfs->mode();
+        return _localModifier.applyModificationsAndSync(*this, mode);
+    }
+
+    Q_REQUIRED_RESULT bool applyLocalModificationsWithoutSync()
+    {
+        return _localModifier.applyModifications();
+    }
+
+    bool isDehydratedPlaceholder(const QString &filePath);
+    QSharedPointer<OCC::Vfs> vfs() const;
+
 private:
     static void toDisk(QDir &dir, const FileInfo &templateFi);
 
-    static void fromDisk(QDir &dir, FileInfo &templateFi);
+    void fromDisk(QDir &dir, FileInfo &templateFi);
 };
 
 
@@ -571,7 +700,7 @@ inline void addFiles(QStringList &dest, const FileInfo &fi)
         for (const auto &fi : fi.children)
             addFiles(dest, fi);
     } else {
-        dest += QStringLiteral("%1 - %2 %3-bytes").arg(fi.path()).arg(fi.size).arg(fi.contentChar);
+        dest += QStringLiteral("%1 - %2 %3-bytes (%4)").arg(fi.path(), QString::number(fi.contentSize), QChar::fromLatin1(fi.contentChar), fi.lastModifiedInUtc().toString());
     }
 }
 
@@ -593,11 +722,11 @@ inline void addFilesDbData(QStringList &dest, const FileInfo &fi)
 {
     // could include etag, permissions etc, but would need extra work
     if (fi.isDir) {
-        dest += QStringLiteral("%1 - %2 %3 %4").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.lastModified.toSecsSinceEpoch()), QString::fromUtf8(fi.fileId));
+        dest += QStringLiteral("%1 - %2 %3 %4").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.lastModifiedInSecondsUTC()), QString::fromUtf8(fi.fileId));
         for (const auto &fi : fi.children)
             addFilesDbData(dest, fi);
     } else {
-        dest += QStringLiteral("%1 - %2 %3 %4 %5").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.size), QString::number(fi.lastModified.toSecsSinceEpoch()), QString::fromUtf8(fi.fileId));
+        dest += QStringLiteral("%1 - %2 %3 %4 %5").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.contentSize), QString::number(fi.lastModifiedInSecondsUTC()), QString::fromUtf8(fi.fileId));
     }
 }
 
@@ -608,3 +737,53 @@ inline char *printDbData(const FileInfo &fi)
         addFilesDbData(files, fi);
     return QTest::toString(QStringLiteral("FileInfo with %1 files(%2)").arg(files.size()).arg(files.join(QStringLiteral(", "))));
 }
+
+struct OperationCounter
+{
+    int nGET = 0;
+    int nPUT = 0;
+    int nMOVE = 0;
+    int nDELETE = 0;
+
+    OperationCounter() {};
+    OperationCounter(const OperationCounter &) = delete;
+    OperationCounter(OperationCounter &&) = delete;
+    void operator=(OperationCounter const &) = delete;
+    void operator=(OperationCounter &&) = delete;
+
+    void reset()
+    {
+        nGET = 0;
+        nPUT = 0;
+        nMOVE = 0;
+        nDELETE = 0;
+    }
+
+    QNetworkReply *serverOverride(QNetworkAccessManager::Operation op, const QNetworkRequest &req, QIODevice *)
+    {
+        if (op == QNetworkAccessManager::GetOperation) {
+            ++nGET;
+        } else if (op == QNetworkAccessManager::PutOperation) {
+            ++nPUT;
+        } else if (op == QNetworkAccessManager::DeleteOperation) {
+            ++nDELETE;
+        } else if (req.attribute(QNetworkRequest::CustomVerbAttribute) == QLatin1String("MOVE")) {
+            ++nMOVE;
+        }
+        return nullptr;
+    }
+    auto functor()
+    {
+        return [this](QNetworkAccessManager::Operation op, const QNetworkRequest &req, QIODevice *device) { return serverOverride(op, req, device); };
+    }
+
+    OperationCounter(FakeFolder &fakeFolder)
+    {
+        fakeFolder.setServerOverride(functor());
+    }
+
+    friend inline QDebug operator<<(QDebug dbg, const OperationCounter &oc)
+    {
+        return dbg << "nGET:" << oc.nGET << " nPUT:" << oc.nPUT << " nMOVE:" << oc.nMOVE << " nDELETE:" << oc.nDELETE;
+    }
+};
